﻿/*
 * K.I.S.S. Konsole for High Performance Software Defined Radio
 *
 * 
 * Copyright (C) 2009 Phil Harman, VK6APH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  V1.0.0 
 */

/*
 * Change log 
 * 
 * 20 May 2009 - Added Sampling Rate control
 * 21 May 2009 - Added Noise Blanker Thresholds
 * 22 May 2009 - Added Bandscope Controls
 * 23 May 2009 - Added IQ Gain control
 * 30 Jun 2010 - Added Network Adapter selection
 * 14 Feb 2011 - Added DelayPTT control
 * 
 * 
 * 
 * TODO: Remove focus from control once used
 */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;   // use View > Output to see debug messages


namespace Moving_Forms_Test
{
    public partial class SetupForm : Form
    {
        Form1 form1;             // reference to Form1

        bool suspendUpdate = false;

        private void SafeUpDownValueChange(decimal newValue, NumericUpDown control)
        {
            if (newValue < control.Minimum)
                newValue = control.Minimum;
            if (newValue > control.Maximum)
                newValue = control.Maximum;

            control.Value = newValue;
        }

        private void SafeTrackBarValueChange(int newValue, TrackBar control)
        {
            if (newValue < control.Minimum)
                newValue = control.Minimum;
            if (newValue > control.Maximum)
                newValue = control.Maximum;

            control.Value = newValue;
        }


        public void SetIPAddress(string w3szip)  //w3sz
        {
            w3szip = textBox1.Text;  //w3sz
        }

        public bool AGCHangEnabled
        {
            set
            {
                trackBarAGCHangThreshold.Enabled = value;
            }
        }

        public SetupForm(Form1 parent) // get reference from Form1 so we know where variables are 
        {
            InitializeComponent();

            form1 = parent;

            suspendUpdate = true;

            // display Average  state, on/off
            if (Average.Checked)   // only enable Smoothing control if Average checked
                Smoothing.Enabled = true;
            else Smoothing.Enabled = false;
            // display Bandscope update rate 
            // set Waterfall AGC on/off
   //         WaterfallAGC.Checked = form1.WaterfallAGC;
            // set Mic Gain selected
     //       chkMicGain.Checked = form1.MicGain20dB;
            // set Atlas10MHz selected
//            TxFilterHigh.Value = (int) form1.TxFilterHigh;
            // set Low filter value
  //          TxFilterLow.Value = (int)form1.TxFilterLow;

            // set the gains per band
    //        Gain10m.Value  = form1.Gain10m;
      //      Gain6m.Value   = form1.Gain6m;


            // set Tune power percentage
      //      TuneLevel.Value = form1.TuneLevel;

            // display callsign
       //     textBoxCallSign.Text = form1.Your_call;
            
            // Bandscope settings
    //        GridMax.Value = form1.GridMax;
      //      GridMin.Value = form1.GridMin;
        //    GridStep.Value = form1.GridStep;

            // AGC settings
      //      numericUpDownAGCSlope.Value = Convert.ToDecimal(form1.AGCSlope);
       //     numericUpDownAGCMaxGain.Value = Convert.ToDecimal(form1.AGCMaximumGainDB);
       //     SafeTrackBarValueChange(form1.AGCHangThreshold, trackBarAGCHangThreshold);
       //     numericUpDownAGCFixedGain.Value = Convert.ToDecimal(form1.AGCFixedGainDB);

            // Waterfall settings
   //         WaterFall_High.Value = Convert.ToDecimal(form1.WaterfallHighThreshold);
     //       WaterFall_Low.Value = Convert.ToDecimal(form1.WaterfallLowThreshold);

            //Penny Open Collector settings
    //        chkExtCtrlEnable.Checked = form1.PennyOC;
            
            // set FM deviation value
            FM_deviation.Value = form1.FM_deviation;

            // set PTT Delay
            DelayPTT.Value = form1.DelayPTT;

            // END OF SUSPENDED UPDATE ZONE!
            suspendUpdate = false;

            // set Line In selected
            chkLineIn.Checked = form1.LineIn;
            chkLineIn_CheckedChanged(this, EventArgs.Empty);
         }

        // convert selected SampleRate.Text to an integer and send to the form1 & DSP 
        public void SampleRate_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            // originally 48000, 96000 or 192000.  384000 now added first for Hermes.
            // Now also for Metis and Ozy.  No way to tell if valid except to try and crash and burn!
            int sampleRate = Int32.Parse(this.SampleRate.Text);

            form1.SampleRate = sampleRate;

            form1.SampleRate = form1.SampleRate;      // change the DSP sample rate

            // set the bandwidth since dependent on sample rate
            form1.AdjustFilterByMode();
        }

        // Threshold value control is from 1 to 200, convert to float as 0.1 to 20
        private void NB1Threshold_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.BlockNBThreshold = (float)((float)NB1Threshold.Value / 10f);
        }

        // Threshold value control is from 1 to 200, convert to float as 0.1 to 20
        private void NB2Threshold_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.AveNBThreshold = (float)((float)NB2Threshold.Value / 10f);
        }

        // Turn Bandscope Averaging on/off
        private void Average_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.PowerSpectrumAveragingOn = Average.Checked;
            if (Average.Checked)            // only display Smoothing control if Average is on 
                Smoothing.Enabled = true;
            else Smoothing.Enabled = false; 
        }

        // Set Bandscope Smoothing Factor as float 0.1 to 20
        private void Smoothing_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.PowerSpectrumSmoothingFactor = (float)((float)Smoothing.Value / 100f); 
        }

        // Set Bandscope update rate 
        public void UpdateRate_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Rate = (int)UpdateRate.Value;
        }
        
   
        public void GridMax_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.GridMax = (int)this.GridMax.Value;            
        }

        public void GridMin_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.GridMin = (int)this.GridMin.Value;
        }

        public void GridStep_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.GridStep = (int)this.GridStep.Value;
        }



        private void numericUpDownAGCMaxGain_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.UpdateMaxGainAndSlider((double)numericUpDownAGCMaxGain.Value);
        }

        private void numericUpDownAGCDecayTime_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.AGCDecayTime = (float)numericUpDownAGCDecayTime.Value;
        }

        private void trackBarAGCHangThreshold_Scroll(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.AGCHangThreshold = trackBarAGCHangThreshold.Value;
        }

        private void Setupform_FormClosed(object sender, FormClosedEventArgs e)
        {
        }

        public void WaterFall_Low_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.WaterfallLowThreshold = (int)WaterFall_Low.Value;
        }

        private void WaterFall_High_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.WaterfallHighThreshold = (int)WaterFall_High.Value;
        }

        // turn on Waterfall AGC when selected
        public void WaterfallAGC_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.WaterfallAGC = WaterfallAGC.Checked;
            if (WaterfallAGC.Checked)            // only display control if AGC is off 
                WaterFall_Low.Enabled = false;
            else WaterFall_Low.Enabled = true; 
        }


        // enable Hermes if selected and all relevant controls & disable Penny 
        private void chkHermes_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }


            // Update Mic gain control
            chkMicGain_CheckedChanged(this, EventArgs.Empty);
            // Update ExtCtrlGroup
            chkExtCtrlEnable_CheckedChanged(this, EventArgs.Empty);
            // Update GainByBand
            GainByBandGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Tx filter group
            TransmitFilterGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Tune Group
            TuneGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Rx 0n PTT enable 
            chkMuteRxOnPTT_EnabledChanged(this, EventArgs.Empty);
            // Update Tx on PTT enable 
            chkOnlyTxOnPTT_EnabledChanged(this, EventArgs.Empty);
        
            // Update mic status
            chkMicGain_CheckedChanged(this, EventArgs.Empty);
            // Update GainByBand
            GainByBandGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Tx filter group
            TransmitFilterGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Tune Group
            TuneGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Rx 0n PTT enable 
            chkMuteRxOnPTT_EnabledChanged(this, EventArgs.Empty);
            // Update Tx on PTT enable 
            chkOnlyTxOnPTT_EnabledChanged(this, EventArgs.Empty);
        }

        // enable PennyLane if selected and all relevant controls, disable Penelope
        private void chkPennyLane_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            // Update mic status
            chkMicGain_CheckedChanged(this, EventArgs.Empty);
            // Update ExtCtrlGroup
            chkExtCtrlEnable_CheckedChanged(this, EventArgs.Empty);
            // Update GainByBand
            GainByBandGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Tx filter group
            TransmitFilterGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Tune Group
            TuneGroup_EnabledChanged(this, EventArgs.Empty);
            // Update Rx 0n PTT enable 
            chkMuteRxOnPTT_EnabledChanged(this, EventArgs.Empty);
            // Update Tx on PTT enable 
            chkOnlyTxOnPTT_EnabledChanged(this, EventArgs.Empty);

        }

        public void chkMicGain_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.MicGain20dB = chkMicGain.Checked;

            if (chkPenelope.Checked || chkPennyLane.Checked || chkHermes.Checked) // *** does Hermes have line input ? ***
            {
                chkMicGain.Enabled = true;
                chkLineIn.Enabled = true;
                form1.SetMicGain();  // select new mic gain setting
            }
            else
            {
                chkMicGain.Enabled = false;
                chkLineIn.Enabled = false;
            }
        }

        public void TxFilterHigh_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.TxFilterHigh = (float)TxFilterHigh.Value;
            form1.AdjustFilterByMode();  // force read of new value
        }

        private void TxFilterLow_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.TxFilterLow = (float)TxFilterLow.Value;
            form1.AdjustFilterByMode(); // force read of new value 
        }

        private void Gain160m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain160m = (int)Gain160m.Value;
            form1.UpdateBandGain();
        }

        private void Gain80m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain80m = (int)Gain80m.Value;
            form1.UpdateBandGain();
        }

        private void Gain60m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain60m = (int)Gain60m.Value;
            form1.UpdateBandGain();
        }

        private void Gain40m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain40m = (int)Gain40m.Value;
            form1.UpdateBandGain();
        }

        private void Gain30m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain30m = (int)Gain30m.Value;
            form1.UpdateBandGain();
        }

        private void Gain20m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain20m = (int)Gain20m.Value;
            form1.UpdateBandGain();
        }

        private void Gain17m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain17m = (int)Gain17m.Value;
            form1.UpdateBandGain();
        }

        private void Gain15m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain15m = (int)Gain15m.Value;
            form1.UpdateBandGain();
        }

        private void Gain12m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain12m = (int)Gain12m.Value;
            form1.UpdateBandGain();
        }

        private void Gain10m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain10m = (int)Gain10m.Value;
            form1.UpdateBandGain();
        }

        private void Gain6m_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Gain6m = (int)Gain6m.Value;
            form1.UpdateBandGain();
        }

        private void chkMuteRxOnPTT_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.Duplex = !chkMuteRxOnPTT.Checked;
            form1.setVolumeSquelched(); // force update of receiver audio mute
        }

        private void TuneLevel_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.TuneLevel = (int)TuneLevel.Value;
        }

        private void Set_CWPitch_ValueChanged(object sender, EventArgs e)
        {
            form1.CWPitch = (int)Set_CWPitch.Value;      // if user types in a number, get the new value 
            form1.AdjustFilterByMode();                  // force read of new value
        }

        // Penny Receive Open Collector settings

   
        
        private void chkOnlyTxOnPTT_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.OnlyTxOnPTT = chkOnlyTxOnPTT.Checked;
        }
        
        private void TransmitFilterGroup_EnabledChanged(object sender, EventArgs e)
        {
            if (chkPenelope.Checked || chkPennyLane.Checked || chkHermes.Checked)
                TransmitFilterGroup.Enabled = true;
            else
                TransmitFilterGroup.Enabled = false;
        }

        private void TuneGroup_EnabledChanged(object sender, EventArgs e)
        {
            if (chkPenelope.Checked || chkPennyLane.Checked || chkHermes.Checked)
                TuneGroup.Enabled = true;
            else
                TuneGroup.Enabled = false;
        }

        private void chkMuteRxOnPTT_EnabledChanged(object sender, EventArgs e)
        {
            if (chkPenelope.Checked || chkPennyLane.Checked || chkHermes.Checked)
                chkMuteRxOnPTT.Enabled = true;
            else
                chkMuteRxOnPTT.Enabled = false;
        }

        private void chkOnlyTxOnPTT_EnabledChanged(object sender, EventArgs e)
        {
            if (chkPenelope.Checked || chkPennyLane.Checked || chkHermes.Checked)
                chkOnlyTxOnPTT.Enabled = true;
            else
                chkOnlyTxOnPTT.Enabled = false;
        }

        private void FM_deviation_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            form1.FM_deviation = (int)FM_deviation.Value;
        }

        private void chkLineIn_CheckedChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            if ((chkPenelope.Checked || chkPennyLane.Checked) && chkLineIn.Checked)
                form1.LineIn = true;
            else
                form1.LineIn = false;
        }

        private void DelayPTT_ValueChanged(object sender, EventArgs e)
        {
            if (suspendUpdate)
            {
                return;
            }

            else form1.DelayPTT = (int)DelayPTT.Value;
        }
                
        private void textBox1_TextChanged(object sender, EventArgs e)
        {
            form1.textBox1.Text = textBox1.Text;
            Properties.Settings.Default.w3szendpoint = textBox1.Text;
            Properties.Settings.Default.Save();
        }
    }
}
