﻿/* 
 * This software is derived from K.I.S.S. Konsole
 * K.I.S.S. Konsole for High Performance Software Defined Radio
 *
 * Developed from original code  Copyright 2006 (C) Phil Covington, N8VB
 * 
 * Copyright (C) 2009, 2010, 2011 Phil Harman, VK6APH
 * Copyright (C) 2009 David McQuate WA8YWQ
 * Copyright (C) 2009 Joe Martin K5SO
 * Copyright (C) 2009-2013 George Byrkit K9TRV
 * Copyright (C) 2009 Mark Amos W8XR
 * Copyright (C) 2011 Erik Anderson KE7YOA
 * Copyright (C) 2011-2012 Warren Pratt NR0V (wcpAGC code)
 * 
 * 
 * Copyright (C) 2009-2017 Roger Rehr, W3SZ
 * 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * 
 */
 
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Collections.Concurrent;
using System.Diagnostics;   // use View > Output to see debug messages
using System.Globalization;
using System.Net;
using System.Net.NetworkInformation;  // so we don't have country issues with '.' and ',' in KK.csv files
using System.Timers;  //w3sz
using System.Net.Sockets; //w3sz
using System.Media; //w3sz
using System.Collections; //w3sz
using System.Linq; //w3sz
using System.Resources; //w3sz
using System.Reflection;  //w3sz
using System.Numerics;  //w3sz
using System.Runtime.InteropServices;  //w3sz
using System.Xml; //w3sz
using CSCore; //w3sz
using CSCore.CoreAudioAPI;
using CSCore.SoundIn;
using CSCore.Streams;  //w3sz
using CSCore.SoundOut;  //w3sz

using System.IO.Ports; //w3sz
using FragLabs.Audio.Codecs;  //w3sz This Is Opus.

// References removed with no apparent ill effect caused by removal include:
//Microsoft.CSharp
//System.Core
//System.Data.DataSetExtensions
//System.Xml.Linq
//Microsoft.DirectX.DirectSound

/*

Below is summary by W3SZ

The program consists of two parts:
KISS KONSOLE SERVER: Server at radio location
MOVING FORMS TEST: Client running at operator location

MOVING FORMS TEST THREADS:

Form1: 
Primary GUI Thread for Main Controller Bar
Receive Audio Data Receiver Thread for Main Radio
Receive Audio Data Receiver Thread for Aux Radio
N1MM Communications Thread for Main Radio
N1MM Communications Thread for Aux Radio
Microphone Audio Sender Thread
CW Keying Sender Thread


50/144/222/432/903/1296/Micro/HF Radio Forms:
Primary Radio GUI Thread
Spectrum Data Receiver Thread
Power/SMeter/FFTSize/MOXSet Data Receiver Thread

 
Setup forms for 50/144/222/432/903/1296/Micro/HF radios:
Primary GUI Thread


Setup form for Main Controller bar (Form1):
Primary GUI Thread


KISS KONSOLE SERVER THREADS:

Form1:
Primary GUI Thread for Radio
Microphone Audio Data Receiver Thread (Data from Client)
CW Keying Data Receiver Thread (Data from Client)
Command Data Receiver Thread (Data from Client)
Power/SMeter/FFTSize/MOXSet Data Sender Thread (Data to Client)

EthernetDevice.cs:
Primary Thread
DataLoop Thread (Communicates with Metis)

HPSDRDevice.cs:
Primary Thread

USBDevice.cs:
Primary Thread
DataLoop Thread (Communicates with Ozy)

Wisdom.cs Form:
Primary Thread
Wisdom Thread

Setupform.cs Form:
Primary GUI Thread


See Code for details regarding SharpDSP, KISSKonsole.Setup, KISSAboutBox, DeviceTypeForm, DeviceChooserForm

*/

namespace Moving_Forms_Test
{
    public partial class Form1 : Form
    {

        const int OPUS_BANDWIDTH_NARROWBAND = 1101; /**< 4 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_MEDIUMBAND = 1102; /**< 6 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_WIDEBAND = 1103; /**< 8 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_SUPERWIDEBAND = 1104; /**<12 kHz bandpass @hideinitializer*/
        const int OPUS_BANDWIDTH_FULLBAND = 1105; /**<20 kHz bandpass @hideinitializer*/

        static SerialPort port;
        static SerialPort port2;
        static SerialPort port3;
        static SerialPort port4;
        static SerialPort port5;

        //Daughter forms, one for each "band"
        public _50MHz _50;
        public _144MHz _144;
        public _222MHz _222;
        public _432MHz _432;
        public _903MHz _903;
        public _1296MHz _1296;
        public MicroMHz Micro;
        public _HF _HF;

        //Setup forms 
        SetupForm2 Setup2;  //General Setup form available from Controller Bar

        /// <summary>
        /// Variables for use with KKCSV
        /// </summary>
        public string app_data_path = "";
        public string KKCSVName = "";
        public string version = "V1.1.28";  // change this for each release!
        public SetupForm2 Setup_form2 = null;  // get reference to the Setup form.
        NumberFormatInfo nfi = NumberFormatInfo.InvariantInfo;  // so we are region independent in terms of ',' and '.' for floats
        /// <summary>
        /// end of variables for KKCSV
        /// </summary>
        /// 

        public long xvtr_frequency_50_minimum;// = 50000000;  // low band limit
        public long xvtr_frequency_50_maximum;// = 54000000;  // high band limit
        public long xvtr_frequency_144_minimum;// = 144000000;  // low band limit
        public long xvtr_frequency_144_maximum;// = 148000000;  // high band limit
        public long xvtr_frequency_222_minimum;// = 222000000;  // low band limit
        public long xvtr_frequency_222_maximum;// = 225000000;  // high band limit
        public long xvtr_frequency_432_minimum;// = 432000000;  // low band limit
        public long xvtr_frequency_432_maximum;// = 436000000;  // high band limit
        public long xvtr_frequency_903_minimum;// = 902000000;  // low band limit
        public long xvtr_frequency_903_maximum;// = 906000000;  // high band limit
        public long xvtr_frequency_1296_minimum;// = 1296000000;  // low band limit
        public long xvtr_frequency_1296_maximum;// = 1300000000;  // high band limit
        public long xvtr_frequency_2304_minimum;// = 2304000000;  // low band limit
        public long xvtr_frequency_2304_maximum;// = 2308000000;  // high band limit
        public long xvtr_frequency_3456_minimum;// = 3456000000;  // low band limit
        public long xvtr_frequency_3456_maximum;// = 3460000000;  // high band limit
        public long xvtr_frequency_5760_minimum;// = 5760000000;  // low band limit
        public long xvtr_frequency_5760_maximum;// = 5764000000;  // high band limit
        public long xvtr_frequency_10368_minimum;// = 10368000000;  // low band limit
        public long xvtr_frequency_10368_maximum;// = 10372000000;  // high band limit
        public long xvtr_frequency_24192_minimum;// = 24192000000;  // low band limit
        public long xvtr_frequency_24192_maximum;// = 24196000000;  // high band limit
        public long xvtr_frequency_HF_minimum;// = 1000000;  // low band limit
        public long xvtr_frequency_HF_maximum;// = 55000000;  // high band limit
        public static long frequency_160_minimum;// = 1800000;  // low band limit
        public static long frequency_160_maximum;// = 2000000;  // high band limit
        public static long frequency_80_minimum;// = 3500000;  // low band limit
        public static long frequency_80_maximum;// = 4000000;  // high band limit
        public static long frequency_40_minimum;// = 7000000;  // low band limit
        public static long frequency_40_maximum;// = 7300000;  // high band limit
        public static long frequency_30_minimum;// = 10100000;  // low band limit
        public static long frequency_30_maximum;// = 10100000;  // high band limit
        public static long frequency_20_minimum;// = 14000000;  // low band limit
        public static long frequency_20_maximum;// = 14350000;  // high band limit
        public static long frequency_17_minimum;// = 18068000;  // low band limit
        public static long frequency_17_maximum;// = 18168000;  // high band limit
        public static long frequency_15_minimum;// = 21000000;  // low band limit
        public static long frequency_15_maximum;// = 21450000;  // high band limit
        public static long frequency_12_minimum;// = 24880000;  // low band limit
        public static long frequency_12_maximum;// = 24980000;  // high band limit
        public static long frequency_10_minimum;// = 28000000;  // low band limit
        public static long frequency_10_maximum;// = 29700000;  // high band limit
        public static long frequency_6_minimum;// = 50000000;  // low band limit
        public static long frequency_6_maximum;// = 55000000;  // high band limit
        public static long frequency_GC_minimum;// = 00000000;  // low band limit
        public static long frequency_GC_maximum;// = 55000000;  // high band limit

        // CSCore audio setup below
        // receive Providers 1 and 2 are for standard operation; receiveProviders 3 and 4 are for Digital Modes
        public static WriteableBufferingSource receiveProvider2;  //this buffers main receive audio to prevent clicks/dropouts
        public static WriteableBufferingSource receiveProvider1;  //this buffers aux receive audio to prevent clicks/dropouts
        public static WriteableBufferingSource receiveProvider3;  //this buffers main receive audio to prevent clicks/dropouts for Digital modes
        public static WriteableBufferingSource receiveProvider4;  //this buffers aux receive audio to prevent clicks/dropouts for Digital modes
        public static WriteableBufferingSource receiveProvider5;  //this buffers main receive audio to prevent clicks/dropouts for N1MM+
        public static WriteableBufferingSource receiveProvider6;  //this buffers aux receive audio to prevent clicks/dropouts for N1MM+

        public static VolumeSource sidetone_volume;
        public static VolumeSource digimain_volume;
        public static VolumeSource digiaux_volume;
        public static VolumeSource N1MMmain_volume;
        public static VolumeSource N1MMaux_volume;
        public static float form1sidetonevolume;

        public readonly string udpbyteon = "1";
        public readonly string udpbyteoff = "0";

        //cw send
        public static readonly byte[] cwbyteon = { 1 };
        public static readonly byte[] cwbyteoff = { 0 };

        //mic main  audio sender
        public static UdpClient udpServer7 = new UdpClient(); //w3sz
        public static IPEndPoint localpt7 = new IPEndPoint(IPAddress.Parse(_50MHz.w3szendpoint1), 11000);  //w3sz

        //mic aux audio sender
        public static UdpClient udpServer18 = new UdpClient(); //w3sz
        public static IPEndPoint localpt18 = new IPEndPoint(IPAddress.Parse(_144MHz.w3szendpoint1), 11000);  //w3sz

        //cw sender
        public static UdpClient udpServer8 = new UdpClient(); //w3sz
        public static IPEndPoint localpt8 = new IPEndPoint(IPAddress.Parse(_50MHz.w3szendpoint1), 12000);  //w3sz

        public static ConcurrentQueue<byte[]> mic_data_queue;  //Main Radio Tx audio buffer
        public static ConcurrentQueue<byte[]> mic_data_queue2;  //Aux Radio Tx audio buffer

        public SerialPort DigiSerialPort;  //main serial port for WSJT-X, etc.
        public SerialPort DigiSerialPortAux;  //aux serial port for WSJT-X, etc.

        public OpusEncoder _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
        public OpusEncoder _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);

        //Initialize form     
        public Form1(string appDataDir)
        {
            //Open Daughter forms for each band
            //The form for each band will only be shown if that band is selected on Setup Form
            _50 = new _50MHz(this);
            _144 = new _144MHz(this);
            _222 = new _222MHz(this);
            _432 = new _432MHz(this);
            _903 = new _903MHz(this);
            _1296 = new _1296MHz(this);
            Micro = new MicroMHz(this);
            _HF = new _HF(this);
            
            Setup2 = new SetupForm2(this);
            
            DigiSerialPort = new SerialPort(Properties.Settings.Default.DigiCOM, 9600, Parity.None, 8, StopBits.One);
            DigiSerialPort.Open();


            DigiSerialPortAux = new SerialPort(Properties.Settings.Default.DigiCOMAux, 9600, Parity.None, 8, StopBits.One);
            DigiSerialPortAux.Open();
            //set up WASAPI devices for Tx and Rx audio

            setRxAudioOut();
            setDigiAudioOut();
            setDigiAudioOutAux();
            setN1MMAudioOut();
            setTxAudioIn();
            setTxDigiAudioIn();
            setTxDigiAudioInAux();
            setTxN1MMAudioIn();

            string bandMessage = "";
            if (Properties.Settings.Default.Checked50) { bandMessage = bandMessage + "50 "; }
            if (Properties.Settings.Default.Checked144) { bandMessage = bandMessage + "144 "; }
            if (Properties.Settings.Default.Checked222) { bandMessage = bandMessage + "222 "; }
            if (Properties.Settings.Default.Checked432) { bandMessage = bandMessage + "432 "; }
            if (Properties.Settings.Default.Checked903) { bandMessage = bandMessage + "903 "; }
            if (Properties.Settings.Default.Checked1296) { bandMessage = bandMessage + "1296 "; }
            if (Properties.Settings.Default.GHzChecked) { bandMessage = bandMessage + "Micro "; }
            if (Properties.Settings.Default.HFChecked) { bandMessage = bandMessage + "HF "; }

            DialogResult bandDialog = MessageBox.Show("Bands Selected include " + bandMessage + ".  Do You Want to Change Band Selection or other Setup Parameters?", "Setup Options", MessageBoxButtons.YesNo);
            if (bandDialog == DialogResult.Yes)
            { gotoSetup(); }

            Form_Load(this, EventArgs.Empty);

            if (Properties.Settings.Default.PositionControllerBar == null)
            {
                Globals.form1Loc = new Point(0, 1145); //was (0,1150)              
            }
            else
            {
                Globals.form1Loc = Properties.Settings.Default.PositionControllerBar.Location;
            }

            InitializeComponent();

            DigiChkBox.Checked = false;
            TxDigiChkBox.Checked = false;
            N1MMChkBox.Checked = false;
            TxN1MMChkBox.Checked = false;
            DigiChkBoxAux.Checked = false;
            TxDigiChkBoxAux.Checked = false;
            N1MMChkBoxAux.Checked = false;
            TxN1MMChkBoxAux.Checked = false;

            RcvVolume.Value = Properties.Settings.Default.RcvVolume;
            txMonVol.Value = Properties.Settings.Default.txMonVol;
            monVol = Properties.Settings.Default.txMonVol / 100.0f;

     

            openPort();
            openPort2();
            openPort3();
            openPort4();
            openPort5();

            //set sidetone on or off
            if (Properties.Settings.Default.sidetonechecked == true)
            {
                form1sidetonevolume = 1.0f;
                Globals.sidetonechecked = true;
            }
            else if (Properties.Settings.Default.sidetonechecked == false)
            {
                form1sidetonevolume = 0.0f;
                Globals.sidetonechecked = false;
            }

            this.Location = Globals.form1Loc;
            this.Refresh();

            // Load the previous radio settings from the Test_Moving_Forms.csv file and allocate values.
            // First check that the file exists
            KKCSVName = app_data_path + "MovingFormsTest.csv";
            if (!File.Exists(KKCSVName))   //if file doesn't exist, create it
            {
                CreateKKCSV(KKCSVName);
            }

            // now read the lines in the config file.  Warn if action above didn't create one!
            // This allows KK.CSV to be no longer part of the SVN tree.  This means that the KK.CSV
            // file won't get mangled/updated by SVN when someone updates the code!
            if (File.Exists(KKCSVName))   //if file exists open it
            {
                ReadKKCSV(KKCSVName);

                groupBox3.Visible = true;
                MuteEther.Visible = true;
                RcvVolume.Visible = true;


                //       _50.DisplayFrequency((int)(Micro.xvtr_frequency_50 - 1000000 * Properties.Settings.Default.LO_50 - Properties.Settings.Default.LOoffset_50));
                //       _144.DisplayFrequency((int)(Micro.xvtr_frequency_144 - 1000000 * Properties.Settings.Default.LO_144 - Properties.Settings.Default.LOoffset_144));
                //       _222.DisplayFrequency((int)(Micro.xvtr_frequency_222 - 1000000 * Properties.Settings.Default.LO_222 - Properties.Settings.Default.LOoffset_222));
                //       _432.DisplayFrequency((int)(Micro.xvtr_frequency_432 - 1000000 * Properties.Settings.Default.LO_432 - Properties.Settings.Default.LOoffset_432));
                //       _903.DisplayFrequency((int)(Micro.xvtr_frequency_903 - 1000000 * Properties.Settings.Default.LO_903 - Properties.Settings.Default.LOoffset_903));
                //       _1296.DisplayFrequency((int)(Micro.xvtr_frequency_1296 - 1000000 * Properties.Settings.Default.LO_1296 - Properties.Settings.Default.LOoffset_1296));

                _50.xvtr_frequency = Micro.xvtr_frequency_50;
                _144.xvtr_frequency = Micro.xvtr_frequency_144;
                _222.xvtr_frequency = Micro.xvtr_frequency_222;
                _432.xvtr_frequency = Micro.xvtr_frequency_432;
                _903.xvtr_frequency = Micro.xvtr_frequency_903;
                _1296.xvtr_frequency = Micro.xvtr_frequency_1296;

                _50.DisplayFrequency(_50.set_frequency.Value);
                _144.DisplayFrequency(_144.set_frequency.Value);
                _222.DisplayFrequency(_222.set_frequency.Value);
                _432.DisplayFrequency(_432.set_frequency.Value);
                _903.DisplayFrequency(_903.set_frequency.Value);
                _1296.DisplayFrequency(_1296.set_frequency.Value);

                //  Console.Writeline("Transverter Frequencies are:");
                //  Console.Writeline(_50.xvtr_frequency);
                //  Console.Writeline(_144.xvtr_frequency);
                //  Console.Writeline(_222.xvtr_frequency);
                //  Console.Writeline(_432.xvtr_frequency);
                //  Console.Writeline(_903.xvtr_frequency);
                //  Console.Writeline(_1296.xvtr_frequency);

                switch (_HF.BandText)
                {
                    case "160m":
                        _HF.CurrentBand = _HF.OpBand.M160;
                        _HF.set_frequency.Minimum = 1800000;  // low band limit
                        _HF.set_frequency.Maximum = 2000000;  // high band limit
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_160;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "USB")
                        {
                            _HF.ModeText = "LSB";                  // select the appropriate sideband
                            //                          _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_160;
                        _HF.BandGain = _HF.Gain160m; // set the band gain 
                        break;

                    case "80m":
                        _HF.CurrentBand = _HF.OpBand.M80;
                        _HF.set_frequency.Minimum = 3500000;
                        _HF.set_frequency.Maximum = 4000000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_80;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "USB")
                        {
                            _HF.ModeText = "LSB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_80;
                        _HF.BandGain = _HF.Gain80m;
                        break;

                    case "40m":
                        _HF.CurrentBand = _HF.OpBand.M40;
                        _HF.set_frequency.Minimum = 7000000;
                        _HF.set_frequency.Maximum = 7300000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_40;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "USB")
                        {
                            _HF.ModeText = "LSB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_40;
                        _HF.BandGain = _HF.Gain40m;
                        break;

                    case "30m":
                        _HF.CurrentBand = _HF.OpBand.M30;
                        _HF.set_frequency.Minimum = 10100000;
                        _HF.set_frequency.Maximum = 10150000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_30;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_30;
                        _HF.BandGain = _HF.Gain30m;
                        break;

                    case "20m":
                        _HF.CurrentBand = _HF.OpBand.M20;
                        _HF.set_frequency.Minimum = 14000000;
                        _HF.set_frequency.Maximum = 14350000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_20;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_20;
                        _HF.BandGain = _HF.Gain20m;
                        break;

                    case "17m":
                        _HF.CurrentBand = _HF.OpBand.M17;
                        _HF.set_frequency.Minimum = 18068000;
                        _HF.set_frequency.Maximum = 18168000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_17;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_17;
                        _HF.BandGain = _HF.Gain17m;
                        break;

                    case "15m":
                        _HF.CurrentBand = _HF.OpBand.M15;
                        _HF.set_frequency.Minimum = 21000000;
                        _HF.set_frequency.Maximum = 21450000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_15;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_15;
                        _HF.BandGain = _HF.Gain15m;
                        break;

                    case "12m":
                        _HF.CurrentBand = _HF.OpBand.M12;
                        _HF.set_frequency.Minimum = 24880000;
                        _HF.set_frequency.Maximum = 24980000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_12;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_12;
                        _HF.BandGain = _HF.Gain12m;
                        break;

                    case "10m":
                        _HF.CurrentBand = _HF.OpBand.M10;
                        _HF.set_frequency.Minimum = 28000000;
                        _HF.set_frequency.Maximum = 29700000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_10;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_10;
                        _HF.BandGain = _HF.Gain10m;
                        break;

                    case "6m":
                        _HF.CurrentBand = _HF.OpBand.M6;
                        _HF.set_frequency.Minimum = 50000000;
                        _HF.set_frequency.Maximum = 55000000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_6;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                        }
                        if (_HF.ModeText == "LSB")
                        {
                            _HF.ModeText = "USB";                  // select the appropriate sideband
                            _HF.ChangeMode();  // reset the Bandwidth slider
                        }
                        _HF.Preamp.Checked = _HF.Preamp_6;
                        _HF.BandGain = _HF.Gain6m;
                        break;

                    case "GC":
                        _HF.CurrentBand = _HF.OpBand.GC;
                        _HF.set_frequency.Minimum = 0;
                        _HF.set_frequency.Maximum = 55000000;
                        try
                        {
                            _HF.set_frequency.Value = _HF.set_frequency_GC;
                        }
                        catch
                        {
                            _HF.set_frequency.Value = 10000000;
                        }
                        _HF.Preamp.Checked = _HF.Preamp_GC;
                        _HF.BandGain = 0;
                        break;

                    default:

                        break;


                }
                _HF.BandSelect.Text = _HF.BandText;    // set  band to start on 
                _HF.Frequency_change();                 // check we are in band, update frequency and display              
                _HF.ChangeMode();                   // Set Bandwidth slider value based on Mode in us
            }
            else
            {
                MessageBox.Show("Can't find MovingFormsTest.csv, should be in the same directory as Moving_Forms_Test.exe!", "File Error");
            }

            //set band forms to initial sizes and locations
            _50.Size = Globals.formsmallsize;
            _144.Size = Globals.formsmallsize;
            _50.Location = Globals.form50smallloc;
            _144.Location = Globals.form144smallloc;
            _222.Size = Globals.formsmallsize;
            _432.Size = Globals.formsmallsize;
            _222.Location = Globals.form222smallloc;
            _432.Location = Globals.form432smallloc;
            _903.Size = Globals.formsmallsize;
            _1296.Size = Globals.formsmallsize;
            _903.Location = Globals.form903smallloc;
            _1296.Location = Globals.form1296smallloc;
            Micro.Size = Globals.formMicrosmallsize;
            Micro.Location = Globals.formMicrosmallloc;
            _HF.Size = Globals.formsmallsize;
            _HF.Location = Globals.formHFsmallloc;

            //initially no bands are selected
            mainHFBut.Checked = false;
            main50But.Checked = false;
            main144But.Checked = false;
            main222But.Checked = false;
            main432But.Checked = false;
            main903But.Checked = false;
            main1296But.Checked = false;
            main2304But.Checked = false;
            main3456But.Checked = false;
            main5760But.Checked = false;
            main10GBut.Checked = false;
            main24GBut.Checked = false;

            auxHFBut.Checked = false;
            aux50But.Checked = false;
            aux144But.Checked = false;
            aux222But.Checked = false;
            aux432But.Checked = false;
            aux903But.Checked = false;
            aux1296But.Checked = false;
            aux2304But.Checked = false;
            aux3456But.Checked = false;
            aux5760But.Checked = false;
            aux10GBut.Checked = false;
            aux24GBut.Checked = false;


            //band-change event handlers
            mainHFBut.CheckedChanged += new EventHandler(mainHFBut_CheckedChanged);
            main50But.CheckedChanged += new EventHandler(main50But_CheckedChanged);
            main144But.CheckedChanged += new EventHandler(main144But_CheckedChanged);
            main222But.CheckedChanged += new EventHandler(main222But_CheckedChanged);
            main432But.CheckedChanged += new EventHandler(main432But_CheckedChanged);
            main903But.CheckedChanged += new EventHandler(main903But_CheckedChanged);
            main1296But.CheckedChanged += new EventHandler(main1296But_CheckedChanged);
            main2304But.CheckedChanged += new EventHandler(main2304But_CheckedChanged);
            main3456But.CheckedChanged += new EventHandler(main3456But_CheckedChanged);
            main5760But.CheckedChanged += new EventHandler(main5760But_CheckedChanged);
            main10GBut.CheckedChanged += new EventHandler(main10GBut_CheckedChanged);
            main24GBut.CheckedChanged += new EventHandler(main24GBut_CheckedChanged);

            auxHFBut.CheckedChanged += new EventHandler(auxHFBut_CheckedChanged);
            aux50But.CheckedChanged += new EventHandler(aux50But_CheckedChanged);
            aux144But.CheckedChanged += new EventHandler(aux144But_CheckedChanged);
            aux222But.CheckedChanged += new EventHandler(aux222But_CheckedChanged);
            aux432But.CheckedChanged += new EventHandler(aux432But_CheckedChanged);
            aux903But.CheckedChanged += new EventHandler(aux903But_CheckedChanged);
            aux1296But.CheckedChanged += new EventHandler(aux1296But_CheckedChanged);
            aux2304But.CheckedChanged += new EventHandler(aux2304But_CheckedChanged);
            aux3456But.CheckedChanged += new EventHandler(aux3456But_CheckedChanged);
            aux5760But.CheckedChanged += new EventHandler(aux5760But_CheckedChanged);
            aux10GBut.CheckedChanged += new EventHandler(aux10GBut_CheckedChanged);
            aux24GBut.CheckedChanged += new EventHandler(aux24GBut_CheckedChanged);
            
            Properties.Settings.Default.PropertyChanged += Default_PropertyChanged;

            //event handler for closing program and this form
            this.FormClosing += new FormClosingEventHandler(Form_Closing);

            selectBands();

            Setup2.N1MMCOMcomboBox_Clicked(this, EventArgs.Empty);

            micMainChk.Checked = true;
            KeyMainBut.Checked = true;

            digiCATBox.Checked = Globals.digiCAT;
            digiCATBoxAux.Checked = Globals.digiCATAux;

            if(DigiChkBox.Checked)
            {
                MRBW.Text = bw2str(Properties.Settings.Default.dataBandwidth);
                MRBR.Text = Properties.Settings.Default.dataBitrate.ToString();
                if (Properties.Settings.Default.dataCodecIsVoip)
                {
                    MRType.Text = "Voip";
                }
                else
                {
                    MRType.Text = "Audio";
                }
            }
            else
            {
                MRBW.Text = bw2str(Properties.Settings.Default.voiceBandwidth);
                MRBR.Text = Properties.Settings.Default.voiceBitrate.ToString();
                if (Properties.Settings.Default.voiceCodecIsVoip)
                {
                    MRType.Text = "Voip";
                }
                else
                {
                    MRType.Text = "Audio";
                }
            }


            if (DigiChkBoxAux.Checked)
            {
                ARBW.Text = bw2str(Properties.Settings.Default.dataBandwidth);
                ARBR.Text = Properties.Settings.Default.dataBitrate.ToString();
                if (Properties.Settings.Default.dataCodecIsVoip)
                {
                    ARType.Text = "Voip";
                }
                else
                {
                    ARType.Text = "Audio";
                }
            }
            else
            {
                ARBW.Text = bw2str(Properties.Settings.Default.voiceBandwidth);
                ARBR.Text = Properties.Settings.Default.voiceBitrate.ToString();
                if (Properties.Settings.Default.voiceCodecIsVoip)
                {
                    ARType.Text = "Voip";
                }
                else
                {
                    ARType.Text = "Audio";
                }
            }


            if (TxDigiChkBox.Checked)
            {
                MTBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
                MTBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                if (Properties.Settings.Default.txdataCodecIsVoip)
                {
                    MTType.Text = "Voip";
                }
                else
                {
                    MTType.Text = "Audio";
                }

            }
            else
            {
                MTBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);
                MTBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                if (Properties.Settings.Default.txvoiceCodecIsVoip)
                {
                    MTType.Text = "Voip";
                }
                else
                {
                    MTType.Text = "Audio";
                }
            }


            if (TxDigiChkBoxAux.Checked)
            {
                ATBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
                ATBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                if (Properties.Settings.Default.txdataCodecIsVoip)
                {
                    ATType.Text = "Voip";
                }
                else
                {
                    ATType.Text = "Audio";
                }
            }
            else
            {
                ATBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);
                ATBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                if (Properties.Settings.Default.txvoiceCodecIsVoip)
                {
                    ATType.Text = "Voip";
                }
                else
                {
                    ATType.Text = "Audio";
                }
            }

            /////;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

            // start getting main receiver audio data from main client socket and place in CSCore buffer "receiveProvider2"
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                UdpClient udpServer6 = new UdpClient(10000);
                IPEndPoint localpt6 = new IPEndPoint(IPAddress.Any, 10000);
                _decoder = OpusDecoder.Create(48000, 1);
                //  Console.Writeline("MainReceiver thread working");
                while (Globals.stopper == false) //w3sz
                {
                    try
                    {
                        byte[] buffer1 = udpServer6.Receive(ref localpt6); //w3sz
                        int len2;
                        len2 = buffer1.Length;
                        if (buffer1.Length > 0)
                        {
                            if (Properties.Settings.Default.opusComp)
                            {
                                byte[] buffer2 = _decoder.Decode(buffer1, buffer1.Length, out len2);
                                Form1.receiveProvider2.Write(buffer2, 0, len2);
                                Form1.receiveProvider4.Write(buffer2, 0, len2);
                                Form1.receiveProvider6.Write(buffer2, 0, len2);
                                //               //  Console.Writeline("Buffer2.Length = {0}, len2 = {1}", buffer1.Length,len2);
                            }
                            else
                            {
                                Form1.receiveProvider2.Write(buffer1, 0, len2);
                                Form1.receiveProvider4.Write(buffer1, 0, len2);
                                Form1.receiveProvider6.Write(buffer1, 0, len2);
                            }
                        }
                    }
                        catch
                        {
                            //                    //  Console.Writeline("Exception occurred in main receiver audio thread");
                        }
                    
                }
            });

            // start getting aux receiver audio data from aux client socket and place in CSCore buffer "receiveProvider1"
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                UdpClient udpServer9 = new UdpClient(13000);
                IPEndPoint localpt9 = new IPEndPoint(IPAddress.Any, 13000);  //aux receiver audio data from server

                _decoder2 = OpusDecoder.Create(48000, 1);
                while (Globals.stopper == false) //w3sz
                {
                    try
                    {
                        byte[] buffer3 = udpServer9.Receive(ref localpt9); //w3sz
                        int len2;
                        len2 = buffer3.Length;
                        if (buffer3.Length > 0)
                        {
                            if (Properties.Settings.Default.opusComp)
                            {
                                byte[] buffer4 = _decoder2.Decode(buffer3, buffer3.Length, out len2);

                                Form1.receiveProvider1.Write(buffer4, 0, len2);
                                Form1.receiveProvider3.Write(buffer4, 0, len2);
                                Form1.receiveProvider5.Write(buffer4, 0, len2);
                            }
                            else
                            {
                                Form1.receiveProvider1.Write(buffer3, 0, len2);
                                Form1.receiveProvider3.Write(buffer3, 0, len2);
                                Form1.receiveProvider5.Write(buffer3, 0, len2);
                            }
                        }
                    }
                    catch
                    {
                        //                      //  Console.Writeline("Exception occurred in aux receiver audio thread");
                    }
                    }
            });


            mainN1MMCAT();
            auxN1MMCAT();

            //--------------------------------------------------------------------------
            //Start Tx and Receive Audio streams
            MicAudioStart();
            StartMon();
            StartMonAux();
            StartAudio();

            // start thread for sending Main mic audio to server ;;;;;;;;;;;;;;;;;;;
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                //      MessageBox.Show("starting mic thread");
                while (Globals.stopper == false)
                {
                    try
                    {
                        while (mic_data_queue.TryDequeue(out final_mic_bytes))
                        {
                            udpServer7.Send(final_mic_bytes, final_mic_bytes.Length, localpt7); //w3sz 
                        }
                    }
                    catch
                    {
                        //                      //  Console.Writeline("Error in Mic Thread");
                    }
                }
            });

            // start thread for sending Aux mic audio to server ;;;;;;;;;;;;;;;;;;;
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                //      MessageBox.Show("starting Aux mic thread");
                while (Globals.stopper == false)
                {
                    try
                    {
                        while (mic_data_queue2.TryDequeue(out final_mic_bytes2))
                        {
                            udpServer18.Send(final_mic_bytes2, final_mic_bytes2.Length, localpt18); //w3sz 
                        }
                    }
                    catch
                    {
                        //                      //  Console.Writeline("Error in Aux Mic Thread");
                    }
                }
            });


            // start thread for sending CW Data to server ;;;;;;;;;;;;;;;;;;;
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                //      MessageBox.Show("starting CW thread");
                while (Globals.stopper == false)
                {
                    Thread.Sleep(20);
                    try
                    {
                        if (Globals.sendCW == true)
                        {
                            if (Globals.CW_dot || Globals.CW_dash)
                            {
                                udpServer8.Send(cwbyteon, 1, localpt8); //w3sz 
                                sidetone_volume.Volume = form1sidetonevolume;
                            }
                            else if ((!Globals.CW_dot) & (!Globals.CW_dash))
                            {
                                udpServer8.Send(cwbyteoff, 1, localpt8); //w3sz 
                                sidetone_volume.Volume = 0.0f;
                            }
                        }
                    }
                    catch
                    {
                        //                      //  Console.Writeline("Error in CW Thread");
                    }
                }
            });


        }
/*
        public class DigiCalcDelegate
        {
            public delegate StringBuilder[] getNewParamsDelegate(StringBuilder[] a);
            public getNewParamsDelegate getNewParams = null;
            public delegate StringBuilder getFAorFBDelegate(string FF, string freqstring);
            public delegate void setMDDelegate(string MD);
            public delegate void setTXorRXDelegate(string TR);
            public delegate void testDelegate(string RR);
            public testDelegate testdel = null;

        }
        */

        string bw2str(int s)
        {
            if (s==OPUS_BANDWIDTH_NARROWBAND)
            {
                return "4 kHz";
            }
            else if (s == OPUS_BANDWIDTH_MEDIUMBAND)
            {
                return "6 kHz";
            }
            else if (s == OPUS_BANDWIDTH_WIDEBAND)
            {
                return "8 kHz";
            }
            else if (s == OPUS_BANDWIDTH_SUPERWIDEBAND)
            {
                return "12 kHz";
            }
            else if (s == OPUS_BANDWIDTH_FULLBAND)
            {
                return "20 kHz";
            }
            return "errorin bandwidth";
        }
        
        private void Default_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "txvoiceBandwidth":
                    {
                        if (!TxDigiChkBox.Checked)
                        {
                            _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                            _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            MTBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);
                            MTBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                        }
                        if (!TxDigiChkBoxAux.Checked)
                        {
                            _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                            _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            ATBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);
                            ATBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                        }
                        break;
                    }
                case "txvoiceBitrate":
                    {
                        if (!TxDigiChkBox.Checked)
                        {
                            _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                            _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            MTBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);
                            MTBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                        }
                        if (!TxDigiChkBoxAux.Checked)
                        {
                            _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                            _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            ATBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);
                            ATBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                        }

                        break;
                    }
                case "txvoiceCodecIsVoip":
                    {
                        if (!TxDigiChkBox.Checked)
                        {
                            if (Properties.Settings.Default.txvoiceCodecIsVoip)
                            {
                                MTType.Text = "Voip";
                                if (_encoder != null)
                                {
                                    _encoder.Dispose();
                                }
                                _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                                _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                                _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            }
                            else
                            {
                                MTType.Text = "Audio";
                                if (_encoder != null)
                                {
                                    _encoder.Dispose();
                                }
                                _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                                _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                                _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            }
                        }
                        if (!TxDigiChkBoxAux.Checked)
                        {
                            if (Properties.Settings.Default.txvoiceCodecIsVoip == true)
                            {
                                ATType.Text = "Voip";
                                if (_encoder2 != null)
                                {
                                    _encoder2.Dispose();
                                }
                                _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                                _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                                _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            }
                            else
                            {
                                ATType.Text = "Audio";
                                if (_encoder2 != null)
                                {
                                    _encoder2.Dispose();
                                }
                                _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                                _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                                _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                            }
                        }

                        break;
                    }
                case "txdataBandwidth":
                    {
                        if (TxDigiChkBox.Checked)
                        {
                            _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                            _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            MTBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
                            MTBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                        }
                        else if (TxDigiChkBoxAux.Checked)
                        {
                            _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                            _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            ATBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
                            ATBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                        }
                        break;
                    }
                case "txdataBitrate":
                    {
                        if (TxDigiChkBox.Checked)
                        {
                            _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                            _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            MTBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
                            MTBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                        }
                        else if (TxDigiChkBoxAux.Checked)
                        {
                            _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                            _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            ATBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
                            ATBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                        }
                        break;
                    }
                case "txdataCodecIsVoip":
                    {
                        if (TxDigiChkBox.Checked)
                        {
                            if (Properties.Settings.Default.txdataCodecIsVoip)
                            {
                                MTType.Text = "Voip";
                                if (_encoder != null)
                                {
                                    _encoder.Dispose();
                                }
                                _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                                _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                                _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            }
                            else
                            {
                                MTType.Text = "Audio";
                                if (_encoder != null)
                                {
                                    _encoder.Dispose();
                                }
                                _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                                _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                                _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            }
                        }
                        else if (TxDigiChkBoxAux.Checked)
                        {
                            if (Properties.Settings.Default.txdataCodecIsVoip)
                            {
                                ATType.Text = "Voip";
                                if (_encoder2 != null)
                                {
                                    _encoder2.Dispose();
                                }
                                _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                                _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                                _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            }
                            else
                            {
                                ATType.Text = "Audio";
                                if (_encoder2 != null)
                                {
                                    _encoder2.Dispose();
                                }
                                _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                                _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                                _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                            }
                        }
                        break;
                    }
                case "voiceBandwidth":
                    {
                        if (!DigiChkBox.Checked)
                        {
                            MRBW.Text = bw2str(Properties.Settings.Default.voiceBandwidth);
                        }
                        if (!DigiChkBoxAux.Checked)
                        {
                            ARBW.Text = bw2str(Properties.Settings.Default.voiceBandwidth);
                        }
                        break;
                    }
                case "voiceBitrate":
                    {
                        if (!DigiChkBox.Checked)
                        {
                            MRBR.Text = Properties.Settings.Default.voiceBitrate.ToString();
                        }
                        if (!DigiChkBoxAux.Checked)
                        {
                            ARBR.Text = Properties.Settings.Default.voiceBitrate.ToString();
                        }
                        break;
                    }
                case "voiceCodecIsVoip":
                    {
                        if (!DigiChkBox.Checked)
                        {
                            if (Properties.Settings.Default.voiceCodecIsVoip)
                            {
                                MRType.Text = "Voip";
                            }
                            else
                            {
                                MRType.Text = "Audio";
                            }
                        }
                        if (!DigiChkBoxAux.Checked)
                        {
                            if (Properties.Settings.Default.voiceCodecIsVoip)
                            {
                                ARType.Text = "Voip";
                            }
                            else
                            {
                                ARType.Text = "Audio";
                            }
                        }                    
                    break;
                    }   
                case "dataBandwidth":
                    {
                        if (DigiChkBox.Checked)
                        {
                            MRBW.Text = bw2str(Properties.Settings.Default.dataBandwidth);
                        }
                        if (DigiChkBoxAux.Checked)
                        {
                            ARBW.Text = bw2str(Properties.Settings.Default.dataBandwidth);
                        }
                        break;
                    }
                case "dataBitrate":
                    {
                        if (DigiChkBox.Checked)
                        {
                            MRBR.Text = Properties.Settings.Default.dataBitrate.ToString();
                        }
                        if (DigiChkBoxAux.Checked)
                        {
                            ARBR.Text = Properties.Settings.Default.dataBitrate.ToString();
                        }
                        break;
                    }
                case "dataCodecIsVoip":
                    {
                        if (DigiChkBox.Checked)
                        {
                            if (Properties.Settings.Default.dataCodecIsVoip)
                            {
                                MRType.Text = "Voip";
                            }
                            else
                            {
                                MRType.Text = "Audio";
                            }
                        }
                        if (DigiChkBoxAux.Checked)
                        {
                            if (Properties.Settings.Default.dataCodecIsVoip)
                            {
                                ARType.Text = "Voip";
                            }
                            else
                            {
                                ARType.Text = "Audio";
                            }
                        }
                        break;
                    }
            }
        }


        //-----------------------------------------------------

        public StringBuilder[] newfsb = new StringBuilder[4];

        public void mainN1MMCAT()
        {
            try
            {
                SerialPort N1MMSerialPort;
                N1MMSerialPort = new SerialPort(Properties.Settings.Default.N1MMCOM, 9600, Parity.None, 8, StopBits.One);
                N1MMSerialPort.Open();

                // Thread that communicates with N1MM for MAIN radio 
                ThreadPool.QueueUserWorkItem(delegate //w3sz
                {

                    char n1mmin;
                    StringBuilder n1mmstrb = new StringBuilder();
                    string n1mmstr;
                    StringBuilder newfsb = new StringBuilder();
                    int hpsdrmode;
                    string newf = "00007150000";
                    string mdmode = "2";
                    long n1mmfreq;
                    try
                    {
                        while (Globals.stopper == false) //w3sz
                        {
                            n1mmin = (char)N1MMSerialPort.ReadChar();
                            n1mmstrb.Append(n1mmin);
                            n1mmstr = n1mmstrb.ToString();
                            if (n1mmstr.Contains(";"))
                            {
                                //              if (n1mmstr.Contains("FT"))
                                //              {
                                //                  //  Console.Writeline(n1mmstr);
                                //              }
                                if (Globals.main50 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("000").Append(Convert.ToString(_50.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        hpsdrmode = _50.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }

                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.main144 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("00").Append(Convert.ToString(_144.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        hpsdrmode = _144.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.main222 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("00").Append(Convert.ToString(_222.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        hpsdrmode = _222.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.main432 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("00").Append(Convert.ToString(_432.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        hpsdrmode = _432.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.main903 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("00").Append(Convert.ToString(_903.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                        newfsb.Clear();
                                        hpsdrmode = _903.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                    });
                                }
                                else if (Globals.main1296 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("0").Append(Convert.ToString(_1296.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        hpsdrmode = _1296.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });

                                }
                                else if (Globals.mainMicro == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("0").Append(Convert.ToString(Micro.xvtr_frequency));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                        if (newf.Length > 11)
                                        {
                                            newf = newf.Substring(1);
                                            //                                Console.WriteLine("New length of frequency is " + newf.Length.ToString() + "New Frequency is " + newf);
                                        }
                                        hpsdrmode = Micro.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                    });

                                }
                                else if (Globals.mainHF == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsb = newfsb.Append("0000").Append(Convert.ToString(_HF.set_frequency.Value));
                                        newf = newfsb.ToString();
                                        newfsb.Clear();
                                        //     Console.WriteLine("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                        if (newf.Length > 11)
                                        {
                                            newf = newf.Substring(1);
                                            //     Console.WriteLine("New length of frequency is " + newf.Length.ToString() + "New Frequency is " + newf);
                                        }
                                        hpsdrmode = _HF.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                    });

                                }

                                if (n1mmstr.Contains("AG0;"))
                                {
                                    N1MMSerialPort.Write("AG0000;");
                                }
                                else if (n1mmstr.Contains("IF;"))
                                {
                                    N1MMSerialPort.Write("IF" + newf + "0011+00000000000" + mdmode + "0000000;");
                                }
                                else if (n1mmstr.Contains("FA;"))
                                {
                                    N1MMSerialPort.Write("FA" + newf + ";");
                                }
                                else if (n1mmstr.Contains("FB;"))
                                {
                                    N1MMSerialPort.Write("FB" + newf + ";");
                                }
                                else if (n1mmstr.Contains("FR0;"))
                                {
                                    N1MMSerialPort.Write("FR0;");
                                }
                                else if (n1mmstr.Contains("FR1;"))
                                {
                                    N1MMSerialPort.Write("FR1;");
                                }
                                //MD is for mode
                                // MD1 LSB  OpenHPSDR 4
                                // MD2 USB            3
                                // MD3 CWU            5
                                // MD4 FMN            2
                                // MD5 AM             0
                                // MD 6 RTTY <not used>
                                // MD7 CWL            6
                                else if (n1mmstr.Contains("MD1;"))
                                {
                                    if (Globals.main50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.main144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 4;
                                        });
                                    }
                                    else if (Globals.main222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.main432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 4;
                                        });
                                    }
                                    else if (Globals.main903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 4;
                                        });
                                    }
                                    else if (Globals.main1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.mainMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.mainHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 4;
                                        });
                                    }
                                } //end of MD1 if
                                else if (n1mmstr.Contains("MD2;"))
                                {
                                    if (Globals.main50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 3;
                                        });

                                    }
                                    else if (Globals.main144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.main222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 3;
                                        });

                                    }
                                    else if (Globals.main432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.main903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.main1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.mainMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.mainHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 3;
                                        });
                                    }

                                } //end of MD2 if
                                else if (n1mmstr.Contains("MD3;"))
                                {
                                    if (Globals.main50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 5;
                                        });

                                    }
                                    else if (Globals.main144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.main222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 5;
                                        });

                                    }
                                    else if (Globals.main432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.main903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.main1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.mainMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.mainHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 5;
                                        });
                                    }

                                } //end of MD3 if
                                else if (n1mmstr.Contains("MD4;"))
                                {
                                    if (Globals.main50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 2;
                                        });

                                    }
                                    else if (Globals.main144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.main222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 2;
                                        });

                                    }
                                    else if (Globals.main432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.main903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.main1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.mainMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.mainHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 2;
                                        });

                                    }
                                } //end of MD4 if
                                else if (n1mmstr.Contains("MD5;"))
                                {
                                    if (Globals.main50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 0;
                                        });

                                    }
                                    else if (Globals.main144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.main222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 0;
                                        });

                                    }
                                    else if (Globals.main432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.main903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.main1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.mainMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.mainHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 0;
                                        });

                                    }
                                } //end of MD5 if
                                else if (n1mmstr.Contains("MD7;"))
                                {
                                    if (Globals.main50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 6;
                                        });

                                    }
                                    else if (Globals.main144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.main222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 6;
                                        });

                                    }
                                    else if (Globals.main432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.main903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.main1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.mainMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.mainHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 6;
                                        });

                                    }
                                } //end of MD7 if

                                else if ((n1mmstr.Contains("FA")) & (n1mmstr.Length == 14))
                                {
                                    //  Console.Writeline("Frequency change");
                                    n1mmfreq = Convert.ToInt64(n1mmstr.Substring(2, n1mmstr.Length - 3));
                                    //  Console.Writeline("Frequency is " + n1mmfreq.ToString());

                                    if ((n1mmfreq >= 50000000) & (n1mmfreq <= 54000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main50But.Checked = true;
                                            _50.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_50 - MicroMHz.LOoffset_50);
                                            _50.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                        });
                                    }
                                    else if ((n1mmfreq >= 144000000) & (n1mmfreq <= 148000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main144But.Checked = true;
                                            _144.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_144 - MicroMHz.LOoffset_144);
                                            _144.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _144.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 222000000) & (n1mmfreq <= 226000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main222But.Checked = true;
                                            _222.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_222 - MicroMHz.LOoffset_222);
                                            _222.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _222.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 432000000) & (n1mmfreq <= 436000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main432But.Checked = true;
                                            _432.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_432 - MicroMHz.LOoffset_432);
                                            _432.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _432.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 902000000) & (n1mmfreq <= 906000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main903But.Checked = true;
                                            _903.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_903 - MicroMHz.LOoffset_903);
                                            _903.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _903.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 1296000000) & (n1mmfreq <= 1300000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main1296But.Checked = true;
                                            _1296.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_1296 - MicroMHz.LOoffset_1296);
                                            _1296.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 2304000000) & (n1mmfreq <= 2308000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main2304But.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_2304 - MicroMHz.LOoffset_2304);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 3456000000) & (n1mmfreq <= 3460000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main3456But.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_3456 - MicroMHz.LOoffset_3456);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 5760000000) & (n1mmfreq <= 5764000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main5760But.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_5760 - MicroMHz.LOoffset_5760);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 10368000000) & (n1mmfreq <= 10372000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main10GBut.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_10368 - MicroMHz.LOoffset_10368);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 24192000000) & (n1mmfreq <= 24196000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            main24GBut.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_24192 - MicroMHz.LOoffset_24192);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 1800000) & (n1mmfreq <= 2000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 0;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 3500000) & (n1mmfreq <= 4000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 1;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 7000000) & (n1mmfreq <= 7300000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 2;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 10000000) & (n1mmfreq <= 10150000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 3;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 14000000) & (n1mmfreq <= 14350000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 4;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 18068000) & (n1mmfreq <= 18168000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 5;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 21000000) & (n1mmfreq <= 21450000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 6;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 24890000) & (n1mmfreq <= 24990000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 7;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 28000000) & (n1mmfreq <= 30000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 8;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if (n1mmfreq <= 50000000)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            mainHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 10;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }


                                }
                                n1mmstrb.Clear();
                            }
                        }
                    }

                    catch
                    {
                        //  Console.Writeline("Exception occurred in N1MM Main Radio thread");
                    }
                });
            }
            catch { }
        }


        public StringBuilder[] newfsbN1MM = new StringBuilder[4];

        public void auxN1MMCAT()
        {
            try
            {
                SerialPort N1MMSerialPort;
                N1MMSerialPort = new SerialPort(Properties.Settings.Default.N1MMCOMLiaison, 9600, Parity.None, 8, StopBits.One);
                N1MMSerialPort.Open();

                // Thread that communicates with N1MM for LIAISON [AUX] radio 
                ThreadPool.QueueUserWorkItem(delegate //w3sz
                {
                    char n1mmin;
                    StringBuilder n1mmstrb = new StringBuilder();
                    string n1mmstr;
                    StringBuilder newfsbN1MM = new StringBuilder();
                    int hpsdrmode;
                    string newf = "00007150000";
                    string mdmode = "2";
                    long n1mmfreq;

                    try
                    {

                        while (Globals.stopper == false) //w3sz
                        {
                            n1mmin = (char)N1MMSerialPort.ReadChar();
                            n1mmstrb.Append(n1mmin);
                            n1mmstr = n1mmstrb.ToString();

                            if (n1mmstr.Contains(";"))
                            {
                                if (Globals.aux50 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("000").Append(Convert.ToString(_50.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        hpsdrmode = _50.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }

                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.aux144 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("00").Append(Convert.ToString(_144.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        hpsdrmode = _144.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.aux222 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("00").Append(Convert.ToString(_222.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        hpsdrmode = _222.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.aux432 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("00").Append(Convert.ToString(_432.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        hpsdrmode = _432.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });
                                }
                                else if (Globals.aux903 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("00").Append(Convert.ToString(_903.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                        newfsbN1MM.Clear();
                                        hpsdrmode = _903.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                    });
                                }
                                else if (Globals.aux1296 == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("0").Append(Convert.ToString(_1296.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        hpsdrmode = _1296.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                    });

                                }
                                else if (Globals.auxMicro == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("0").Append(Convert.ToString(Micro.xvtr_frequency));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                        if (newf.Length > 11)
                                        {
                                            newf = newf.Substring(1);
                                            //     //  Console.Writeline("New length of frequency is " + newf.Length.ToString() + "New Frequency is " + newf);
                                        }
                                        hpsdrmode = Micro.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                    });

                                }
                                else if (Globals.auxHF == true)
                                {
                                    Invoke((MethodInvoker)delegate
                                    {
                                        newfsbN1MM = newfsbN1MM.Append("0000").Append(Convert.ToString(_HF.set_frequency.Value));
                                        newf = newfsbN1MM.ToString();
                                        newfsbN1MM.Clear();
                                        //     //  Console.Writeline("Length of frequency is " + newf.Length.ToString() + "Frequency is " + newf);
                                        if (newf.Length > 11)
                                        {
                                            newf = newf.Substring(1);
                                            //     //  Console.Writeline("New length of frequency is " + newf.Length.ToString() + "New Frequency is " + newf);
                                        }
                                        hpsdrmode = _HF.Mode.SelectedIndex;
                                        switch (hpsdrmode)
                                        {
                                            case 0:
                                                mdmode = "5";
                                                break;
                                            case 1:
                                                mdmode = "5";
                                                break;
                                            case 2:
                                                mdmode = "4";
                                                break;
                                            case 3:
                                                mdmode = "2";
                                                break;
                                            case 4:
                                                mdmode = "1";
                                                break;
                                            case 5:
                                                mdmode = "3";
                                                break;
                                            case 6:
                                                mdmode = "7";
                                                break;
                                        }
                                    });

                                }

                                if (n1mmstr.Contains("AG0;"))
                                {
                                    N1MMSerialPort.Write("AG0000;");
                                }
                                else if (n1mmstr.Contains("IF;"))
                                {
                                    N1MMSerialPort.Write("IF" + newf + "0011+00000000000" + mdmode + "0000000;");
                                }
                                else if (n1mmstr.Contains("FA;"))
                                {
                                    N1MMSerialPort.Write("FA" + newf + ";");
                                }
                                else if (n1mmstr.Contains("FB;"))
                                {
                                    N1MMSerialPort.Write("FB" + newf + ";");
                                }
                                else if (n1mmstr.Contains("FR0;"))
                                {
                                    N1MMSerialPort.Write("FR0;");
                                }
                                else if (n1mmstr.Contains("FR1;"))
                                {
                                    N1MMSerialPort.Write("FR1;");
                                }
                                //MD is for mode
                                // MD1 LSB  OpenHPSDR 4
                                // MD2 USB            3
                                // MD3 CWU            5
                                // MD4 FMN            2
                                // MD5 AM             0
                                // MD 6 RTTY <not used>
                                // MD7 CWL            6
                                else if (n1mmstr.Contains("MD1;"))
                                {
                                    if (Globals.aux50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.aux144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 4;
                                        });
                                    }
                                    else if (Globals.aux222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.aux432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 4;
                                        });
                                    }
                                    else if (Globals.aux903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 4;
                                        });
                                    }
                                    else if (Globals.aux1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.auxMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 4;
                                        });

                                    }
                                    else if (Globals.auxHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 4;
                                        });

                                    }
                                } //end of MD1 if

                                else if (n1mmstr.Contains("MD2;"))
                                {
                                    if (Globals.aux50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 3;
                                        });

                                    }
                                    else if (Globals.aux144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.aux222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 3;
                                        });

                                    }
                                    else if (Globals.aux432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.aux903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.aux1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.auxMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 3;
                                        });
                                    }
                                    else if (Globals.auxHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 3;
                                        });
                                    }
                                } //end of MD2 if
                                else if (n1mmstr.Contains("MD3;"))
                                {
                                    if (Globals.aux50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 5;
                                        });

                                    }
                                    else if (Globals.aux144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.aux222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 5;
                                        });

                                    }
                                    else if (Globals.aux432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.aux903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.aux1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.auxMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 5;
                                        });
                                    }
                                    else if (Globals.auxHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 5;
                                        });

                                    }
                                } //end of MD3 if
                                else if (n1mmstr.Contains("MD4;"))
                                {
                                    if (Globals.aux50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 2;
                                        });

                                    }
                                    else if (Globals.aux144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.aux222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 2;
                                        });

                                    }
                                    else if (Globals.aux432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.aux903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.aux1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.auxMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 2;
                                        });
                                    }
                                    else if (Globals.auxHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 2;
                                        });

                                    }
                                } //end of MD4 if
                                else if (n1mmstr.Contains("MD5;"))
                                {
                                    if (Globals.aux50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 0;
                                        });

                                    }
                                    else if (Globals.aux144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.aux222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 0;
                                        });

                                    }
                                    else if (Globals.aux432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.aux903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.aux1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.auxMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 0;
                                        });
                                    }
                                    else if (Globals.auxHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 0;
                                        });

                                    }
                                } //end of MD5 if
                                else if (n1mmstr.Contains("MD7;"))
                                {
                                    if (Globals.aux50 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _50.Mode.SelectedIndex = 6;
                                        });

                                    }
                                    else if (Globals.aux144 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _144.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.aux222 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _222.Mode.SelectedIndex = 6;
                                        });

                                    }
                                    else if (Globals.aux432 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _432.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.aux903 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _903.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.aux1296 == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _1296.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.auxMicro == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            Micro.Mode.SelectedIndex = 6;
                                        });
                                    }
                                    else if (Globals.auxHF == true)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            _HF.Mode.SelectedIndex = 6;
                                        });

                                    }
                                } //end of MD7 if

                                else if ((n1mmstr.Contains("FA")) & (n1mmstr.Length == 14))
                                {
                                    //  Console.Writeline("Frequency change");
                                    n1mmfreq = Convert.ToInt64(n1mmstr.Substring(2, n1mmstr.Length - 3));
                                    //  Console.Writeline("Frequency is " + n1mmfreq.ToString());

                                    if ((n1mmfreq >= 50000000) & (n1mmfreq <= 54000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux50But.Checked = true;
                                            _50.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_50 - MicroMHz.LOoffset_50);
                                            _50.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                        });
                                    }
                                    else if ((n1mmfreq >= 144000000) & (n1mmfreq <= 148000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux144But.Checked = true;
                                            _144.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_144 - MicroMHz.LOoffset_144);
                                            _144.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _144.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 222000000) & (n1mmfreq <= 226000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux222But.Checked = true;
                                            _222.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_222 - MicroMHz.LOoffset_222);
                                            _222.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _222.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 432000000) & (n1mmfreq <= 436000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux432But.Checked = true;
                                            _432.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_432 - MicroMHz.LOoffset_432);
                                            _432.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _432.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 902000000) & (n1mmfreq <= 906000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux903But.Checked = true;
                                            _903.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_903 - MicroMHz.LOoffset_903);
                                            _903.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _903.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 1296000000) & (n1mmfreq <= 1300000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux1296But.Checked = true;
                                            _1296.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_1296 - MicroMHz.LOoffset_1296);
                                            _1296.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 2304000000) & (n1mmfreq <= 2308000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux2304But.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_2304 - MicroMHz.LOoffset_2304);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 3456000000) & (n1mmfreq <= 3460000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux3456But.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_3456 - MicroMHz.LOoffset_3456);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 5760000000) & (n1mmfreq <= 5764000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux5760But.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_5760 - MicroMHz.LOoffset_5760);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 10368000000) & (n1mmfreq <= 10372000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux10GBut.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_10368 - MicroMHz.LOoffset_10368);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("N1MM Freq is " + n1mmfreq + " IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 24192000000) & (n1mmfreq <= 24196000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            aux24GBut.Checked = true;
                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_24192 - MicroMHz.LOoffset_24192);
                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 1800000) & (n1mmfreq <= 2000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 0;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 3500000) & (n1mmfreq <= 4000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 1;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 7000000) & (n1mmfreq <= 7300000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 2;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 10000000) & (n1mmfreq <= 10150000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 3;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 14000000) & (n1mmfreq <= 14350000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 4;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 18068000) & (n1mmfreq <= 18168000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 5;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 21000000) & (n1mmfreq <= 21450000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 6;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 24890000) & (n1mmfreq <= 24990000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 7;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if ((n1mmfreq >= 28000000) & (n1mmfreq <= 30000000))
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 8;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }
                                    else if (n1mmfreq <= 50000000)
                                    {
                                        Invoke((MethodInvoker)delegate
                                        {
                                            auxHFBut.Checked = true;
                                            _HF.BandSelect.SelectedIndex = 10;
                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                        });

                                    }



                                }
                                n1mmstrb.Clear();
                            }
                        }
                    }
                    catch
                    {
                        //  Console.Writeline("Exception occurred in N1MM Liaison Radio thread");
                    }

                });
            }
            catch { }
        }

        //add Digi Serial Ports for WSJTX for both main and aux radios

        public StringBuilder[] newfsbMain = new StringBuilder[4];
        public StringBuilder[] newfsbAux = new StringBuilder[4];

        public void DigiCATAux()
        {

            if  (Globals.auxradio != null)
            {
                //  Console.Writeline("Globals.auxradio is {0}", Globals.auxradio);
                try
                {
                    DigiSerialPortAux.DiscardInBuffer();
                    DigiSerialPortAux.DiscardOutBuffer();
                    // Thread that communicates with N1MM for LIAISON [AUX] radio 
                    ThreadPool.QueueUserWorkItem(delegate //w3sz
                    {
                        char n1mmin;
                        StringBuilder n1mmstrb = new StringBuilder("");
                        string n1mmstr;
                        StringBuilder[] newfsbAux = new StringBuilder[4];
                        for (int i = 0; i < 4; i++)
                        {
                            newfsbAux[i] = new StringBuilder("");
                        }

                        string newf = "00050150000";
                        string newfB = "00050150000";  //for VFO B
                        string mdmode = "2";
                        string split = "0";  // 0 if split off, 1 if split on
                        long n1mmfreq;
                        bool xmit = false;

                        try
                        {
                            while (Globals.digiCATAux && !Globals.stopper) //w3sz
                            {
                                if (DigiSerialPortAux.IsOpen)
                                {
                                    digiCATBoxAux.BackColor = Color.Green;
                                }
                                else
                                {
                                    digiCATBoxAux.BackColor = Color.Red;
                                }

                                n1mmin = (char)DigiSerialPortAux.ReadChar();
                                n1mmstrb.Append(n1mmin);
                                n1mmstr = n1mmstrb.ToString();

                                if (n1mmstr.Length >= 3 && n1mmstr.Contains(";"))
                                {
                                   

                                    if (newfsbAux != null)
                                    {
                                        newfsbAux = DigiCATCalcsAux(DigiChkBoxAux.Checked, Globals.auxradio, newfsbAux);
                                    }
                                    if (newfsbAux != null)
                                    {

                                        newf = newfsbAux[0].ToString();
                                        newfB = newfsbAux[1].ToString();
                                        mdmode = newfsbAux[2].ToString();
                                        if (newfsbAux[3].ToString().Contains("False"))
                                        { split = "0"; }
                                        else if (newfsbAux[3].ToString().Contains("True")) { split = "1"; }
                                        //       Console.WriteLine("newf is {0}, newfB is {1}, mdmode is {2}, split is {3}, newfsbAux[3] is {4} ", newf, newfB, mdmode, split, newfsb[3].ToString());
                                        //       Console.WriteLine("Properties.settings.default.50split is {0}", Properties.Settings.Default.split50);
                                        newfsbAux[0].Clear();
                                        newfsbAux[1].Clear();
                                        newfsbAux[2].Clear();
                                        newfsbAux[3].Clear();
                                        if (n1mmstr.Contains("AG0;"))
                                        {
                                            DigiSerialPortAux.Write("AG0000;");
                                        }
                                        else if (n1mmstr.Contains("IF;"))
                                        {
                                            if (!xmit)
                                            {
                                                DigiSerialPortAux.Write("IF" + newf + "0011+00000000000" + mdmode + "00" + int.Parse(split) + "0000;");
                                                //              Console.WriteLine("newf line 3266 is {0}", newf);
                                            }
                                            else
                                            {
                                                if (split == "1")
                                                {
                                                    DigiSerialPortAux.Write("IF" + newf + "0011+00000000001" + mdmode + "1010000;"); // was mdmode + "1010000;"); 1-10-2017
                                                }
                                                else
                                                {
                                                    DigiSerialPortAux.Write("IF" + newfB + "0011+00000000001" + mdmode + "0000000;");
                                                }

                                            }
                                            //        Console.WriteLine("XMIT is {0}", xmit);
                                        }
                                        else if (n1mmstr.Contains("FA;"))
                                        {
                                            DigiSerialPortAux.Write("FA" + newf + ";");
                                        }
                                        else if (n1mmstr.Contains("FB;"))
                                        {
                                            DigiSerialPortAux.Write("FB" + newfB + ";");
                                        }

                                        else if (n1mmstr.Contains("AI"))
                                        {
                                            DigiSerialPortAux.Write("AI0;");
                                        }
                                        else if (n1mmstr.Contains("ID"))
                                        {
                                            DigiSerialPortAux.Write("ID019;");
                                        }
                                        else if (n1mmstr.Contains("MD;"))
                                        {
                                            DigiSerialPortAux.Write("MD" + mdmode + ";");
                                        }
                                    }

                                    if (n1mmstr != null)
                                    {
                                            if (n1mmstr.Contains("MD1;"))
                                            {
                                                if (Globals.aux50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.aux144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                                else if (Globals.aux222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.aux432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                                else if (Globals.aux903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                                else if (Globals.aux1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.auxMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.auxHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                            } //end of MD1 if

                                            else if (n1mmstr.Contains("MD2;"))
                                            {
                                                if (Globals.aux50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 3;
                                                        });

                                                }
                                                else if (Globals.aux144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.aux222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 3;
                                                        });

                                                }
                                                else if (Globals.aux432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.aux903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.aux1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.auxMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.auxHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                            } //end of MD2 if
                                            else if (n1mmstr.Contains("MD3;"))
                                            {
                                                if (Globals.aux50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 5;
                                                        });

                                                }
                                                else if (Globals.aux144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.aux222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 5;
                                                        });

                                                }
                                                else if (Globals.aux432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.aux903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.aux1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.auxMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.auxHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 5;
                                                        });

                                                }
                                            } //end of MD3 if
                                            else if (n1mmstr.Contains("MD4;"))
                                            {
                                                if (Globals.aux50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 2;
                                                        });

                                                }
                                                else if (Globals.aux144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.aux222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 2;
                                                        });

                                                }
                                                else if (Globals.aux432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.aux903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.aux1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.auxMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.auxHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 2;
                                                        });

                                                }
                                            } //end of MD4 if
                                            else if (n1mmstr.Contains("MD5;"))
                                            {
                                                if (Globals.aux50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 0;
                                                        });

                                                }
                                                else if (Globals.aux144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.aux222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 0;
                                                        });

                                                }
                                                else if (Globals.aux432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.aux903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.aux1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.auxMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.auxHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 0;
                                                        });

                                                }
                                            } //end of MD5 if
                                            else if (n1mmstr.Contains("MD7;"))
                                            {
                                                if (Globals.aux50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 6;
                                                        });

                                                }
                                                else if (Globals.aux144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.aux222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 6;
                                                        });

                                                }
                                                else if (Globals.aux432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.aux903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.aux1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.auxMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.auxHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 6;
                                                        });

                                                }
                                            } //end of MD7 if

                                            else if ((n1mmstr.Contains("FA")) & (n1mmstr.Length == 14))
                                            {
                                                //  Console.Writeline("Frequency change");
                                                n1mmfreq = Convert.ToInt64(n1mmstr.Substring(2, n1mmstr.Length - 3));
                                                //  Console.Writeline("Frequency is " + n1mmfreq.ToString());

                                                if ((n1mmfreq >= 50000000) & (n1mmfreq <= 54000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux50But.Checked = true;
                                                            _50.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_50 - MicroMHz.LOoffset_50);
                                                            _50.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                                        });
                                                }
                                                else if ((n1mmfreq >= 144000000) & (n1mmfreq <= 148000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux144But.Checked = true;
                                                            _144.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_144 - MicroMHz.LOoffset_144);
                                                            _144.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _144.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 222000000) & (n1mmfreq <= 226000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux222But.Checked = true;
                                                            _222.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_222 - MicroMHz.LOoffset_222);
                                                            _222.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _222.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 432000000) & (n1mmfreq <= 436000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux432But.Checked = true;
                                                            _432.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_432 - MicroMHz.LOoffset_432);
                                                            _432.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _432.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 902000000) & (n1mmfreq <= 906000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux903But.Checked = true;
                                                            _903.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_903 - MicroMHz.LOoffset_903);
                                                            _903.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _903.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 1296000000) & (n1mmfreq <= 1300000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux1296But.Checked = true;
                                                            _1296.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_1296 - MicroMHz.LOoffset_1296);
                                                            _1296.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 2304000000) & (n1mmfreq <= 2308000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux2304But.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_2304 - MicroMHz.LOoffset_2304);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 3456000000) & (n1mmfreq <= 3460000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux3456But.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_3456 - MicroMHz.LOoffset_3456);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 5760000000) & (n1mmfreq <= 5764000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux5760But.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_5760 - MicroMHz.LOoffset_5760);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 10368000000) & (n1mmfreq <= 10372000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux10GBut.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_10368 - MicroMHz.LOoffset_10368);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("N1MM Freq is " + n1mmfreq + " IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 24192000000) & (n1mmfreq <= 24196000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux24GBut.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_24192 - MicroMHz.LOoffset_24192);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 1800000) & (n1mmfreq <= 2000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 0;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 3500000) & (n1mmfreq <= 4000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 1;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 7000000) & (n1mmfreq <= 7300000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 2;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 10000000) & (n1mmfreq <= 10150000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 3;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 14000000) & (n1mmfreq <= 14350000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 4;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 18068000) & (n1mmfreq <= 18168000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 5;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 21000000) & (n1mmfreq <= 21450000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 6;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 24890000) & (n1mmfreq <= 24990000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 7;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 28000000) & (n1mmfreq <= 30000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 8;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if (n1mmfreq <= 50000000)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 10;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }

                                            }


                                            else if ((n1mmstr.Contains("FB")) & (n1mmstr.Length == 14))
                                            {
                                                //  Console.Writeline("Frequency change");
                                                n1mmfreq = Convert.ToInt64(n1mmstr.Substring(2, n1mmstr.Length - 3));
                                                //  Console.Writeline("Frequency is " + n1mmfreq.ToString());

                                                if ((n1mmfreq >= 50000000) & (n1mmfreq <= 54000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux50But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freq50 = n1mmfreq - MicroMHz.LO_50 - MicroMHz.LOoffset_50;
                                                            _50.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq50);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _50.display_freqB);
                                                        });
                                                }
                                                else if ((n1mmfreq >= 144000000) & (n1mmfreq <= 148000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux144But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freq144 = n1mmfreq - MicroMHz.LO_144 - MicroMHz.LOoffset_144;
                                                            _144.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq144);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _144.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 222000000) & (n1mmfreq <= 226000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux222But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freq222 = n1mmfreq - MicroMHz.LO_222 - MicroMHz.LOoffset_222;
                                                            _222.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq222);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _222.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 432000000) & (n1mmfreq <= 436000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux432But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freq432 = n1mmfreq - MicroMHz.LO_432 - MicroMHz.LOoffset_432;
                                                            _432.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq432);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _432.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 902000000) & (n1mmfreq <= 906000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux903But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freq903 = n1mmfreq - MicroMHz.LO_903 - MicroMHz.LOoffset_903;
                                                            _903.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq903);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _903.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 1296000000) & (n1mmfreq <= 1300000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux1296But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freq1296 = n1mmfreq - MicroMHz.LO_1296 - MicroMHz.LOoffset_1296;
                                                            _1296.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq1296);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _1296.display_freqB); ;
                                                        });

                                                }
                                                else if ((n1mmfreq >= 2304000000) & (n1mmfreq <= 2308000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux2304But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_2304 - MicroMHz.LOoffset_2304;
                                                            Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                            //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 3456000000) & (n1mmfreq <= 3460000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux3456But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_3456 - MicroMHz.LOoffset_3456;
                                                            Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                            //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 5760000000) & (n1mmfreq <= 5764000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux5760But.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_5760 - MicroMHz.LOoffset_5760;
                                                            Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                            //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 10368000000) & (n1mmfreq <= 10372000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux10GBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_10368 - MicroMHz.LOoffset_10368;
                                                            Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                            //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 24192000000) & (n1mmfreq <= 24196000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            aux24GBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_24192 - MicroMHz.LOoffset_24192;
                                                            Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                            //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 1800000) & (n1mmfreq <= 2000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 3500000) & (n1mmfreq <= 4000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 7000000) & (n1mmfreq <= 7300000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 10000000) & (n1mmfreq <= 10150000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 14000000) & (n1mmfreq <= 14350000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 18068000) & (n1mmfreq <= 18168000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 21000000) & (n1mmfreq <= 21450000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 24890000) & (n1mmfreq <= 24990000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if ((n1mmfreq >= 28000000) & (n1mmfreq <= 30000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }
                                                else if (n1mmfreq <= 50000000)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            auxHFBut.Checked = true;
                                                            Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                            _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                            //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                        });

                                                }

                                            }

                                            else if (n1mmstr.Contains("TX"))
                                            {
                                                // put radio into transmit
                                                xmit = true;
                                                SwitchBand(Globals.auxradio);
                                                moxoff(tempendpoint, temprcvdmcport, false);
                                                //  Console.Writeline("Transmit is {0}", xmit);
                                            }
                                            else if (n1mmstr.Contains("RX"))
                                            {
                                                xmit = false;
                                                SwitchBand(Globals.auxradio);
                                                moxoff(tempendpoint, temprcvdmcport, true);
                                                //  Console.Writeline("Receive is {0}", !xmit);
                                            }

                                            n1mmstrb.Clear();

                                    } //end of if digichkboxchecked
                                } //end of if n1mmstr != null                   


                            //  Console.Writeline("LEFT DIGICAT WHILE LOOP");
                            if (DigiSerialPortAux.IsOpen)
                            {
                                //  Console.Writeline("DigiSerialPortAux IS OPEN, LEFT WHILELOOP");
                            }
                            else
                            {
                                //  Console.Writeline("DigiSerialPortAux is Closed, still within thread pool but outside while");
                            }
                        }
                        }
                        catch
                {
                    //  Console.Writeline("Exception occurred in Digi Liaison Radio thread");
                    if (DigiSerialPortAux.IsOpen)
                    {
                        //  Console.Writeline("DigiSerialPortAux IS OPEN");
                    }
                    else
                    {
                        digiCATBoxAux.BackColor = Color.Red;
                    }

                }

            });

            if (!Globals.digiCATAux)
            {
                if (DigiSerialPortAux.IsOpen)
                {
                    //  Console.Writeline("DigiSerialPortAux IS OPEN and Globals.digiCAT is false");
                }
                else
                {
                    //  Console.Writeline("DigiSerialPortAux is Closed and Globals.digiCAT is false");
                }
            }
        }
                catch
                {
                    //  Console.Writeline("Entire DigiCAT Try failed");
                }
            }
    //  Console.Writeline("Leaving DigiCAT method");
}

//end DigiCAT Method

        public void DigiCAT()
{

    if (Globals.mainradio != null)
    {
        //  Console.Writeline("Globals.auxradio is {0}", Globals.auxradio);
        try
        {
            DigiSerialPort.DiscardInBuffer();
            DigiSerialPort.DiscardOutBuffer();
            // Thread that communicates with N1MM for LIAISON [AUX] radio 
            ThreadPool.QueueUserWorkItem(delegate //w3sz
            {
                char n1mmin;
                StringBuilder n1mmstrb = new StringBuilder("");
                string n1mmstr;
                StringBuilder[] newfsbMain = new StringBuilder[4];
                for (int i = 0; i < 4; i++)
                {
                    newfsbMain[i] = new StringBuilder("");
                }

                string newf = "00050150000";
                string newfB = "00050150000";  //for VFO B
                string mdmode = "2";
                string split = "0";  // 0 if split off, 1 if split on
                long n1mmfreq;
                bool xmit = false;

                try
                {
                    while (Globals.digiCAT && !Globals.stopper) //w3sz
                    {
                        if (DigiSerialPort.IsOpen)
                        {
                            digiCATBox.BackColor = Color.Green;
                        }
                        else
                        {
                            digiCATBox.BackColor = Color.Red;
                        }

                        n1mmin = (char)DigiSerialPort.ReadChar();
                        n1mmstrb.Append(n1mmin);
                        n1mmstr = n1mmstrb.ToString();

                        if (n1mmstr.Length >= 3 && n1mmstr.Contains(";"))
                        {
                            //  Console.Writeline("n1mmstr is {0}", n1mmstr);
                            //               Invoke((MethodInvoker)delegate
                            //         {

                            if (newfsbMain != null)
                            {
                                newfsbMain = DigiCATCalcs(DigiChkBox.Checked, Globals.mainradio,  newfsbMain);
                            }
                            if (newfsbMain != null)
                            {
                                newf = newfsbMain[0].ToString();
                                newfB = newfsbMain[1].ToString();
                                mdmode = newfsbMain[2].ToString();
                                if (newfsbMain[3].ToString().Contains("False"))
                                { split = "0"; }
                                else if (newfsbMain[3].ToString().Contains("True")) { split = "1"; }
                                //       Console.WriteLine("newf is {0}, newfB is {1}, mdmode is {2}, split is {3}, newfsbMain[3] is {4} ", newf, newfB, mdmode, split, newfsb[3].ToString());
                                //       Console.WriteLine("Properties.settings.default.50split is {0}", Properties.Settings.Default.split50);
                                newfsbMain[0].Clear();
                                newfsbMain[1].Clear();
                                newfsbMain[2].Clear();
                                newfsbMain[3].Clear();



                        //        Invoke((MethodInvoker)delegate
                        //        {
                        //            Console.WriteLine("n1mmstr is {0}", n1mmstr);
                        //        });



                                if (n1mmstr.Contains("AG0;"))
                                {
                                    DigiSerialPort.Write("AG0000;");
                                }
                                else if (n1mmstr.Contains("IF;"))
                                {
                                    if (!xmit)
                                    {
                                        DigiSerialPort.Write("IF" + newf + "0011+00000000000" + mdmode + "00" + int.Parse(split) + "0000;");
                                        //              Console.WriteLine("newf line 3266 is {0}", newf);
                                    }
                                    else
                                    {
                                        if (split == "1")
                                        {
                                            DigiSerialPort.Write("IF" + newf + "0011+00000000001" + mdmode + "1010000;"); // was mdmode + "1010000;"); 1-10-2017
                                        }
                                        else
                                        {
                                            DigiSerialPort.Write("IF" + newfB + "0011+00000000001" + mdmode + "0000000;");
                                        }

                                    }
                                    //        Console.WriteLine("XMIT is {0}", xmit);
                                }
                                else if (n1mmstr.Contains("FA;"))
                                {
                                    DigiSerialPort.Write("FA" + newf + ";");
                                }
                                else if (n1mmstr.Contains("FB;"))
                                {
                                    DigiSerialPort.Write("FB" + newfB + ";");
                                }

                                else if (n1mmstr.Contains("AI"))
                                {
                                    DigiSerialPort.Write("AI0;");
                                }
                                else if (n1mmstr.Contains("ID"))
                                {
                                    DigiSerialPort.Write("ID019;");
                                }
                                else if (n1mmstr.Contains("MD;"))
                                {
                                    DigiSerialPort.Write("MD" + mdmode + ";");
                                }
                            }

                            if (n1mmstr != null)
                            {
                                            if (n1mmstr.Contains("MD1;"))
                                            {
                                                if (Globals.main50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.main144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                                else if (Globals.main222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.main432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                                else if (Globals.main903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                                else if (Globals.main1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.mainMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 4;
                                                        });

                                                }
                                                else if (Globals.mainHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 4;
                                                        });
                                                }
                                            } //end of MD1 if
                                            else if (n1mmstr.Contains("MD2;"))
                                            {
                                                if (Globals.main50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 3;
                                                        });

                                                }
                                                else if (Globals.main144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.main222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 3;
                                                        });

                                                }
                                                else if (Globals.main432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.main903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.main1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.mainMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 3;
                                                        });
                                                }
                                                else if (Globals.mainHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 3;
                                                        });
                                                }

                                            } //end of MD2 if
                                            else if (n1mmstr.Contains("MD3;"))
                                            {
                                                if (Globals.main50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 5;
                                                        });

                                                }
                                                else if (Globals.main144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.main222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 5;
                                                        });

                                                }
                                                else if (Globals.main432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.main903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.main1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.mainMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 5;
                                                        });
                                                }
                                                else if (Globals.mainHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 5;
                                                        });
                                                }

                                            } //end of MD3 if
                                            else if (n1mmstr.Contains("MD4;"))
                                            {
                                                if (Globals.main50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 2;
                                                        });

                                                }
                                                else if (Globals.main144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.main222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 2;
                                                        });

                                                }
                                                else if (Globals.main432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.main903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.main1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.mainMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 2;
                                                        });
                                                }
                                                else if (Globals.mainHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 2;
                                                        });

                                                }
                                            } //end of MD4 if
                                            else if (n1mmstr.Contains("MD5;"))
                                            {
                                                if (Globals.main50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 0;
                                                        });

                                                }
                                                else if (Globals.main144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.main222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 0;
                                                        });

                                                }
                                                else if (Globals.main432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.main903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.main1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.mainMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 0;
                                                        });
                                                }
                                                else if (Globals.mainHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 0;
                                                        });

                                                }
                                            } //end of MD5 if
                                            else if (n1mmstr.Contains("MD7;"))
                                            {
                                                if (Globals.main50 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _50.Mode.SelectedIndex = 6;
                                                        });

                                                }
                                                else if (Globals.main144 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _144.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.main222 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _222.Mode.SelectedIndex = 6;
                                                        });

                                                }
                                                else if (Globals.main432 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _432.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.main903 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _903.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.main1296 == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _1296.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.mainMicro == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            Micro.Mode.SelectedIndex = 6;
                                                        });
                                                }
                                                else if (Globals.mainHF == true)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            _HF.Mode.SelectedIndex = 6;
                                                        });

                                                }
                                            } //end of MD7 if

                                            else if ((n1mmstr.Contains("FA")) & (n1mmstr.Length == 14))
                                            {
                                                //  Console.Writeline("Frequency change");
                                                n1mmfreq = Convert.ToInt64(n1mmstr.Substring(2, n1mmstr.Length - 3));
                                                //  Console.Writeline("Frequency is " + n1mmfreq.ToString());

                                                if ((n1mmfreq >= 50000000) & (n1mmfreq <= 54000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main50But.Checked = true;
                                                            _50.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_50 - MicroMHz.LOoffset_50);
                                                            _50.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                                        });
                                                }
                                                else if ((n1mmfreq >= 144000000) & (n1mmfreq <= 148000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main144But.Checked = true;
                                                            _144.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_144 - MicroMHz.LOoffset_144);
                                                            _144.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _144.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 222000000) & (n1mmfreq <= 226000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main222But.Checked = true;
                                                            _222.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_222 - MicroMHz.LOoffset_222);
                                                            _222.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _222.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 432000000) & (n1mmfreq <= 436000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main432But.Checked = true;
                                                            _432.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_432 - MicroMHz.LOoffset_432);
                                                            _432.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _432.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 902000000) & (n1mmfreq <= 906000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main903But.Checked = true;
                                                            _903.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_903 - MicroMHz.LOoffset_903);
                                                            _903.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _903.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 1296000000) & (n1mmfreq <= 1300000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main1296But.Checked = true;
                                                            _1296.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_1296 - MicroMHz.LOoffset_1296);
                                                            _1296.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + _50.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 2304000000) & (n1mmfreq <= 2308000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main2304But.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_2304 - MicroMHz.LOoffset_2304);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 3456000000) & (n1mmfreq <= 3460000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main3456But.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_3456 - MicroMHz.LOoffset_3456);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 5760000000) & (n1mmfreq <= 5764000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main5760But.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_5760 - MicroMHz.LOoffset_5760);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 10368000000) & (n1mmfreq <= 10372000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main10GBut.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_10368 - MicroMHz.LOoffset_10368);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 24192000000) & (n1mmfreq <= 24196000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            main24GBut.Checked = true;
                                                            Micro.set_frequency.Value = (int)(n1mmfreq - MicroMHz.LO_24192 - MicroMHz.LOoffset_24192);
                                                            Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("IF Radio Frequency is " + Micro.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 1800000) & (n1mmfreq <= 2000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 0;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 3500000) & (n1mmfreq <= 4000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 1;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 7000000) & (n1mmfreq <= 7300000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 2;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 10000000) & (n1mmfreq <= 10150000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 3;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 14000000) & (n1mmfreq <= 14350000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 4;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 18068000) & (n1mmfreq <= 18168000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 5;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 21000000) & (n1mmfreq <= 21450000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 6;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 24890000) & (n1mmfreq <= 24990000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 7;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if ((n1mmfreq >= 28000000) & (n1mmfreq <= 30000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 8;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }
                                                else if (n1mmfreq <= 50000000)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                        {
                                                            mainHFBut.Checked = true;
                                                            _HF.BandSelect.SelectedIndex = 10;
                                                            _HF.set_frequency.Value = (int)(n1mmfreq);
                                                            _HF.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                                                            //  Console.Writeline("Radio Frequency is " + _HF.set_frequency.Value.ToString());
                                                        });

                                                }


                                            }


                                            else if ((n1mmstr.Contains("FB")) & (n1mmstr.Length == 14))
                                            {
                                                //  Console.Writeline("Frequency change");
                                                n1mmfreq = Convert.ToInt64(n1mmstr.Substring(2, n1mmstr.Length - 3));
                                                //  Console.Writeline("Frequency is " + n1mmfreq.ToString());

                                                if ((n1mmfreq >= 50000000) & (n1mmfreq <= 54000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main50But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freq50 = n1mmfreq - MicroMHz.LO_50 - MicroMHz.LOoffset_50;
                                                        _50.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq50);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _50.display_freqB);
                                                    });
                                                }
                                                else if ((n1mmfreq >= 144000000) & (n1mmfreq <= 148000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main144But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freq144 = n1mmfreq - MicroMHz.LO_144 - MicroMHz.LOoffset_144;
                                                        _144.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq144);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _144.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 222000000) & (n1mmfreq <= 226000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main222But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freq222 = n1mmfreq - MicroMHz.LO_222 - MicroMHz.LOoffset_222;
                                                        _222.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq222);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _222.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 432000000) & (n1mmfreq <= 436000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main432But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freq432 = n1mmfreq - MicroMHz.LO_432 - MicroMHz.LOoffset_432;
                                                        _432.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq432);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _432.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 902000000) & (n1mmfreq <= 906000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main903But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freq903 = n1mmfreq - MicroMHz.LO_903 - MicroMHz.LOoffset_903;
                                                        _903.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq903);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _903.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 1296000000) & (n1mmfreq <= 1300000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main1296But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freq1296 = n1mmfreq - MicroMHz.LO_1296 - MicroMHz.LOoffset_1296;
                                                        _1296.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq1296);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _1296.display_freqB); ;
                                                    });

                                                }
                                                else if ((n1mmfreq >= 2304000000) & (n1mmfreq <= 2308000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main2304But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_2304 - MicroMHz.LOoffset_2304;
                                                        Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                        //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 3456000000) & (n1mmfreq <= 3460000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main3456But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_3456 - MicroMHz.LOoffset_3456;
                                                        Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                        //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 5760000000) & (n1mmfreq <= 5764000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main5760But.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_5760 - MicroMHz.LOoffset_5760;
                                                        Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                        //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 10368000000) & (n1mmfreq <= 10372000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main10GBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_10368 - MicroMHz.LOoffset_10368;
                                                        Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                        //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 24192000000) & (n1mmfreq <= 24196000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        main24GBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqGHz = n1mmfreq - MicroMHz.LO_24192 - MicroMHz.LOoffset_24192;
                                                        Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                                                        //  Console.Writeline("IF Radio Frequency B is " + Micro.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 1800000) & (n1mmfreq <= 2000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 3500000) & (n1mmfreq <= 4000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 7000000) & (n1mmfreq <= 7300000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 10000000) & (n1mmfreq <= 10150000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 14000000) & (n1mmfreq <= 14350000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 18068000) & (n1mmfreq <= 18168000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 21000000) & (n1mmfreq <= 21450000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 24890000) & (n1mmfreq <= 24990000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if ((n1mmfreq >= 28000000) & (n1mmfreq <= 30000000))
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }
                                                else if (n1mmfreq <= 50000000)
                                                {
                                                    Invoke((MethodInvoker)delegate
                                                    {
                                                        mainHFBut.Checked = true;
                                                        Properties.Settings.Default.split_tx_freqHF = (int)(n1mmfreq);
                                                        _HF.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqHF);
                                                        //  Console.Writeline("IF Radio Frequency B is " + _HF.display_freqB);
                                                    });

                                                }

                                            }

                                            else if (n1mmstr.Contains("TX"))
                                            {
                                                // put radio into transmit
                                                xmit = true;
                                                SwitchBand(Globals.mainradio);
                                                //     moxoffAsync(tempendpoint, temprcvdmcport, false);
                                                moxoff(tempendpoint, temprcvdmcport, false);
                                                //  Console.Writeline("Transmit is {0}", xmit);
                                            }


                                            else if (n1mmstr.Contains("RX"))
                                            {
                                                xmit = false;
                                                SwitchBand(Globals.mainradio);
                                                //    moxoffAsync(tempendpoint, temprcvdmcport, true);
                                                moxoff(tempendpoint, temprcvdmcport, true);
                                                //  Console.Writeline("Receive is {0}", !xmit);
                                            }

                                            n1mmstrb.Clear();
                                    } //end of if n1mmstr != null                   

                                }  //end of while globals.stopper = false


                                //  Console.Writeline("LEFT DIGICAT WHILE LOOP");
                                if (DigiSerialPort.IsOpen)
                                {
                                    //  Console.Writeline("DIGISERIALPORT IS OPEN, LEFT WHILELOOP");
                                }
                                else
                                {
                                    //  Console.Writeline("DigiSerialPort is Closed, still within thread pool but outside while");
                                }
                            }
                        }
                        catch
                        {
                            //  Console.Writeline("Exception occurred in Digi Liaison Radio thread");
                            if (DigiSerialPort.IsOpen)
                            {
                                //  Console.Writeline("DIGISERIALPORT IS OPEN");
                            }
                            else
                            {
                                digiCATBox.BackColor = Color.Red;
                            }

                        }

                    });

                    if (!Globals.digiCAT)
                    {
                        if (DigiSerialPort.IsOpen)
                        {
                            //  Console.Writeline("DIGISERIALPORT IS OPEN and Globals.digiCAT is false");
                        }
                        else
                        {
                            //  Console.Writeline("DigiSerialPort is Closed and Globals.digiCAT is false");
                        }
                    }
                }
                catch
                {
                    //  Console.Writeline("Entire DigiCAT Try failed");
                }
            }
            //  Console.Writeline("Leaving DigiCAT method");
        }

        //end DigiCAT Method


        //Below is Method to Convert HPSDRmode to CATmode, used by DigiCATCalcs Method
        //MD is for mode
        // MD1 LSB  OpenHPSDR 4
        // MD2 USB            3
        // MD3 CWU            5
        // MD4 FMN            2
        // MD5 AM             0
        // MD 6 RTTY <not used>
        // MD7 CWL            6
        string getCATMODE(int HPSDRmode)
        {
            switch (HPSDRmode)
            {
                case 4:
                    {
                        return "MD1;";
                    }
                case 3:
                    {
                        return "MD2;";
                    }
                case 5:
                    {
                        return "MD3;";
                    }
                case 2:
                    {
                        return "MD4;";
                    }
                case 0:
                    {
                        return "MD5;";
                    }
                case 6:
                    {
                        return "MD7;";
                    }
                default:
                    return "";

            }
        }

        void gotoSetup()
        {
            // only allow user to open form if not already open
            if (!SetupForm2Valid())
            {
                Setup_form2 = new SetupForm2(this);
                Setup_form2.TopMost = true;          // have the setup form appear over the main form.
                Setup_form2.Owner = this;
                Setup_form2.ShowDialog();                  // display the Setup form 
            }
            else  // form already open 
            {
                // handle setup form already being shown.  Make sure the user sees it...
                Setup_form2.ShowDialog();
            }
        }

        public void selectBands()
        {
            string bandMessage = "";
            if (Properties.Settings.Default.Checked50) { bandMessage = bandMessage + "50 "; }
            if (Properties.Settings.Default.Checked144) { bandMessage = bandMessage + "144 "; }
            if (Properties.Settings.Default.Checked222) { bandMessage = bandMessage + "222 "; }
            if (Properties.Settings.Default.Checked432) { bandMessage = bandMessage + "432 "; }
            if (Properties.Settings.Default.Checked903) { bandMessage = bandMessage + "903 "; }
            if (Properties.Settings.Default.Checked1296) { bandMessage = bandMessage + "1296 "; }
            if (Properties.Settings.Default.GHzChecked) { bandMessage = bandMessage + "Micro "; }
            if (Properties.Settings.Default.HFChecked) { bandMessage = bandMessage + "HF "; }
            //  Console.Writeline("Form1 selectBands Bands Selected Changed and now include " + bandMessage);

            //show bands that were selected on setup form
            if (Properties.Settings.Default.Checked50)
            {
                _50.Show();
                try
                {
                    main50But.Visible = true;
                    main50But.Enabled = true;
                    aux50But.Visible = true;
                    aux50But.Enabled = true;
                }
                catch { }
            }
            //hide 50 buttons on control bar if 50 not enabled
            else
            {
                _50.Hide();
                try
                {
                    main50But.Visible = false;
                    main50But.Enabled = false;
                    aux50But.Visible = false;
                    aux50But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.Checked144)
            {
                _144.Show();
                try
                {
                    main144But.Visible = true;
                    main144But.Enabled = true;
                    aux144But.Visible = true;
                    aux144But.Enabled = true;
                }
                catch { }
            }
            //hide 144 buttons on control bar if 144 not enabled
            else
            {
                _144.Hide();
                try
                {
                    main144But.Visible = false;
                    main144But.Enabled = false;
                    aux144But.Visible = false;
                    aux144But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.Checked222)
            {
                _222.Show();
                try
                {
                    main222But.Visible = true;
                    main222But.Enabled = true;
                    aux222But.Visible = true;
                    aux222But.Enabled = true;
                }
                catch { }
            }
            //hide 222 buttons on control bar if 222 not enabled
            else
            {
                _222.Hide();
                try
                {
                    main222But.Visible = false;
                    main222But.Enabled = false;
                    aux222But.Visible = false;
                    aux222But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.Checked432)
            {
                _432.Show();
                try
                {
                    main432But.Visible = true;
                    main432But.Enabled = true;
                    aux432But.Visible = true;
                    aux432But.Enabled = true;
                }
                catch { }
            }
            //hide 432 buttons on control bar if 432 not enabled
            else
            {
                _432.Hide();
                try
                {
                    main432But.Visible = false;
                    main432But.Enabled = false;
                    aux432But.Visible = false;
                    aux432But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.Checked903)
            {
                _903.Show();
                try
                {
                    main903But.Visible = true;
                    main903But.Enabled = true;
                    aux903But.Visible = true;
                    aux903But.Enabled = true;
                }
                catch { }
            }
            //hide 903 buttons on control bar if 903 not enabled
            else
            {
                _903.Hide();
                try
                {
                    main903But.Visible = false;
                    main903But.Enabled = false;
                    aux903But.Visible = false;
                    aux903But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.Checked1296)
            {
                _1296.Show();
                try
                {
                    main1296But.Visible = true;
                    main1296But.Enabled = true;
                    aux1296But.Visible = true;
                    aux1296But.Enabled = true;
                }
                catch { }
            }
            //hide 1296 buttons on control bar if 1296 not enabled
            else
            {
                _1296.Hide();
                try
                {
                    main1296But.Visible = false;
                    main1296But.Enabled = false;
                    aux1296But.Visible = false;
                    aux1296But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.GHzChecked)
            {
                Micro.Show();
                try
                {
                    main24GBut.Visible = true;
                    main24GBut.Enabled = true;
                    aux24GBut.Visible = true;
                    aux24GBut.Enabled = true;
                    main10GBut.Visible = true;
                    main10GBut.Enabled = true;
                    aux10GBut.Visible = true;
                    aux10GBut.Enabled = true;
                    main5760But.Visible = true;
                    main5760But.Enabled = true;
                    aux5760But.Visible = true;
                    aux5760But.Enabled = true;
                    main3456But.Visible = true;
                    main3456But.Enabled = true;
                    aux3456But.Visible = true;
                    aux3456But.Enabled = true;
                    main2304But.Visible = true;
                    main2304But.Enabled = true;
                    aux2304But.Visible = true;
                    aux2304But.Enabled = true;
                }
                catch { }
            }
            //hide GHz buttons on control bar if GHz not enabled
            else
            {
                Micro.Hide();
                try
                {
                    main24GBut.Visible = false;
                    main24GBut.Enabled = false;
                    aux24GBut.Visible = false;
                    aux24GBut.Enabled = false;
                    main10GBut.Visible = false;
                    main10GBut.Enabled = false;
                    aux10GBut.Visible = false;
                    aux10GBut.Enabled = false;
                    main5760But.Visible = false;
                    main5760But.Enabled = false;
                    aux5760But.Visible = false;
                    aux5760But.Enabled = false;
                    main3456But.Visible = false;
                    main3456But.Enabled = false;
                    aux3456But.Visible = false;
                    aux3456But.Enabled = false;
                    main2304But.Visible = false;
                    main2304But.Enabled = false;
                    aux2304But.Visible = false;
                    aux2304But.Enabled = false;
                }
                catch { }
            }

            if (Properties.Settings.Default.HFChecked)
            {
                _HF.Show();
                try
                {
                    mainHFBut.Visible = true;
                    mainHFBut.Enabled = true;
                    auxHFBut.Visible = true;
                    auxHFBut.Enabled = true;
                }
                catch { }
            }
            // hide HF bottons on control bar if HF not enabled
            else
            {
                _HF.Hide();
                try
                {
                    mainHFBut.Visible = false;
                    mainHFBut.Enabled = false;
                    auxHFBut.Visible = false;
                    auxHFBut.Enabled = false;
                }
                catch { }
            }
        }


        public void setRxAudioOut()
        {
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            RxAudioOut = deviceEnum.EnumAudioEndpoints(DataFlow.Render, DeviceState.Active);
            waveOutIndex = Properties.Settings.Default.WaveOutIndex;
            int rxnum = RxAudioOut.Count;
            if (waveOutIndex > rxnum)
            {
                waveOutIndex = rxnum;
                Properties.Settings.Default.WaveOutIndex = rxnum;
                Properties.Settings.Default.Save();
            }
            Setup2.waveOutDevices = new string[rxnum];
            int irnum;
            for (irnum = 0; irnum < rxnum; irnum++)
            {
                Setup2.waveOutDevices[irnum] = RxAudioOut[irnum].FriendlyName;
            }
            Setup2.RxAudioOutBox.Items.AddRange(Setup2.waveOutDevices);
            try
            {
                Properties.Settings.Default.RxAudioOutName = RxAudioOut[waveOutIndex].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen WaveOut Device Index is " + waveOutIndex);
                //  Console.Writeline("WaveOut Device Name is " + RxAudioOut[waveOutIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up mic Receive Audio");
            }

        }

        public void setDigiAudioOut()
        {
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            RxDigAudioOut = deviceEnum.EnumAudioEndpoints(DataFlow.Render, DeviceState.Active);
            DigiWaveOutIndex = Properties.Settings.Default.DigiWaveOutIndex;
            int drxnum = RxDigAudioOut.Count;
            if (DigiWaveOutIndex > drxnum)
            {
                DigiWaveOutIndex = drxnum;
                Properties.Settings.Default.DigiWaveOutIndex = drxnum;
                Properties.Settings.Default.Save();
            }
            Setup2.DigiWaveOutDevices = new string[drxnum];
            int dirnum;
            for (dirnum = 0; dirnum < drxnum; dirnum++)
            {
                Setup2.DigiWaveOutDevices[dirnum] = RxDigAudioOut[dirnum].FriendlyName;
            }
            Setup2.RxDigAudioOutBox.Items.AddRange(Setup2.DigiWaveOutDevices);
            try
            {
                Properties.Settings.Default.DigiRxAudioOutName = RxDigAudioOut[DigiWaveOutIndex].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen Digital WaveOut Device Index is " + DigiWaveOutIndex);
                //  Console.Writeline("Digital WaveOut Device Name is " + RxDigAudioOut[DigiWaveOutIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up Digi Receive Audio");
            }

        }


        public void setDigiAudioOutAux()
        {
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            RxDigAudioOutAux = deviceEnum.EnumAudioEndpoints(DataFlow.Render, DeviceState.Active);
            DigiWaveOutIndexAux = Properties.Settings.Default.DigiWaveOutIndexAux;
            int drxnum = RxDigAudioOutAux.Count;
            if (DigiWaveOutIndexAux > drxnum)
            {
                DigiWaveOutIndexAux = drxnum;
                Properties.Settings.Default.DigiWaveOutIndexAux = drxnum;
                Properties.Settings.Default.Save();
            }
            Setup2.DigiWaveOutDevicesAux = new string[drxnum];
            int dirnum;
            for (dirnum = 0; dirnum < drxnum; dirnum++)
            {
                Setup2.DigiWaveOutDevicesAux[dirnum] = RxDigAudioOutAux[dirnum].FriendlyName;
            }
            Setup2.RxDigAudioOutBoxAux.Items.AddRange(Setup2.DigiWaveOutDevicesAux);
            try
            {
                Properties.Settings.Default.DigiRxAudioOutNameAux = RxDigAudioOutAux[DigiWaveOutIndexAux].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen Digital WaveOut Device Index is " + DigiWaveOutIndex);
                //  Console.Writeline("Digital WaveOut Device Name is " + RxDigAudioOut[DigiWaveOutIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up Aux Digi Receive Audio");
            }

        }

        public void setN1MMAudioOut()
        {
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            RxN1MMAudioOut = deviceEnum.EnumAudioEndpoints(DataFlow.Render, DeviceState.Active);
            N1MMWaveOutIndex = Properties.Settings.Default.N1MMWaveOutIndex;
            int N1MMOxnum = RxN1MMAudioOut.Count;
            if (N1MMWaveOutIndex > N1MMOxnum)
            {
                N1MMWaveOutIndex = N1MMOxnum;
                Properties.Settings.Default.N1MMWaveOutIndex = N1MMOxnum;
                Properties.Settings.Default.Save();
            }
            Setup2.N1MMWaveOutDevices = new string[N1MMOxnum];
            int N1onum;
            for (N1onum = 0; N1onum < N1MMOxnum; N1onum++)
            {
                Setup2.N1MMWaveOutDevices[N1onum] = RxN1MMAudioOut[N1onum].FriendlyName;
            }
            Setup2.RxN1MMAudioOutBox.Items.AddRange(Setup2.N1MMWaveOutDevices);
            try
            {
                Properties.Settings.Default.N1MMRxAudioOutName = RxN1MMAudioOut[N1MMWaveOutIndex].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen N1MM+ WaveOut Device Index is " + N1MMWaveOutIndex);
                //  Console.Writeline("N1MM+ WaveOut Device Name is " + RxN1MMAudioOut[N1MMWaveOutIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up N1MM Receive Audio");
            }

        }

        public void setTxAudioIn()
        {
            waveInIndex = Properties.Settings.Default.WaveInIndex;
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            micdevices = deviceEnum.EnumAudioEndpoints(DataFlow.Capture, DeviceState.Active);
            int txnum = micdevices.Count;
            if (waveInIndex > txnum)
            {
                waveInIndex = txnum;
                Properties.Settings.Default.WaveInIndex = txnum;
                Properties.Settings.Default.Save();
            }
            int inum;
            Setup2.waveInDevices = new string[txnum];
            for (inum = 0; inum < txnum; inum++)
            {
                if (micdevices[inum].FriendlyName != null)
                {
                    Setup2.waveInDevices[inum] = micdevices[inum].FriendlyName;
                    //  Console.Writeline("Index = {0}, Mic Device Name = {1} , Mic Device State =  {2} ", inum, micdevices[inum].FriendlyName, micdevices[inum].DeviceState);
                }
            }
            Setup2.TxAudioInBox.Items.AddRange(Setup2.waveInDevices);
            try
            {
                Properties.Settings.Default.TxAudioInName = micdevices[waveInIndex].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen WaveIn Device Index is " + waveInIndex);
                //  Console.Writeline("WaveIn Device Name is " + micdevices[waveInIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up mic Tx Audio");
            }

        }

        public void setTxDigiAudioIn()
        {
            DigiWaveInIndex = Properties.Settings.Default.DigiWaveInIndex;
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            TxDigidevices = deviceEnum.EnumAudioEndpoints(DataFlow.Capture, DeviceState.Active);
            int TxDiginum = TxDigidevices.Count;
            if (DigiWaveInIndex > TxDiginum)
            {
                DigiWaveInIndex = TxDiginum;
                Properties.Settings.Default.DigiWaveInIndex = TxDiginum;
                Properties.Settings.Default.Save();
            }
            int isum;
            Setup2.DigiWaveInDevices = new string[TxDiginum];
            for (isum = 0; isum < TxDiginum; isum++)
            {
                if (TxDigidevices[isum].FriendlyName != null)
                {
                    Setup2.DigiWaveInDevices[isum] = TxDigidevices[isum].FriendlyName;
                }
            }
            Setup2.TxDigAudioInBox.Items.AddRange(Setup2.DigiWaveInDevices);
            try
            {
                Properties.Settings.Default.DigiTxAudioInName = TxDigidevices[DigiWaveInIndex].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen Digi WaveIn Device Index is " + DigiWaveInIndex);
                //  Console.Writeline("Digi WaveIn Device Name is " + TxDigidevices[DigiWaveInIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up Digi Tx Audio");
            }

        }


        public void setTxDigiAudioInAux()
        {
            DigiWaveInIndexAux = Properties.Settings.Default.DigiWaveInIndexAux;
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            TxDigidevicesAux = deviceEnum.EnumAudioEndpoints(DataFlow.Capture, DeviceState.Active);
            int TxDiginum = TxDigidevicesAux.Count;
            if (DigiWaveInIndexAux > TxDiginum)
            {
                DigiWaveInIndexAux = TxDiginum;
                Properties.Settings.Default.DigiWaveInIndexAux = TxDiginum;
                Properties.Settings.Default.Save();
            }
            int isum;
            Setup2.DigiWaveInDevicesAux = new string[TxDiginum];
            for (isum = 0; isum < TxDiginum; isum++)
            {
                if (TxDigidevicesAux[isum].FriendlyName != null)
                {
                    Setup2.DigiWaveInDevicesAux[isum] = TxDigidevicesAux[isum].FriendlyName;
                }
            }
            Setup2.TxDigAudioInBoxAux.Items.AddRange(Setup2.DigiWaveInDevicesAux);
            try
            {
                Properties.Settings.Default.DigiTxAudioInNameAux = TxDigidevicesAux[DigiWaveInIndexAux].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen Digi WaveIn Device Index is " + DigiWaveInIndex);
                //  Console.Writeline("Digi WaveIn Device Name is " + TxDigidevices[DigiWaveInIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up Aux Digi Tx Audio");
            }

        }

        public void setTxN1MMAudioIn()
        {
            N1MMWaveInIndex = Properties.Settings.Default.N1MMWaveInIndex;
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            TxN1MMdevices = deviceEnum.EnumAudioEndpoints(DataFlow.Capture, DeviceState.Active);
            int N1MMxnum = TxN1MMdevices.Count;
            if (N1MMWaveInIndex > N1MMxnum)
            {
                N1MMWaveInIndex = N1MMxnum;
                Properties.Settings.Default.N1MMWaveInIndex = N1MMxnum;
                Properties.Settings.Default.Save();
            }
            Setup2.N1MMWaveInDevices = new string[N1MMxnum];
            int N1num;
            for (N1num = 0; N1num < N1MMxnum; N1num++)
            {
                if (TxN1MMdevices[N1num].FriendlyName != null)
                {
                    Setup2.N1MMWaveInDevices[N1num] = TxN1MMdevices[N1num].FriendlyName;
                }
            }
            Setup2.TxN1MMAudioInBox.Items.AddRange(Setup2.N1MMWaveInDevices);
            try
            {
                Properties.Settings.Default.N1MMTxAudioInName = TxN1MMdevices[N1MMWaveInIndex].FriendlyName;
                Properties.Settings.Default.Save();
                //  Console.Writeline("Chosen N1MM+ WaveIn Device Index is " + N1MMWaveInIndex);
                //  Console.Writeline("N1MM+ WaveIn Device Name is " + TxN1MMdevices[N1MMWaveInIndex].FriendlyName);
            }
            catch
            {
                MessageBox.Show("Error in setting up N1MM Tx Audio");
            }

        }



        //serial port for CW and PTT input 
        public void openPort()
        {
            try { port.Close(); }
            catch { }
            try { port.Dispose(); }
            catch { }
            port = new SerialPort(Properties.Settings.Default.CWInputCOM, 9600, Parity.None, 8, StopBits.One);

            port.PinChanged += new SerialPinChangedEventHandler(Com_StatusChanged);

            try
            {
                port.Open();
                //  Console.Writeline("CW Input COM Port Opened to " + Properties.Settings.Default.CWInputCOM);
            }
            catch
            {
                //  Console.Writeline("Unable to open port (Hardware CW/PTT port)");
            }
            //DTR is used as common pullup for CW dot and dash input
            //RTS is used as pullup for footpedal input
            port.DtrEnable = true;
            port.RtsEnable = true;
        }

        //serial port for Main Digital PTT (WSJT etc.)
        public void openPort2()
        {
            try { port2.Close(); }
            catch { }
            try { port2.Dispose(); }
            catch { }
            port2 = new SerialPort(Properties.Settings.Default.DigitalPTTCOM, 9600, Parity.None, 8, StopBits.One);

            port2.PinChanged += new SerialPinChangedEventHandler(Com_StatusChanged2);
            try
            {
                port2.Open();
                //  Console.Writeline("Digital COM Port Opened to " + Properties.Settings.Default.DigitalPTTCOM);
            }
            catch
            {
                //  Console.Writeline("Unable to open port2 (Digital PTT port)");
            }
        }


        //serial port for N1MM Main PTT input
        public void openPort3()
        {
            try { port3.Close(); }
            catch { }
            try { port3.Dispose(); }
            catch { }

            port3 = new SerialPort(Properties.Settings.Default.N1MMPTTCOM, 4800, Parity.None, 8, StopBits.Two);
            port3.PinChanged += new SerialPinChangedEventHandler(Com_StatusChanged3);
            try
            {
                port3.Open();
                //  Console.Writeline("N1MM COM Port Opened to " + Properties.Settings.Default.N1MMPTTCOM);
            }
            catch
            {
                //  Console.Writeline("Unable to open port3 (N1MM Main PTT port)");
            }
        }


        //serial port for N1MM Aux PTT input
        public void openPort5()
        {
            try { port5.Close(); }
            catch { }
            try { port5.Dispose(); }
            catch { }

            port5 = new SerialPort(Properties.Settings.Default.N1MMPTTCOMAux, 4800, Parity.None, 8, StopBits.Two);
            port5.PinChanged += new SerialPinChangedEventHandler(Com_StatusChanged5);
            try
            {
                port5.Open();
                //  Console.Writeline("N1MM COM Port Opened to " + Properties.Settings.Default.N1MMPTTCOMAux);
            }
            catch
            {
                //  Console.Writeline("Unable to open port5 (N1MM Aux PTT port)");
            }
        }

        //serial port for Aux Digital PTT (WSJT etc.)
        public void openPort4()
        {
            try { port4.Close(); }
            catch { }
            try { port4.Dispose(); }
            catch { }
            port4 = new SerialPort(Properties.Settings.Default.DigitalPTTCOMAux, 9600, Parity.None, 8, StopBits.One);

            port4.PinChanged += new SerialPinChangedEventHandler(Com_StatusChanged4);
            try
            {
                port4.Open();
                //  Console.Writeline("Digital COM Port Opened to " + Properties.Settings.Default.DigitalPTTCOMAux);
            }
            catch
            {
                //  Console.Writeline("Unable to open port4 (Aux Digital PTT port)");
            }
        }


        public async Task moxoffAsync(string serverip, int port, bool moxon)
        {
            Task task = Task.Run(() => moxoff(serverip, port, moxon));
            await task;
        }


        //routine to turn MOX [transmit] on or off
        void moxoff(string serverip, int port, bool moxon)
        {
            if (serverip != null)
            {
                //uses reverse logic for mox_set, to work properly with existing code for individual radios
                //true turns mox off
                //false turns mox on

                byte[] MOXOntextstrng = StrToByteArray("moxo" + moxon.ToString()); //w3sz  convert to byte array
                int MOXOnstrngsize = MOXOntextstrng.Length;  //w3sz  get byte array size


                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer4 = new UdpClient(); //w3sz

                IPEndPoint localpt4 = new IPEndPoint(IPAddress.Parse(serverip), port);  //w3sz  command

                udpServer4.ExclusiveAddressUse = false;  //w3sz
                udpServer4.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz

                try
                {
                    udpServer4.Client.Connect(localpt4);  //w3sz changed from client.bind Jan 2024
                }
                catch
                {
                }

                // first is code to turn mox OFF
                if (moxon == true)
                {

                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz   

                    switch (port)
                    {
                        case 8000:
                            {

                                Invoke((MethodInvoker)delegate
                                {
                                    Micro.MOX.BackColor = SystemColors.Control;
                                    Micro.TUN.BackColor = SystemColors.Control;
                                });

                                Micro.MOX_set = false;
                                Micro.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + Micro.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + Micro.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz  

                                if ( ((Micro.ModeText == "USB" || Micro.ModeText == "LSB" || Micro.ModeText == "FM" || Micro.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.mainMicro)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {
                                            //  Console.Writeline("Problem in Form1.moxoff stopping mic_in line 3390");
                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.auxMicro)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {
                                            //  Console.Writeline("Problem in Form1.moxoff stopping mic_in line 3407");
                                        }

                                    }

                                }
                                else if ((Micro.ModeText == "CWU" || Micro.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8001:
                            {
                                _50.MOX_set = false;
                                _50.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _50.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _50.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                Invoke((MethodInvoker)delegate
                                {
                                    _50.MOX.BackColor = SystemColors.Control;
                                    _50.TUN.BackColor = SystemColors.Control;
                                });
                                if (((_50.ModeText == "USB" || _50.ModeText == "LSB" || _50.ModeText == "FM" || _50.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.main50)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {
                                            //  Console.Writeline("Problem in Form1.moxoff stopping mic_in line 3444");
                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.aux50)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {
                                            //  Console.Writeline("Problem in Form1.moxoff stopping mic_in line 3461");
                                        }

                                    }
                                }
                                else if ((_50.ModeText == "CWU" || _50.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8002:
                            {
                                Invoke((MethodInvoker)delegate
                                {
                                    _144.MOX.BackColor = SystemColors.Control;
                                    _144.TUN.BackColor = SystemColors.Control;
                                });
                                _144.MOX_set = false;
                                _144.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _144.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _144.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                if (((_144.ModeText == "USB" || _144.ModeText == "LSB" || _144.ModeText == "FM" || _144.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.main144)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.aux144)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                }
                                else if ((_144.ModeText == "CWU" || _144.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8003:
                            {
                                Invoke((MethodInvoker)delegate
                                {
                                    _222.MOX.BackColor = SystemColors.Control;
                                    _222.TUN.BackColor = SystemColors.Control;
                                });
                                _222.MOX_set = false;
                                _222.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _222.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _222.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                if (((_222.ModeText == "USB" || _222.ModeText == "LSB" || _222.ModeText == "FM" || _222.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.main222)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.aux222)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {

                                        }

                                    }
                                }
                                else if ((_222.ModeText == "CWU" || _222.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8004:
                            {
                                Invoke((MethodInvoker)delegate
                                {
                                    _432.MOX.BackColor = SystemColors.Control;
                                    _432.TUN.BackColor = SystemColors.Control;
                                });
                                _432.MOX_set = false;
                                _432.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _432.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _432.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                if ( ((_432.ModeText == "USB" || _432.ModeText == "LSB" || _432.ModeText == "FM" || _432.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.main432)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.aux432)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {

                                        }

                                    }
                                }
                                else if ((_432.ModeText == "CWU" || _432.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8005:
                            {
                                Invoke((MethodInvoker)delegate
                                {
                                    _903.MOX.BackColor = SystemColors.Control;
                                    _903.TUN.BackColor = SystemColors.Control;
                                });
                                _903.MOX_set = false;
                                _903.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _903.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _903.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                if (((_903.ModeText == "USB" || _903.ModeText == "LSB" || _903.ModeText == "FM" || _903.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.main903)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.aux903)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {

                                        }

                                    }
                                }
                                else if ((_903.ModeText == "CWU" || _903.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8006:
                            {
                                Invoke((MethodInvoker)delegate
                                {
                                    _1296.MOX.BackColor = SystemColors.Control;
                                    _1296.TUN.BackColor = SystemColors.Control;
                                });
                                _1296.MOX_set = false;
                                _1296.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _1296.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _1296.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                if (((_1296.ModeText == "USB" || _1296.ModeText == "LSB" || _1296.ModeText == "FM" || _1296.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.main1296)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.aux1296)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {

                                        }

                                    }
                                }
                                else if ((_1296.ModeText == "CWU" || _1296.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        case 8007:
                            {
                                Invoke((MethodInvoker)delegate
                                {
                                    _HF.MOX.BackColor = SystemColors.Control;
                                    _HF.TUN.BackColor = SystemColors.Control;
                                });
                                _HF.MOX_set = false;
                                _HF.TUN_set = false;

                                byte[] textstrng = StrToByteArray("rvol" + _HF.VolumeLeft.ToString()); //w3sz  convert to byte array
                                int textlength = textstrng.Length;
                                byte[] righttextstrng = StrToByteArray("lvol" + _HF.VolumeRight.ToString()); //w3sz  convert to byte array
                                int righttextlength = righttextstrng.Length;

                                udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                udpServer4.Close(); //w3sz   

                                if ( ((_HF.ModeText == "USB" || _HF.ModeText == "LSB" || _HF.ModeText == "FM" || _HF.ModeText == "AM")))
                                {
                                    if ((Globals.sendMic == true) & Globals.mainHF)
                                    {
                                        Globals.sendMic = false;
                                        try
                                        {
                                            micMonOut.Volume = 0.0f;
                                        }
                                        catch { }
                                        try
                                        {
                                            mic_in.Stop();
                                        }
                                        catch
                                        {

                                        }
                                    }
                                    else if ((Globals.sendMicAux == true) & Globals.auxHF)
                                    {
                                        Globals.sendMicAux = false;
                                        try
                                        {
                                            micMonOut2.Volume = 0.0f;
                                        }
                                        catch { }

                                        try
                                        {
                                            mic_in2.Stop();
                                        }
                                        catch
                                        {

                                        }

                                    }
                                }
                                else if ((_HF.ModeText == "CWU" || _HF.ModeText == "CWL"))
                                {
                                    Globals.sendCW = false;
                                }
                                break;
                            }
                        default:
                            break;
                    }
                }

                //now to turn MOX ON
                else if (moxon == false)
                {
                    switch (port)
                    {
                        case 8000:
                            {
                                if (Micro.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz  
                                    if (MicroMHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + Micro.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + Micro.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }

                                    Micro.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        Micro.MOX.BackColor = Color.Green;
                                    });

                                    //                           Micro.MOX.BackColor = Color.Green;
                                    if ( ((Micro.ModeText == "USB" || Micro.ModeText == "LSB" || Micro.ModeText == "FM" || Micro.ModeText == "AM")))
                                    {

                                        if ((Globals.sendMic == false) & Globals.mainMicro)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }
                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.auxMicro)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }
                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((Micro.ModeText == "CWU" || Micro.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        case 8001:
                            {
                                if (_50.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz   

                                    if (_50MHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _50.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _50.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }

                                    _50.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _50.MOX.BackColor = Color.Green;
                                    });

                                    //                  _50.MOX.BackColor = Color.Green;
                                    if ( ((_50.ModeText == "USB" || _50.ModeText == "LSB" || _50.ModeText == "FM" || _50.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.main50)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }
                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.aux50)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }
                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((_50.ModeText == "CWU" || _50.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }

                        case 8002:
                            {
                                if (_144.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz  

                                    if (_144MHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _144.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _144.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }

                                    _144.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _144.MOX.BackColor = Color.Green;
                                    });

                                    //                        _144.MOX.BackColor = Color.Green;
                                    if ( ((_144.ModeText == "USB" || _144.ModeText == "LSB" || _144.ModeText == "FM" || _144.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.main144)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }
                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.aux144)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }
                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((_144.ModeText == "CWU" || _144.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        case 8003:
                            {
                                if (_222.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz   
                                    if (_222MHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _222.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _222.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }


                                    _222.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _222.MOX.BackColor = Color.Green;
                                    });

                                    //                                    _222.MOX.BackColor = Color.Green;
                                    if (((_222.ModeText == "USB" || _222.ModeText == "LSB" || _222.ModeText == "FM" || _222.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.main222)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.aux222)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((_222.ModeText == "CWU" || _222.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        case 8004:
                            {
                                if (_432.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz 

                                    if (_432MHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _432.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _432.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }


                                    _432.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _432.MOX.BackColor = Color.Green;
                                    });

                                    //                                    _432.MOX.BackColor = Color.Green;
                                    if ( ((_432.ModeText == "USB" || _432.ModeText == "LSB" || _432.ModeText == "FM" || _432.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.main432)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.aux432)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                    }
                                    else if ((_432.ModeText == "CWU" || _432.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        case 8005:
                            {
                                if (_903.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz   

                                    if (_903MHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _903.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _903.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }


                                    _903.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _903.MOX.BackColor = Color.Green;
                                    });

                                    //                                    _903.MOX.BackColor = Color.Green;
                                    if (((_903.ModeText == "USB" || _903.ModeText == "LSB" || _903.ModeText == "FM" || _903.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.main903)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.aux903)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((_903.ModeText == "CWU" || _903.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        case 8006:
                            {
                                if (_1296.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz  

                                    if (_1296MHz.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _1296.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _1296.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }


                                    _1296.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _1296.MOX.BackColor = Color.Green;
                                    });

                                    //                                    _1296.MOX.BackColor = Color.Green;
                                    if (((_1296.ModeText == "USB" || _1296.ModeText == "LSB" || _1296.ModeText == "FM" || _1296.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.main1296)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;
                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.aux1296)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;
                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((_1296.ModeText == "CWU" || _1296.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        case 8007:
                            {
                                if (_HF.OnOffButton.BackColor == Color.Green)
                                {
                                    udpServer4.Send(MOXOntextstrng, MOXOnstrngsize, localpt4); // reply back  //w3sz  

                                    if (_HF.Duplex)
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + _HF.VolumeLeft.ToString()); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;
                                        byte[] righttextstrng = StrToByteArray("lvol" + _HF.VolumeRight.ToString()); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Close(); //w3sz   
                                    }
                                    else
                                    {
                                        byte[] textstrng = StrToByteArray("rvol" + "0"); //w3sz  convert to byte array
                                        int textlength = textstrng.Length;

                                        byte[] righttextstrng = StrToByteArray("lvol" + "0"); //w3sz  convert to byte array
                                        int righttextlength = righttextstrng.Length;

                                        udpServer4.Send(textstrng, textlength, localpt4); // reply back  //w3sz 
                                        udpServer4.Send(righttextstrng, righttextlength, localpt4); // reply back  //w3sz   
                                        udpServer4.Close(); //w3sz   
                                    }


                                    _HF.MOX_set = false;

                                    Invoke((MethodInvoker)delegate
                                    {
                                        _HF.MOX.BackColor = Color.Green;
                                    });

                                    //                                    _HF.MOX.BackColor = Color.Green;
                                    if (((_HF.ModeText == "USB" || _HF.ModeText == "LSB" || _HF.ModeText == "FM" || _HF.ModeText == "AM")))
                                    {
                                        if ((Globals.sendMic == false) & Globals.mainHF)
                                        {
                                            Globals.sendMic = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }
                                        else if ((Globals.sendMicAux == false) & Globals.auxHF)
                                        {
                                            Globals.sendMicAux = true;
                                            byte[] junk;

                                            try
                                            {
                                                micMonOut2.Volume = monVol;
                                            }
                                            catch { }

                                            while (mic_data_queue2.TryDequeue(out junk)) ;
                                            try
                                            {
                                                mic_in2.Start();
                                            }
                                            catch
                                            {

                                            }
                                        }

                                    }
                                    else if ((_HF.ModeText == "CWU" || _HF.ModeText == "CWL"))
                                    {
                                        Globals.sendCW = true;
                                    }
                                }
                                break;
                            }
                        default:
                            break;
                    }
                }
            }
        }

        // this is called when we exit the program - terminate the Ethernet connection and save the 
        // current state of the Radio in the Moving_Forms_Test.csv file.


        public async Task closingCommandsAsync()
        {
            Task task = Task.Run(() => closingCommands());
            await task;
        }

        private void closingCommands()
        {
            //uses reverse logic for mox_set, to work properly with existing code
            byte[] MOXOntextstrng = StrToByteArray("moxo" + "True"); //w3sz  convert to byte array
            int MOXOnstrngsize = MOXOntextstrng.Length;  //w3sz  get byte array size

            byte[] RcvOfftextstrng = StrToByteArray("LRON" + udpbyteoff); //w3sz  convert to byte array
            int RcvOffstrngsize = RcvOfftextstrng.Length;  //w3sz  get byte array size

            byte[] RcvOfftextstrng2 = StrToByteArray("MRON" + udpbyteoff); //w3sz  convert to byte array
            int RcvOffstrngsize2 = RcvOfftextstrng2.Length;  //w3sz  get byte array size

            Properties.Settings.Default.F1LocationX = this.Location.X;
            Properties.Settings.Default.F1LocationY = this.Location.Y;

            if (Properties.Settings.Default.Checked50)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer24 = new UdpClient(); //w3sz
                IPEndPoint localpt24 = new IPEndPoint(IPAddress.Parse(_50MHz.w3szendpoint1), 8001);  //w3sz  command

                udpServer24.ExclusiveAddressUse = false;  //w3sz
                udpServer24.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz  
                try
                {
                    udpServer24.Client.Connect(localpt24);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer24.Send(MOXOntextstrng, MOXOnstrngsize, localpt24); // reply back  //w3sz       
                udpServer24.Send(RcvOfftextstrng, RcvOffstrngsize, localpt24); // reply back  //w3sz     
                udpServer24.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt24); // reply back  //w3sz  
                udpServer24.Close(); //w3sz  
            }

            if (Properties.Settings.Default.Checked144)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer25 = new UdpClient(); //w3sz
                IPEndPoint localpt25 = new IPEndPoint(IPAddress.Parse(_144MHz.w3szendpoint1), 8002);  //w3sz  command
                udpServer25.ExclusiveAddressUse = false;  //w3sz
                udpServer25.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer25.Client.Connect(localpt25);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer25.Send(MOXOntextstrng, MOXOnstrngsize, localpt25); // reply back  //w3sz       
                udpServer25.Send(RcvOfftextstrng, RcvOffstrngsize, localpt25); // reply back  //w3sz     
                udpServer25.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt25); // reply back  //w3sz  
                udpServer25.Close(); //w3sz   
            }

            if (Properties.Settings.Default.Checked222)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer27 = new UdpClient(); //w3sz
                IPEndPoint localpt27 = new IPEndPoint(IPAddress.Parse(_222MHz.w3szendpoint1), 8003);  //w3sz  command
                udpServer27.ExclusiveAddressUse = false;  //w3sz
                udpServer27.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer27.Client.Connect(localpt27);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer27.Send(MOXOntextstrng, MOXOnstrngsize, localpt27); // reply back  //w3sz       
                udpServer27.Send(RcvOfftextstrng, RcvOffstrngsize, localpt27); // reply back  //w3sz     
                udpServer27.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt27); // reply back  //w3sz  
                udpServer27.Close(); //w3sz   
            }

            if (Properties.Settings.Default.Checked432)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer28 = new UdpClient(); //w3sz
                IPEndPoint localpt28 = new IPEndPoint(IPAddress.Parse(_432MHz.w3szendpoint1), 8004);  //w3sz  command
                udpServer28.ExclusiveAddressUse = false;  //w3sz
                udpServer28.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer28.Client.Connect(localpt28);  //w3sz  changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer28.Send(MOXOntextstrng, MOXOnstrngsize, localpt28); // reply back  //w3sz       
                udpServer28.Send(RcvOfftextstrng, RcvOffstrngsize, localpt28); // reply back  //w3sz     
                udpServer28.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt28); // reply back  //w3sz  
                udpServer28.Close(); //w3sz  
            }

            if (Properties.Settings.Default.Checked903)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer29 = new UdpClient(); //w3sz
                IPEndPoint localpt29 = new IPEndPoint(IPAddress.Parse(_903MHz.w3szendpoint1), 8005);  //w3sz  command
                udpServer29.ExclusiveAddressUse = false;  //w3sz
                udpServer29.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer29.Client.Connect(localpt29);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer29.Send(MOXOntextstrng, MOXOnstrngsize, localpt29); // reply back  //w3sz         
                udpServer29.Send(RcvOfftextstrng, RcvOffstrngsize, localpt29); // reply back  //w3sz       
                udpServer29.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt29); // reply back  //w3sz  
                udpServer29.Close(); //w3sz   
            }

            if (Properties.Settings.Default.Checked1296)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer30 = new UdpClient(); //w3sz
                IPEndPoint localpt30 = new IPEndPoint(IPAddress.Parse(_1296MHz.w3szendpoint1), 8006);  //w3sz  command
                udpServer30.ExclusiveAddressUse = false;  //w3sz
                udpServer30.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer30.Client.Connect(localpt30);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer30.Send(MOXOntextstrng, MOXOnstrngsize, localpt30); // reply back  //w3sz         
                udpServer30.Send(RcvOfftextstrng, RcvOffstrngsize, localpt30); // reply back  //w3sz       
                udpServer30.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt30); // reply back  //w3sz  
                udpServer30.Close(); //w3sz   
            }

            if (Properties.Settings.Default.GHzChecked)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer31 = new UdpClient(); //w3sz
                IPEndPoint localpt31 = new IPEndPoint(IPAddress.Parse(MicroMHz.w3szendpoint1), 8000);  //w3sz  command
                udpServer31.ExclusiveAddressUse = false;  //w3sz
                udpServer31.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer31.Client.Connect(localpt31);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer31.Send(MOXOntextstrng, MOXOnstrngsize, localpt31); // reply back  //w3sz         
                udpServer31.Send(RcvOfftextstrng, RcvOffstrngsize, localpt31); // reply back  //w3sz       
                udpServer31.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt31); // reply back  //w3sz  
                udpServer31.Close(); //w3sz   
            }

            if (Properties.Settings.Default.HFChecked)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer32 = new UdpClient(); //w3sz
                IPEndPoint localpt32 = new IPEndPoint(IPAddress.Parse(_HF.w3szendpoint1), 8007);  //w3sz  command
                udpServer32.ExclusiveAddressUse = false;  //w3sz
                udpServer32.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer32.Client.Connect(localpt32);  //w3sz   changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer32.Send(MOXOntextstrng, MOXOnstrngsize, localpt32); // reply back  //w3sz       
                udpServer32.Send(RcvOfftextstrng, RcvOffstrngsize, localpt32); // reply back  //w3sz     
                udpServer32.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt32); // reply back  //w3sz  
                udpServer32.Close(); //w3sz 
            }


        }

        private void Form_Closing(object sender, FormClosingEventArgs e)
        {
            //save window positions
            if (Properties.Settings.Default.PositionControllerBar == null)
            {
                Properties.Settings.Default.PositionControllerBar = new WindowSettings();
            }

            Properties.Settings.Default.PositionControllerBar.Record(this);
            Properties.Settings.Default.Save();

            //  Console.Writeline("Transverter Frequencies are:");
            //  Console.Writeline(_50.xvtr_frequency);
            //  Console.Writeline(_144.xvtr_frequency);
            //  Console.Writeline(_222.xvtr_frequency);
            //  Console.Writeline(_432.xvtr_frequency);
            //  Console.Writeline(_903.xvtr_frequency);
            //  Console.Writeline(_1296.xvtr_frequency);


            //  Console.Writeline("Micro Transverter Frequencies are:");
            //  Console.Writeline(Micro.xvtr_frequency_50);
            //  Console.Writeline(Micro.xvtr_frequency_144);
            //  Console.Writeline(Micro.xvtr_frequency_222);
            //  Console.Writeline(Micro.xvtr_frequency_432);
            //  Console.Writeline(Micro.xvtr_frequency_903);
            //  Console.Writeline(Micro.xvtr_frequency_1296);

            if (_50.xvtr_frequency > 40000000) Micro.xvtr_frequency_50 = _50.xvtr_frequency;
            MicroMHz.Preamp_50 = _50.Preamp.Checked;
            MicroMHz.Drive50 = _50.DriveLevel.Value;

            if (_144.xvtr_frequency > 40000000) Micro.xvtr_frequency_144 = _144.xvtr_frequency;
            MicroMHz.Preamp_144 = _144.Preamp.Checked;
            MicroMHz.Drive144 = _144.DriveLevel.Value;

            if (_222.xvtr_frequency > 40000000) Micro.xvtr_frequency_222 = _222.xvtr_frequency;
            MicroMHz.Preamp_222 = _222.Preamp.Checked;
            MicroMHz.Drive222 = _222.DriveLevel.Value;

            if (_432.xvtr_frequency > 40000000) Micro.xvtr_frequency_432 = _432.xvtr_frequency;
            MicroMHz.Preamp_432 = _432.Preamp.Checked;
            MicroMHz.Drive432 = _432.DriveLevel.Value;

            if (_903.xvtr_frequency > 40000000) Micro.xvtr_frequency_903 = _903.xvtr_frequency;
            MicroMHz.Preamp_903 = _903.Preamp.Checked;
            MicroMHz.Drive903 = _903.DriveLevel.Value;

            if (_1296.xvtr_frequency > 40000000) Micro.xvtr_frequency_1296 = _1296.xvtr_frequency;
            MicroMHz.Preamp_1296 = _1296.Preamp.Checked;
            MicroMHz.Drive1296 = _1296.DriveLevel.Value;

            // save program settings in KK.csv file
            WriteKKCSV(KKCSVName);


            try
            {
                StopAudio();
            }
            catch
            {

            }
            try
            {
                port.Close();
            }
            catch
            {

            }
            try
            {

                port2.Close();
            }
            catch
            {

            }
            try
            {

                port3.Close();
            }
            catch
            {

            }
            try
            {

                port4.Close();
            }
            catch
            {

            }
            try
            {

                port5.Close();
            }
            catch
            {

            }

            Globals.stopper = true;

            _50.pictureBox1.Dispose();
            _50.pictureBox3.Dispose();
            _50.Dispose();
            _144.pictureBox1.Dispose();
            _144.pictureBox3.Dispose();
            _144.Dispose();
            _222.pictureBox1.Dispose();
            _222.pictureBox3.Dispose();
            _222.Dispose();
            _432.pictureBox1.Dispose();
            _432.pictureBox3.Dispose();
            _432.Dispose();
            _903.pictureBox1.Dispose();
            _903.pictureBox3.Dispose();
            _903.Dispose();
            _1296.pictureBox1.Dispose();
            _1296.pictureBox3.Dispose();
            _1296.Dispose();
            Micro.pictureBox1.Dispose();
            Micro.pictureBox3.Dispose();
            Micro.Dispose();
            _HF.pictureBox1.Dispose();
            _HF.pictureBox3.Dispose();
            _HF.Dispose();
            _encoder.Dispose();
            try
            {
                _decoder.Dispose();
                _decoder2.Dispose();
            }
            catch
            { }
            try
            {
                soundSource.Dispose();
            }
            catch
            { }
            this.Dispose();
            Thread.Sleep(200);


            try
            {
                System.Environment.Exit(0);
            }
            catch (Exception d)
            {
                //  Console.Writeline("An error during System Exit occurred: '{0}'", d);
            }
        }

        private void SafeUpDownValueChange(decimal newValue, NumericUpDown control)
        {
            if (newValue < control.Minimum)
                newValue = control.Minimum;
            if (newValue > control.Maximum)
                newValue = control.Maximum;

            control.Value = newValue;
        }

        private void SafeTrackBarValueChange(int newValue, TrackBar control)
        {
            if (newValue < control.Minimum)
                newValue = control.Minimum;
            if (newValue > control.Maximum)
                newValue = control.Maximum;

            control.Value = newValue;
        }

        //Below are routines for changing bands for Main and then Aux radios

        byte[] digiOnString = null;
        byte[] digiAuxOnString = null;

        public void main50But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked50)
            {
                if (main50But.Checked == true)
                {
                    if (aux50But.Checked == true)
                    {
                        aux50But.Checked = false;
                    }
                    main144But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.main50 = true;
                    Globals.aux50 = false;
                    _50.Size = Globals.formlargesize;
                    _50.Location = Globals.formmainloc;
                    _50.pictureBox1.Size = Globals.boxlargesize;
                    _50.pictureBox1.Location = Globals.box1largeloc;
                    _50.xvtr_frequency = Micro.xvtr_frequency_50;
                    _50.Preamp.Checked = MicroMHz.Preamp_50;
                    _50.LO = MicroMHz.LO_50;
                    _50.LOoffset = MicroMHz.LOoffset_50;
                    _50.BandGain = MicroMHz.Gain50MHz;
                    _50.DriveLevel.Value = MicroMHz.Drive50;
                    _50.DriveLevel_Scroll(this, EventArgs.Empty);
                    _50.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _50.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _50.Squelch_level_Scroll(this, EventArgs.Empty);
                    _50.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_50 - _50.LO - _50.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _50.DisplayFrequency(_50.set_frequency.Value);
                    _50.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq50);
                    _50.AdjustFilterByMode();
                    _50.Sidetone.Checked = Globals.sidetonechecked;
                    _50.oldfreq = _50.display_freqB.Text;
                    Globals.mainradio = "50";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpoint50, 8001);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main50But.Checked == false)
                {
                    SwitchBand("50");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (aux50But.Checked == false)
                    {
                        Globals.aux50 = false;
                        Globals.main50 = false;
                        _50.Size = Globals.formsmallsize;
                        _50.Location = Globals.form50smallloc;
                        _50.pictureBox1.Size = Globals.boxsmallsize;
                        _50.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                    if ((_50.xvtr_frequency >= xvtr_frequency_50_minimum) & (_50.xvtr_frequency <= xvtr_frequency_50_maximum))
                    {
                        Micro.xvtr_frequency_50 = _50.xvtr_frequency;
                        MicroMHz.Preamp_50 = _50.Preamp.Checked;
                        MicroMHz.Drive50 = _50.DriveLevel.Value;
                    }
                }
            }
        }
        public void main144But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked144)
            {
                if (main144But.Checked == true)
                {
                    if (aux144But.Checked == true)
                    {
                        aux144But.Checked = false;
                    }
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.main144 = true;
                    Globals.aux144 = false;
                    _144.Size = Globals.formlargesize;
                    _144.Location = Globals.formmainloc;
                    _144.pictureBox1.Size = Globals.boxlargesize;
                    _144.pictureBox1.Location = Globals.box1largeloc;
                    //  Console.Writeline("MIcro is " + Micro.xvtr_frequency_144);
                    _144.xvtr_frequency = Micro.xvtr_frequency_144;
                    _144.Preamp.Checked = MicroMHz.Preamp_144;
                    _144.LO = MicroMHz.LO_144;
                    _144.LOoffset = MicroMHz.LOoffset_144;
                    _144.BandGain = MicroMHz.Gain144MHz;
                    _144.DriveLevel.Value = MicroMHz.Drive144;
                    _144.DriveLevel_Scroll(this, EventArgs.Empty);
                    _144.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _144.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _144.Squelch_level_Scroll(this, EventArgs.Empty);
                    _144.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_144 - _144.LO - _144.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _144.DisplayFrequency(_144.set_frequency.Value);
                    _144.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq144);
                    _144.AdjustFilterByMode();
                    _144.Sidetone.Checked = Globals.sidetonechecked;
                    _144.oldfreq = _144.display_freqB.Text;
                    Globals.mainradio = "144";
                    SwitchMainBand(Globals.mainradio);
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpoint144, 8002);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main144But.Checked == false)
                {
                    SwitchBand("144");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (aux144But.Checked == false)
                    {
                        Globals.aux144 = false;
                        Globals.main144 = false;
                        _144.Size = Globals.formsmallsize;
                        _144.Location = Globals.form144smallloc;
                        _144.pictureBox1.Size = Globals.boxsmallsize;
                        _144.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                    if ((_144.xvtr_frequency >= xvtr_frequency_144_minimum) & (_144.xvtr_frequency <= xvtr_frequency_144_maximum))
                    {
                        Micro.xvtr_frequency_144 = _144.xvtr_frequency;
                        //  Console.Writeline("MIcro is " + Micro.xvtr_frequency_144);
                        MicroMHz.Preamp_144 = _144.Preamp.Checked;
                        MicroMHz.Drive144 = _144.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_144.xvtr_frequency is {0}", _144.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void main222But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked222)
            {
                if (main222But.Checked == true)
                {
                    if (aux222But.Checked == true)
                    {
                        aux222But.Checked = false;
                    }
                    main50But.Checked = false;
                    main144But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.main222 = true;
                    Globals.aux222 = false;

                    //  Console.Writeline("xWidth after click before resize is " + _222MHz.xWidth.ToString());

                    _222.Size = Globals.formlargesize;
                    _222.Location = Globals.formmainloc;
                    _222.pictureBox1.Size = Globals.boxlargesize;
                    _222.pictureBox1.Location = Globals.box1largeloc;


                    //  Console.Writeline("xWidth after click after resize is " + _222MHz.xWidth.ToString());

                    _222.xvtr_frequency = Micro.xvtr_frequency_222;
                    _222.Preamp.Checked = MicroMHz.Preamp_222;
                    _222.LO = MicroMHz.LO_222;
                    _222.LOoffset = MicroMHz.LOoffset_222;
                    _222.BandGain = MicroMHz.Gain222MHz;
                    _222.DriveLevel.Value = MicroMHz.Drive222;
                    _222.DriveLevel_Scroll(_222, EventArgs.Empty);
                    _222.ProcessorGain_Scroll(_222, EventArgs.Empty);
                    _222.Autoset_frequency_Scroll_1(_222, EventArgs.Empty);
                    _222.Squelch_level_Scroll(_222, EventArgs.Empty);
                    _222.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_222 - _222.LO - _222.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _222.DisplayFrequency(_222.set_frequency.Value);
                    _222.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq222);
                    _222.AdjustFilterByMode();
                    _222.Sidetone.Checked = Globals.sidetonechecked;
                    _222.oldfreq = _222.display_freqB.Text;
                    Globals.mainradio = "222";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpoint222, 8003);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main222But.Checked == false)
                {
                    SwitchBand("222");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (aux222But.Checked == false)
                    {
                        Globals.aux222 = false;
                        Globals.main222 = false;
                        _222.Size = Globals.formsmallsize;
                        _222.Location = Globals.form222smallloc;
                        _222.pictureBox1.Size = Globals.boxsmallsize;
                        _222.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                    if ((_222.xvtr_frequency >= xvtr_frequency_222_minimum) & (_222.xvtr_frequency <= xvtr_frequency_222_maximum))
                    {
                        Micro.xvtr_frequency_222 = _222.xvtr_frequency;
                        MicroMHz.Preamp_222 = _222.Preamp.Checked;
                        MicroMHz.Drive222 = _222.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_222.xvtr_frequency is {0}", _222.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void main432But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked432)
            {
                if (main432But.Checked == true)
                {
                    if (aux432But.Checked == true)
                    {
                        aux432But.Checked = false;
                    }
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main144But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.main432 = true;
                    Globals.aux432 = false;
                    _432.Size = Globals.formlargesize;
                    _432.Location = Globals.formmainloc;
                    _432.pictureBox1.Size = Globals.boxlargesize;
                    _432.pictureBox1.Location = Globals.box1largeloc;
                    _432.xvtr_frequency = Micro.xvtr_frequency_432;
                    _432.Preamp.Checked = MicroMHz.Preamp_432;
                    _432.LO = MicroMHz.LO_432;
                    _432.LOoffset = MicroMHz.LOoffset_432;
                    _432.BandGain = MicroMHz.Gain432MHz;
                    _432.DriveLevel.Value = MicroMHz.Drive432;
                    _432.DriveLevel_Scroll(this, EventArgs.Empty);
                    _432.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _432.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _432.Squelch_level_Scroll(this, EventArgs.Empty);
                    _432.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_432 - _432.LO - _432.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _432.DisplayFrequency(_432.set_frequency.Value);
                    _432.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq432);
                    _432.AdjustFilterByMode();
                    _432.Sidetone.Checked = Globals.sidetonechecked;
                    _432.oldfreq = _432.display_freqB.Text;
                    Globals.mainradio = "432";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpoint432, 8004);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main432But.Checked == false)
                {
                    SwitchBand("432");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (aux432But.Checked == false)
                    {
                        Globals.aux432 = false;
                        Globals.main432 = false;
                        _432.Size = Globals.formsmallsize;
                        _432.Location = Globals.form432smallloc;
                        _432.pictureBox1.Size = Globals.boxsmallsize;
                        _432.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                    if ((_432.xvtr_frequency >= xvtr_frequency_432_minimum) & (_432.xvtr_frequency <= xvtr_frequency_432_maximum))
                    {
                        Micro.xvtr_frequency_432 = _432.xvtr_frequency;
                        MicroMHz.Preamp_432 = _432.Preamp.Checked;
                        MicroMHz.Drive432 = _432.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_432.xvtr_frequency is {0}", _432.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void main903But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked903)
            {
                if (main903But.Checked == true)
                {
                    if (aux903But.Checked == true)
                    {
                        aux903But.Checked = false;
                    }
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main144But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.main903 = true;
                    Globals.aux903 = false;
                    _903.Size = Globals.formlargesize;
                    _903.Location = Globals.formmainloc;
                    _903.pictureBox1.Size = Globals.boxlargesize;
                    _903.pictureBox1.Location = Globals.box1largeloc;
                    _903.xvtr_frequency = Micro.xvtr_frequency_903;
                    _903.Preamp.Checked = MicroMHz.Preamp_903;
                    _903.LO = MicroMHz.LO_903;
                    _903.LOoffset = MicroMHz.LOoffset_903;
                    _903.BandGain = MicroMHz.Gain903MHz;
                    _903.DriveLevel.Value = MicroMHz.Drive903;
                    _903.DriveLevel_Scroll(this, EventArgs.Empty);
                    _903.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _903.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _903.Squelch_level_Scroll(this, EventArgs.Empty);
                    _903.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_903 - _903.LO - _903.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _903.DisplayFrequency(_903.set_frequency.Value);
                    _903.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq903);
                    _903.AdjustFilterByMode();
                    _903.Sidetone.Checked = Globals.sidetonechecked;
                    _903.oldfreq = _903.display_freqB.Text;
                    Globals.mainradio = "903";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpoint903, 8005);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main903But.Checked == false)
                {
                    SwitchBand("903");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (aux903But.Checked == false)
                    {
                        Globals.aux903 = false;
                        Globals.main903 = false;
                        _903.Size = Globals.formsmallsize;
                        _903.Location = Globals.form903smallloc;
                        _903.pictureBox1.Size = Globals.boxsmallsize;
                        _903.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                    if ((_903.xvtr_frequency >= xvtr_frequency_903_minimum) & (_903.xvtr_frequency <= xvtr_frequency_903_maximum))
                    {
                        Micro.xvtr_frequency_903 = _903.xvtr_frequency;
                        MicroMHz.Preamp_903 = _903.Preamp.Checked;
                        MicroMHz.Drive903 = _903.DriveLevel.Value;
                    }
                }
            }
        }
        public void main1296But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked1296)
            {
                if (main1296But.Checked == true)
                {
                    if (aux1296But.Checked == true)
                    {
                        aux1296But.Checked = false;
                    }
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main144But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.main1296 = true;
                    Globals.aux1296 = false;
                    _1296.Size = Globals.formlargesize;
                    _1296.Location = Globals.formmainloc;
                    _1296.pictureBox1.Size = Globals.boxlargesize;
                    _1296.pictureBox1.Location = Globals.box1largeloc;
                    _1296.xvtr_frequency = Micro.xvtr_frequency_1296;
                    _1296.Preamp.Checked = MicroMHz.Preamp_1296;
                    _1296.LO = MicroMHz.LO_1296;
                    _1296.LOoffset = MicroMHz.LOoffset_1296;
                    _1296.BandGain = MicroMHz.Gain1296MHz;
                    _1296.DriveLevel.Value = MicroMHz.Drive1296;
                    _1296.DriveLevel_Scroll(this, EventArgs.Empty);
                    _1296.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _1296.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _1296.Squelch_level_Scroll(this, EventArgs.Empty);
                    _1296.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_1296 - _1296.LO - _1296.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _1296.DisplayFrequency(_1296.set_frequency.Value);
                    _1296.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq1296);
                    _1296.AdjustFilterByMode();
                    _1296.Sidetone.Checked = Globals.sidetonechecked;
                    _1296.oldfreq = _1296.display_freqB.Text;
                    Globals.mainradio = "1296";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpoint1296, 8006);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main1296But.Checked == false)
                {
                    SwitchBand("1296");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (aux1296But.Checked == false)
                    {
                        Globals.aux1296 = false;
                        Globals.main1296 = false;
                        _1296.Size = Globals.formsmallsize;
                        _1296.Location = Globals.form1296smallloc;
                        _1296.pictureBox1.Size = Globals.boxsmallsize;
                        _1296.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                    if ((_1296.xvtr_frequency >= xvtr_frequency_1296_minimum) & (_1296.xvtr_frequency <= xvtr_frequency_1296_maximum))
                    {
                        Micro.xvtr_frequency_1296 = _1296.xvtr_frequency;
                        MicroMHz.Preamp_1296 = _1296.Preamp.Checked;
                        MicroMHz.Drive1296 = _1296.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_1296.xvtr_frequency is {0}", _1296.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void main2304But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (main2304But.Checked == true)
                {
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main144But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = true;
                    Globals.auxMicro = false;
                    Globals.microband = "2304";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formmainloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_2304;
                    Micro.Preamp.Checked = MicroMHz.Preamp_2304;
                    MicroMHz.LO = MicroMHz.LO_2304;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_2304;
                    Micro.BandGain = MicroMHz.Gain2304MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive2304;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_2304 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.mainradio = "GHz";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main2304But.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.auxMicro == false) & (main2304But.Checked == false) & (main3456But.Checked == false) & (main3456But.Checked == false) & (main5760But.Checked == false) & (main10GBut.Checked == false) & (main24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Globals.microband = "2304";
                        Micro.boxMicroBand.Visible = true;
                        Micro.button1.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_2304_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_2304_maximum))
                    {

                        Micro.xvtr_frequency_2304 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_2304 = Micro.Preamp.Checked;
                        MicroMHz.Drive2304 = Micro.DriveLevel.Value;
                    }
                    if (aux2304But.Checked == true)
                    {
                        Globals.mainradio = String.Empty;
                    }
                }
            }
        }
        public void main3456But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (main3456But.Checked == true)
                {
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main144But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = true;
                    Globals.auxMicro = false;
                    Globals.microband = "3456";
                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formmainloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_3456;
                    Micro.Preamp.Checked = MicroMHz.Preamp_3456;
                    MicroMHz.LO = MicroMHz.LO_3456;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_3456;
                    Micro.BandGain = MicroMHz.Gain3456MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive3456;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_3456 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.mainradio = "GHz";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (main3456But.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.auxMicro == false) & (main2304But.Checked == false) & (main3456But.Checked == false) & (main3456But.Checked == false) & (main5760But.Checked == false) & (main10GBut.Checked == false) & (main24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "3456";
                        Micro.button2.BackColor = SystemColors.ControlDark;
                        Micro.button1.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_3456_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_3456_maximum))
                    {
                        Micro.xvtr_frequency_3456 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_3456 = Micro.Preamp.Checked;
                        MicroMHz.Drive3456 = Micro.DriveLevel.Value;
                    }
                    if (aux3456But.Checked == true)
                    {
                        Globals.mainradio = String.Empty;
                    }
                }
            }
        }

        public void main5760But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (main5760But.Checked == true)
                {
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main144But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = true;
                    Globals.auxMicro = false;
                    Globals.microband = "5760";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formmainloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_5760;
                    Micro.Preamp.Checked = MicroMHz.Preamp_5760;
                    MicroMHz.LO = MicroMHz.LO_5760;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_5760;
                    Micro.BandGain = MicroMHz.Gain5760MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive5760;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_5760 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.mainradio = "GHz";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);

                }
                else if (main5760But.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.auxMicro == false) & (main2304But.Checked == false) & (main3456But.Checked == false) & (main3456But.Checked == false) & (main5760But.Checked == false) & (main10GBut.Checked == false) & (main24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "5760";
                        Micro.button3.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button1.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_5760_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_5760_maximum))
                    {
                        Micro.xvtr_frequency_5760 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_5760 = Micro.Preamp.Checked;
                        MicroMHz.Drive5760 = Micro.DriveLevel.Value;
                    }
                    if (aux5760But.Checked == true)
                    {
                        Globals.mainradio = String.Empty;
                    }
                }
            }
        }
        public void main10GBut_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (main10GBut.Checked == true)
                {
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main144But.Checked = false;
                    main24GBut.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = true;
                    Globals.auxMicro = false;
                    Globals.microband = "10368";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formmainloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_10368;
                    Micro.Preamp.Checked = MicroMHz.Preamp_10368;
                    MicroMHz.LO = MicroMHz.LO_10368;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_10368;
                    Micro.BandGain = MicroMHz.Gain10368MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive10368;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_10368 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.mainradio = "GHz";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);

                }
                else if (main10GBut.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.auxMicro == false) & (main2304But.Checked == false) & (main3456But.Checked == false) & (main3456But.Checked == false) & (main5760But.Checked == false) & (main10GBut.Checked == false) & (main24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "10368";
                        Micro.button4.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button1.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_10368_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_10368_maximum))
                    {
                        Micro.xvtr_frequency_10368 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_10368 = Micro.Preamp.Checked;
                        MicroMHz.Drive10368 = Micro.DriveLevel.Value;
                    }
                    if (aux10GBut.Checked == true)
                    {
                        Globals.mainradio = String.Empty;
                    }
                }
            }
        }

        public void main24GBut_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (main24GBut.Checked == true)
                {
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main144But.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.mainMicro = true;
                    Globals.auxMicro = false;
                    Globals.microband = "24192";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formmainloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_24192;
                    Micro.Preamp.Checked = MicroMHz.Preamp_24192;
                    MicroMHz.LO = MicroMHz.LO_24192;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_24192;
                    Micro.BandGain = MicroMHz.Gain24192MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive24192;
                    Micro.DriveLevel_Scroll(Micro, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(Micro, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(Micro, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(Micro, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_24192 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.mainradio = "GHz";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);

                }
                else if (main24GBut.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.auxMicro == false) & (main2304But.Checked == false) & (main3456But.Checked == false) & (main3456But.Checked == false) & (main5760But.Checked == false) & (main10GBut.Checked == false) & (main24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "24192";
                        Micro.button5.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button1.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_24192_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_24192_maximum))
                    {
                        Micro.xvtr_frequency_24192 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_24192 = Micro.Preamp.Checked;
                        MicroMHz.Drive24192 = Micro.DriveLevel.Value;
                    }
                    if (aux24GBut.Checked == true)
                    {
                        Globals.mainradio = String.Empty;
                    }
                }
            }
        }


        public void mainHFBut_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.HFChecked)
            {
                if (mainHFBut.Checked == true)
                {
                    main24GBut.Checked = false;
                    main50But.Checked = false;
                    main222But.Checked = false;
                    main432But.Checked = false;
                    main903But.Checked = false;
                    main1296But.Checked = false;
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main144But.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.mainMicro = false;
                    Globals.mainHF = true;
                    Globals.auxHF = false;
                    _HF.Size = Globals.formlargesize;
                    _HF.Location = Globals.formmainloc;
                    _HF.pictureBox1.Size = Globals.boxlargesize;
                    _HF.pictureBox1.Location = Globals.box1largeloc;    // set the band edges depending on the band selected. Restore the last used frequency from KK.csv

                    switch (_HF.BandSelect.Text)
                    {
                        case "160m":
                            _HF.CurrentBand = _HF.OpBand.M160;
                            _HF.set_frequency.Minimum = 1800000;  // low band limit
                            _HF.set_frequency.Maximum = 2000000;  // high band limit
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_160;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "USB")
                            {
                                _HF.ModeText = "LSB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_160;
                            _HF.BandGain = _HF.Gain160m; // set the band gain 
                            break;

                        case "80m":
                            _HF.CurrentBand = _HF.OpBand.M80;
                            _HF.set_frequency.Minimum = 3500000;
                            _HF.set_frequency.Maximum = 4000000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_80;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "USB")
                            {
                                _HF.ModeText = "LSB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_80;
                            _HF.BandGain = _HF.Gain80m;
                            break;

                        case "40m":
                            _HF.CurrentBand = _HF.OpBand.M40;
                            _HF.set_frequency.Minimum = 7000000;
                            _HF.set_frequency.Maximum = 7300000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_40;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "USB")
                            {
                                _HF.ModeText = "LSB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_40;
                            _HF.BandGain = _HF.Gain40m;
                            break;

                        case "30m":
                            _HF.CurrentBand = _HF.OpBand.M30;
                            _HF.set_frequency.Minimum = 10100000;
                            _HF.set_frequency.Maximum = 10150000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_30;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_30;
                            _HF.BandGain = _HF.Gain30m;
                            break;

                        case "20m":
                            _HF.CurrentBand = _HF.OpBand.M20;
                            _HF.set_frequency.Minimum = 14000000;
                            _HF.set_frequency.Maximum = 14350000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_20;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_20;
                            _HF.BandGain = _HF.Gain20m;
                            break;

                        case "17m":
                            _HF.CurrentBand = _HF.OpBand.M17;
                            _HF.set_frequency.Minimum = 18068000;
                            _HF.set_frequency.Maximum = 18168000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_17;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_17;
                            _HF.BandGain = _HF.Gain17m;
                            break;

                        case "15m":
                            _HF.CurrentBand = _HF.OpBand.M15;
                            _HF.set_frequency.Minimum = 21000000;
                            _HF.set_frequency.Maximum = 21450000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_15;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_15;
                            _HF.BandGain = _HF.Gain15m;
                            break;

                        case "12m":
                            _HF.CurrentBand = _HF.OpBand.M12;
                            _HF.set_frequency.Minimum = 24880000;
                            _HF.set_frequency.Maximum = 24980000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_12;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_12;
                            _HF.BandGain = _HF.Gain12m;
                            break;

                        case "10m":
                            _HF.CurrentBand = _HF.OpBand.M10;
                            _HF.set_frequency.Minimum = 28000000;
                            _HF.set_frequency.Maximum = 29700000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_10;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_10;
                            _HF.BandGain = _HF.Gain10m;
                            break;

                        case "6m":
                            _HF.CurrentBand = _HF.OpBand.M6;
                            _HF.set_frequency.Minimum = 50000000;
                            _HF.set_frequency.Maximum = 55000000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_6;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = _HF.set_frequency.Minimum;
                            }
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_6;
                            _HF.BandGain = _HF.Gain6m;
                            break;

                        case "GC":
                            _HF.CurrentBand = _HF.OpBand.GC;
                            _HF.set_frequency.Minimum = 0;
                            _HF.set_frequency.Maximum = 55000000;
                            try
                            {
                                _HF.set_frequency.Value = _HF.set_frequency_GC;
                            }
                            catch
                            {
                                _HF.set_frequency.Value = 1000000;
                            }
                            _HF.Preamp.Checked = _HF.Preamp_GC;
                            _HF.BandGain = 0;
                            break;

                        default:
                            _HF.CurrentBand = _HF.OpBand.M20;
                            _HF.set_frequency.Minimum = 14000000;
                            _HF.set_frequency.Maximum = 14350000;
                            _HF.set_frequency.Value = _HF.set_frequency_20;
                            if (_HF.ModeText == "LSB")
                            {
                                _HF.ModeText = "USB";                  // select the appropriate sideband
                                _HF.ChangeMode();  // reset the Bandwidth slider
                            }
                            _HF.Preamp.Checked = _HF.Preamp_20;
                            _HF.BandGain = _HF.Gain20m;
                            break;
                    }
                    _HF.AdjustFilterByMode();
                    _HF.Sidetone.Checked = Globals.sidetonechecked;
                    Globals.mainradio = "HF";
                    SwitchMainBand(Globals.mainradio);
                    digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
                    sendCommandAsync(digiOnString, Properties.Settings.Default.w3szendpointHF, 8007);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("MRON", udpbyteon, Globals.w3szendpoint, Globals.rcvaudiocommandport);
                }
                else if (mainHFBut.Checked == false)
                {
                    SwitchBand("HF");
                    SwitchAudioAsync("MRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (auxHFBut.Checked == false)
                    {
                        Globals.auxHF = false;
                        Globals.mainHF = false;
                        _HF.Size = Globals.formsmallsize;
                        _HF.Location = Globals.formHFsmallloc;
                        _HF.pictureBox1.Size = Globals.boxsmallsize;
                        _HF.pictureBox1.Location = Globals.box1smallloc;

                        switch (_HF.BandSelect.Text)
                        {
                            case "160m":
                                _HF.set_frequency_160 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_160 = _HF.Preamp.Checked;
                                _HF.Gain160m = _HF.BandGain;
                                break;

                            case "80m":
                                _HF.set_frequency_80 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_80 = _HF.Preamp.Checked;
                                _HF.Gain80m = _HF.BandGain;
                                break;

                            case "40m":
                                _HF.set_frequency_40 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_40 = _HF.Preamp.Checked;
                                _HF.Gain40m = _HF.BandGain;
                                break;

                            case "30m":
                                _HF.set_frequency_30 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_30 = _HF.Preamp.Checked;
                                _HF.Gain30m = _HF.BandGain;
                                break;

                            case "20m":
                                _HF.set_frequency_20 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_20 = _HF.Preamp.Checked;
                                _HF.Gain20m = _HF.BandGain;
                                break;

                            case "17m":
                                _HF.set_frequency_17 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_17 = _HF.Preamp.Checked;
                                _HF.Gain17m = _HF.BandGain;
                                break;

                            case "15m":
                                _HF.set_frequency_15 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_15 = _HF.Preamp.Checked;
                                _HF.Gain15m = _HF.BandGain;
                                break;

                            case "12m":
                                _HF.set_frequency_12 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_12 = _HF.Preamp.Checked;
                                _HF.Gain12m = _HF.BandGain;
                                break;

                            case "10m":
                                _HF.set_frequency_10 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_10 = _HF.Preamp.Checked;
                                _HF.Gain10m = _HF.BandGain;
                                break;

                            case "6m":
                                _HF.set_frequency_6 = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_6 = _HF.Preamp.Checked;
                                _HF.Gain6m = _HF.BandGain;
                                break;

                            case "GC":
                                _HF.set_frequency_GC = _HF.set_frequency.Value; // last used frequency from KK.csv
                                _HF.Preamp_GC = _HF.Preamp.Checked;
                                break;

                        }
                    }
                    else
                    {
                        Globals.mainradio = String.Empty;
                    }
                }
            }
        }

        public void aux50But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked50)
            {
                if (aux50But.Checked == true)
                {
                    if (main50But.Checked == true)
                    {
                        main50But.Checked = false;
                    }
                    aux144But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.aux50 = true;
                    Globals.main50 = false;
                    _50.Size = Globals.formlargesize;
                    _50.Location = Globals.formauxloc;
                    _50.pictureBox1.Size = Globals.boxlargesize;
                    _50.pictureBox1.Location = Globals.box1largeloc;
                    _50.xvtr_frequency = Micro.xvtr_frequency_50;
                    _50.Preamp.Checked = MicroMHz.Preamp_50;
                    _50.LO = MicroMHz.LO_50;
                    _50.LOoffset = MicroMHz.LOoffset_50;
                    _50.BandGain = MicroMHz.Gain50MHz;
                    _50.DriveLevel.Value = MicroMHz.Drive50;
                    _50.DriveLevel_Scroll(this, EventArgs.Empty);
                    _50.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _50.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _50.Squelch_level_Scroll(this, EventArgs.Empty);
                    _50.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_50 - _50.LO - _50.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _50.DisplayFrequency(_50.set_frequency.Value);
                    _50.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq50);
                    _50.AdjustFilterByMode();
                    _50.Sidetone.Checked = Globals.sidetonechecked;
                    _50.oldfreq = _50.display_freqB.Text;
                    Globals.auxradio = "50";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpoint50, 8001);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux50But.Checked == false)
                {
                    SwitchBand("50");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (main50But.Checked == false)
                    {
                        Globals.aux50 = false;
                        Globals.main50 = false;
                        _50.Size = Globals.formsmallsize;
                        _50.Location = Globals.form50smallloc;
                        _50.pictureBox1.Size = Globals.boxsmallsize;
                        _50.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.auxradio = String.Empty;
                    }
                    if ((_50.xvtr_frequency >= xvtr_frequency_50_minimum) & (_50.xvtr_frequency <= xvtr_frequency_50_maximum))
                    {
                        Micro.xvtr_frequency_50 = _50.xvtr_frequency;
                        MicroMHz.Preamp_50 = _50.Preamp.Checked;
                        MicroMHz.Drive50 = _50.DriveLevel.Value;
                    }
                }
            }
        }

        public void aux144But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked144)
            {
                if (aux144But.Checked == true)
                {
                    if (main144But.Checked == true)
                    {
                        main144But.Checked = false;
                    }
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.aux144 = true;
                    Globals.main144 = false;
                    _144.Size = Globals.formlargesize;
                    _144.Location = Globals.formauxloc;
                    _144.pictureBox1.Size = Globals.boxlargesize;
                    _144.pictureBox1.Location = Globals.box1largeloc;
                    //  Console.Writeline("MIcro is " + Micro.xvtr_frequency_144);
                    _144.xvtr_frequency = Micro.xvtr_frequency_144;
                    _144.Preamp.Checked = MicroMHz.Preamp_144;
                    _144.LO = MicroMHz.LO_144;
                    _144.LOoffset = MicroMHz.LOoffset_144;
                    _144.BandGain = MicroMHz.Gain144MHz;
                    _144.DriveLevel.Value = MicroMHz.Drive144;
                    _144.DriveLevel_Scroll(this, EventArgs.Empty);
                    _144.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _144.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _144.Squelch_level_Scroll(this, EventArgs.Empty);
                    _144.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_144 - _144.LO - _144.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _144.DisplayFrequency(_144.set_frequency.Value);
                    _144.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq144);
                    _144.AdjustFilterByMode();
                    _144.Sidetone.Checked = Globals.sidetonechecked;
                    _144.oldfreq = _144.display_freqB.Text;
                    Globals.auxradio = "144";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpoint144, 8002);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);
                }
                else if (aux144But.Checked == false)
                {
                    SwitchBand("144");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (main144But.Checked == false)
                    {
                        Globals.aux144 = false;
                        Globals.main144 = false;
                        _144.Size = Globals.formsmallsize;
                        _144.Location = Globals.form144smallloc;
                        _144.pictureBox1.Size = Globals.boxsmallsize;
                        _144.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.auxradio = String.Empty;
                    }
                    if ((_144.xvtr_frequency >= xvtr_frequency_144_minimum) & (_144.xvtr_frequency <= xvtr_frequency_144_maximum))
                    {
                        Micro.xvtr_frequency_144 = _144.xvtr_frequency;
                        //  Console.Writeline("MIcro is " + Micro.xvtr_frequency_144);
                        MicroMHz.Preamp_144 = _144.Preamp.Checked;
                        MicroMHz.Drive144 = _144.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_144.xvtr_frequency is {0}", _144.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void aux222But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked222)
            {
                if (aux222But.Checked == true)
                {
                    if (main222But.Checked == true)
                    {
                        main222But.Checked = false;
                    }
                    aux50But.Checked = false;
                    aux144But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.aux222 = true;
                    Globals.main222 = false;
                    _222.Size = Globals.formlargesize;
                    _222.Location = Globals.formauxloc;
                    _222.pictureBox1.Size = Globals.boxlargesize;
                    _222.pictureBox1.Location = Globals.box1largeloc;
                    _222.xvtr_frequency = Micro.xvtr_frequency_222;
                    _222.Preamp.Checked = MicroMHz.Preamp_222;
                    _222.LO = MicroMHz.LO_222;
                    _222.LOoffset = MicroMHz.LOoffset_222;
                    _222.BandGain = MicroMHz.Gain222MHz;
                    _222.DriveLevel.Value = MicroMHz.Drive222;
                    _222.DriveLevel_Scroll(_222, EventArgs.Empty);
                    _222.ProcessorGain_Scroll(_222, EventArgs.Empty);
                    _222.Autoset_frequency_Scroll_1(_222, EventArgs.Empty);
                    _222.Squelch_level_Scroll(_222, EventArgs.Empty);
                    _222.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_222 - _222.LO - _222.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _222.DisplayFrequency(_222.set_frequency.Value);
                    _222.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq222);
                    _222.AdjustFilterByMode();
                    _222.Sidetone.Checked = Globals.sidetonechecked;
                    _222.oldfreq = _222.display_freqB.Text;
                    Globals.auxradio = "222";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpoint222, 8003);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux222But.Checked == false)
                {
                    SwitchBand("222");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (main222But.Checked == false)
                    {
                        Globals.aux222 = false;
                        Globals.main222 = false;
                        _222.Size = Globals.formsmallsize;
                        _222.Location = Globals.form222smallloc;
                        _222.pictureBox1.Size = Globals.boxsmallsize;
                        _222.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.auxradio = String.Empty;
                    }
                    if ((_222.xvtr_frequency >= xvtr_frequency_222_minimum) & (_222.xvtr_frequency <= xvtr_frequency_222_maximum))
                    {
                        Micro.xvtr_frequency_222 = _222.xvtr_frequency;
                        MicroMHz.Preamp_222 = _222.Preamp.Checked;
                        MicroMHz.Drive222 = _222.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_222.xvtr_frequency is {0}", _222.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void aux432But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked432)
            {
                if (aux432But.Checked == true)
                {
                    if (main432But.Checked == true)
                    {
                        main432But.Checked = false;
                    }
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux144But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.aux432 = true;
                    Globals.main432 = false;
                    _432.Size = Globals.formlargesize;
                    _432.Location = Globals.formauxloc;
                    _432.pictureBox1.Size = Globals.boxlargesize;
                    _432.pictureBox1.Location = Globals.box1largeloc;
                    _432.xvtr_frequency = Micro.xvtr_frequency_432;
                    _432.Preamp.Checked = MicroMHz.Preamp_432;
                    _432.LO = MicroMHz.LO_432;
                    _432.LOoffset = MicroMHz.LOoffset_432;
                    _432.BandGain = MicroMHz.Gain432MHz;
                    _432.DriveLevel.Value = MicroMHz.Drive432;
                    _432.DriveLevel_Scroll(this, EventArgs.Empty);
                    _432.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _432.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _432.Squelch_level_Scroll(this, EventArgs.Empty);
                    _432.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_432 - _432.LO - _432.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _432.DisplayFrequency(_432.set_frequency.Value);
                    _432.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq432);
                    _432.AdjustFilterByMode();
                    _432.Sidetone.Checked = Globals.sidetonechecked;
                    _432.oldfreq = _432.display_freqB.Text;
                    Globals.auxradio = "432";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpoint432, 8004);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux432But.Checked == false)
                {
                    SwitchBand("432");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (main432But.Checked == false)
                    {
                        Globals.aux432 = false;
                        Globals.main432 = false;
                        _432.Size = Globals.formsmallsize;
                        _432.Location = Globals.form432smallloc;
                        _432.pictureBox1.Size = Globals.boxsmallsize;
                        _432.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.auxradio = String.Empty;
                    }
                    if ((_432.xvtr_frequency >= xvtr_frequency_432_minimum) & (_432.xvtr_frequency <= xvtr_frequency_432_maximum))
                    {
                        Micro.xvtr_frequency_432 = _432.xvtr_frequency;
                        MicroMHz.Preamp_432 = _432.Preamp.Checked;
                        MicroMHz.Drive432 = _432.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_432.xvtr_frequency is {0}", _432.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void aux903But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked903)
            {
                if (aux903But.Checked == true)
                {
                    if (main903But.Checked == true)
                    {
                        main903But.Checked = false;
                    }
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux144But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.aux903 = true;
                    Globals.main903 = false;
                    _903.Size = Globals.formlargesize;
                    _903.Location = Globals.formauxloc;
                    _903.pictureBox1.Size = Globals.boxlargesize;
                    _903.pictureBox1.Location = Globals.box1largeloc;
                    _903.xvtr_frequency = Micro.xvtr_frequency_903;
                    _903.Preamp.Checked = MicroMHz.Preamp_903;
                    _903.LO = MicroMHz.LO_903;
                    _903.LOoffset = MicroMHz.LOoffset_903;
                    _903.BandGain = MicroMHz.Gain903MHz;
                    _903.DriveLevel.Value = MicroMHz.Drive903;
                    _903.DriveLevel_Scroll(this, EventArgs.Empty);
                    _903.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _903.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _903.Squelch_level_Scroll(this, EventArgs.Empty);
                    _903.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_903 - _903.LO - _903.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _903.DisplayFrequency(_903.set_frequency.Value);
                    _903.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq903);
                    _903.AdjustFilterByMode();
                    _903.Sidetone.Checked = Globals.sidetonechecked;
                    _903.oldfreq = _903.display_freqB.Text;
                    Globals.auxradio = "903";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpoint903, 8005);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux903But.Checked == false)
                {
                    SwitchBand("903");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (main903But.Checked == false)
                    {
                        Globals.aux903 = false;
                        Globals.main903 = false;
                        _903.Size = Globals.formsmallsize;
                        _903.Location = Globals.form903smallloc;
                        _903.pictureBox1.Size = Globals.boxsmallsize;
                        _903.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.auxradio = String.Empty;
                    }
                    if ((_903.xvtr_frequency >= xvtr_frequency_903_minimum) & (_903.xvtr_frequency <= xvtr_frequency_903_maximum))
                    {
                        Micro.xvtr_frequency_903 = _903.xvtr_frequency;
                        MicroMHz.Preamp_903 = _903.Preamp.Checked;
                        MicroMHz.Drive903 = _903.DriveLevel.Value;
                    }
                }
            }
        }
        public void aux1296But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.Checked1296)
            {
                if (aux1296But.Checked == true)
                {
                    if (main1296But.Checked == true)
                    {
                        main1296But.Checked = false;
                    }
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux144But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.aux1296 = true;
                    Globals.main1296 = false;
                    _1296.Size = Globals.formlargesize;
                    _1296.Location = Globals.formauxloc;
                    _1296.pictureBox1.Size = Globals.boxlargesize;
                    _1296.pictureBox1.Location = Globals.box1largeloc;
                    _1296.xvtr_frequency = Micro.xvtr_frequency_1296;
                    _1296.Preamp.Checked = MicroMHz.Preamp_1296;
                    _1296.LO = MicroMHz.LO_1296;
                    _1296.LOoffset = MicroMHz.LOoffset_1296;
                    _1296.BandGain = MicroMHz.Gain1296MHz;
                    _1296.DriveLevel.Value = MicroMHz.Drive1296;
                    _1296.DriveLevel_Scroll(this, EventArgs.Empty);
                    _1296.ProcessorGain_Scroll(this, EventArgs.Empty);
                    _1296.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    _1296.Squelch_level_Scroll(this, EventArgs.Empty);
                    _1296.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_1296 - _1296.LO - _1296.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    _1296.DisplayFrequency(_1296.set_frequency.Value);
                    _1296.DisplayFrequencyB(Properties.Settings.Default.split_tx_freq1296);
                    _1296.AdjustFilterByMode();
                    _1296.Sidetone.Checked = Globals.sidetonechecked;
                    _1296.oldfreq = _1296.display_freqB.Text;
                    Globals.auxradio = "1296";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpoint1296, 8006);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux1296But.Checked == false)
                {
                    SwitchBand("1296");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (main1296But.Checked == false)
                    {
                        Globals.aux1296 = false;
                        Globals.main1296 = false;
                        _1296.Size = Globals.formsmallsize;
                        _1296.Location = Globals.form1296smallloc;
                        _1296.pictureBox1.Size = Globals.boxsmallsize;
                        _1296.pictureBox1.Location = Globals.box1smallloc;
                    }
                    else
                    {
                        Globals.auxradio = String.Empty;
                    }
                    if ((_1296.xvtr_frequency >= xvtr_frequency_1296_minimum) & (_1296.xvtr_frequency <= xvtr_frequency_1296_maximum))
                    {
                        Micro.xvtr_frequency_1296 = _1296.xvtr_frequency;
                        MicroMHz.Preamp_1296 = _1296.Preamp.Checked;
                        MicroMHz.Drive1296 = _1296.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("_1296.xvtr_frequency is {0}", _1296.xvtr_frequency.ToString());
                    }
                }
            }
        }
        public void aux2304But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (aux2304But.Checked == true)
                {
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux144But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = true;
                    Globals.mainMicro = false;
                    Globals.microband = "2304";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formauxloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_2304;
                    Micro.Preamp.Checked = MicroMHz.Preamp_2304;
                    MicroMHz.LO = MicroMHz.LO_2304;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_2304;
                    Micro.BandGain = MicroMHz.Gain2304MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive2304;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_2304 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.auxradio = "GHz";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux2304But.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.mainMicro == false) & (aux2304But.Checked == false) & (aux3456But.Checked == false) & (aux3456But.Checked == false) & (aux5760But.Checked == false) & (aux10GBut.Checked == false) & (aux24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "2304";
                        Micro.button1.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_2304_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_2304_maximum))
                    {
                        Micro.xvtr_frequency_2304 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_2304 = Micro.Preamp.Checked;
                        MicroMHz.Drive2304 = Micro.DriveLevel.Value;
                    }
                    if (main2304But.Checked == true)
                    {
                        Globals.auxradio = String.Empty;
                    }
                }
            }
        }
        public void aux3456But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (aux3456But.Checked == true)
                {
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux144But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = true;
                    Globals.mainMicro = false;
                    Globals.microband = "3456";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formauxloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_3456;
                    Micro.Preamp.Checked = MicroMHz.Preamp_3456;
                    MicroMHz.LO = MicroMHz.LO_3456;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_3456;
                    Micro.BandGain = MicroMHz.Gain3456MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive3456;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_3456 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.auxradio = "GHz";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux3456But.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.mainMicro == false) & (aux2304But.Checked == false) & (aux3456But.Checked == false) & (aux3456But.Checked == false) & (aux5760But.Checked == false) & (aux10GBut.Checked == false) & (aux24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "3456";
                        Micro.button2.BackColor = SystemColors.ControlDark;
                        Micro.button1.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_3456_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_3456_maximum))
                    {
                        Micro.xvtr_frequency_3456 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_3456 = Micro.Preamp.Checked;
                        MicroMHz.Drive3456 = Micro.DriveLevel.Value;
                    }
                    if (main3456But.Checked == true)
                    {
                        Globals.auxradio = String.Empty;
                    }
                }
            }
        }
        public void aux5760But_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (aux5760But.Checked == true)
                {
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux144But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = true;
                    Globals.mainMicro = false;
                    Globals.microband = "5760";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formauxloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_5760;
                    Micro.Preamp.Checked = MicroMHz.Preamp_5760;
                    MicroMHz.LO = MicroMHz.LO_5760;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_5760;
                    Micro.BandGain = MicroMHz.Gain5760MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive5760;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_5760 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.auxradio = "GHz";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux5760But.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.mainMicro == false) & (aux2304But.Checked == false) & (aux3456But.Checked == false) & (aux3456But.Checked == false) & (aux5760But.Checked == false) & (aux10GBut.Checked == false) & (aux24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "5760";
                        Micro.button3.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button1.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_5760_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_5760_maximum))
                    {
                        Micro.xvtr_frequency_5760 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_5760 = Micro.Preamp.Checked;
                        MicroMHz.Drive5760 = Micro.DriveLevel.Value;
                    }
                    if (main5760But.Checked == true)
                    {
                        Globals.auxradio = String.Empty;
                    }
                }
            }
        }
        public void aux10GBut_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (aux10GBut.Checked == true)
                {
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux144But.Checked = false;
                    aux24GBut.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = true;
                    Globals.mainMicro = false;
                    Globals.microband = "10368";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formauxloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_10368;
                    Micro.Preamp.Checked = MicroMHz.Preamp_10368;
                    MicroMHz.LO = MicroMHz.LO_10368;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_10368;
                    Micro.BandGain = MicroMHz.Gain10368MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive10368;
                    Micro.DriveLevel_Scroll(this, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(this, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(this, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(this, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_10368 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.auxradio = "GHz";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux10GBut.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.mainMicro == false) & (aux2304But.Checked == false) & (aux3456But.Checked == false) & (aux3456But.Checked == false) & (aux5760But.Checked == false) & (aux10GBut.Checked == false) & (aux24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "10368";
                        Micro.button4.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button1.BackColor = SystemColors.Control;
                        Micro.button5.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_10368_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_10368_maximum))
                    {
                        Micro.xvtr_frequency_10368 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_10368 = Micro.Preamp.Checked;
                        MicroMHz.Drive10368 = Micro.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("Micro.xvtr_frequency is {0}", Micro.xvtr_frequency.ToString());
                    }
                    if (main10GBut.Checked == true)
                    {
                        Globals.auxradio = String.Empty;
                    }
                }
            }
        }
        public void aux24GBut_CheckedChanged(object sender, EventArgs e)
        {
            if (Properties.Settings.Default.GHzChecked)
            {
                if (aux24GBut.Checked == true)
                {
                    main2304But.Checked = false;
                    main3456But.Checked = false;
                    main5760But.Checked = false;
                    main10GBut.Checked = false;
                    main24GBut.Checked = false;
                    aux50But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux144But.Checked = false;
                    auxHFBut.Checked = false;
                    Globals.auxMicro = true;
                    Globals.mainMicro = false;
                    Globals.microband = "24192";

                    SendBandDataAsync();
                    Micro.Size = Globals.formlargesize;
                    Micro.Location = Globals.formauxloc;
                    Micro.pictureBox1.Size = Globals.boxlargesize;
                    Micro.pictureBox1.Location = Globals.box1largeloc;
                    Micro.boxMicroBand.Visible = false;
                    Micro.xvtr_frequency = Micro.xvtr_frequency_24192;
                    Micro.Preamp.Checked = MicroMHz.Preamp_24192;
                    MicroMHz.LO = MicroMHz.LO_24192;
                    MicroMHz.LOoffset = MicroMHz.LOoffset_24192;
                    Micro.BandGain = MicroMHz.Gain24192MHz;
                    Micro.DriveLevel.Value = MicroMHz.Drive24192;
                    Micro.DriveLevel_Scroll(Micro, EventArgs.Empty);
                    Micro.ProcessorGain_Scroll(Micro, EventArgs.Empty);
                    Micro.Autoset_frequency_Scroll_1(Micro, EventArgs.Empty);
                    Micro.Squelch_level_Scroll(Micro, EventArgs.Empty);
                    Micro.set_frequency.Value = Convert.ToInt32(Micro.xvtr_frequency_24192 - MicroMHz.LO - MicroMHz.LOoffset); // last used frequency from Moving_Forms_Test.csv
                    Micro.DisplayFrequency(Micro.set_frequency.Value);
                    Micro.DisplayFrequencyB(Properties.Settings.Default.split_tx_freqGHz);
                    Micro.AdjustFilterByMode();
                    Micro.Sidetone.Checked = Globals.sidetonechecked;
                    Micro.oldfreq = Micro.display_freqB.Text;
                    Globals.auxradio = "GHz";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpointGHz, 8000);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                }
                else if (aux24GBut.Checked == false)
                {
                    SwitchBand("GHz");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if ((Globals.mainMicro == false) & (aux2304But.Checked == false) & (aux3456But.Checked == false) & (aux3456But.Checked == false) & (aux5760But.Checked == false) & (aux10GBut.Checked == false) & (aux24GBut.Checked == false))
                    {
                        Globals.mainMicro = false;
                        Micro.Size = Globals.formMicrosmallsize;
                        Micro.Location = Globals.formMicrosmallloc;
                        Micro.pictureBox1.Size = Globals.boxsmallsize;
                        Micro.pictureBox1.Location = Globals.box1smallloc;
                        Micro.boxMicroBand.Visible = true;
                        Globals.microband = "24192";
                        Micro.button5.BackColor = SystemColors.ControlDark;
                        Micro.button2.BackColor = SystemColors.Control;
                        Micro.button3.BackColor = SystemColors.Control;
                        Micro.button4.BackColor = SystemColors.Control;
                        Micro.button1.BackColor = SystemColors.Control;
                    }
                    if ((Micro.xvtr_frequency >= xvtr_frequency_24192_minimum) & (Micro.xvtr_frequency <= xvtr_frequency_24192_maximum))
                    {
                        Micro.xvtr_frequency_24192 = Micro.xvtr_frequency;
                        MicroMHz.Preamp_24192 = Micro.Preamp.Checked;
                        MicroMHz.Drive24192 = Micro.DriveLevel.Value;
                    }
                    else
                    {
                        MessageBox.Show("Micro.xvtr_frequency is {0}", Micro.xvtr_frequency.ToString());
                    }
                    if (main24GBut.Checked == true)
                    {
                        Globals.auxradio = String.Empty;
                    }
                }
            }
        }

        public void auxHFBut_CheckedChanged(object sender, EventArgs e)
        {
            //      if (Properties.Settings.Default.HFChecked)
            {
                if (auxHFBut.Checked == true)
                {
                    aux2304But.Checked = false;
                    aux3456But.Checked = false;
                    aux5760But.Checked = false;
                    aux10GBut.Checked = false;
                    aux24GBut.Checked = false;
                    aux50But.Checked = false;
                    aux144But.Checked = false;
                    aux222But.Checked = false;
                    aux432But.Checked = false;
                    aux903But.Checked = false;
                    aux1296But.Checked = false;
                    mainHFBut.Checked = false;
                    Globals.auxMicro = false;
                    Globals.auxHF = true;
                    Globals.mainHF = false;
                    _HF.Size = Globals.formlargesize;
                    _HF.Location = Globals.formauxloc;
                    _HF.pictureBox1.Size = Globals.boxlargesize;
                    _HF.pictureBox1.Location = Globals.box1largeloc;
                    _HF.UpdateGraphs();  //w3sz
                    _HF.AdjustFilterByMode();
                    _HF.Sidetone.Checked = Globals.sidetonechecked;
                    Globals.auxradio = "HF";
                    SwitchAuxBand(Globals.auxradio);
                    digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
                    sendCommandAsync(digiAuxOnString, Properties.Settings.Default.w3szendpointHF, 8007);
                    SwitchMicAsync();
                    SwitchCWAsync();
                    SwitchAudioAsync("LRON", udpbyteon, Globals.w3szendpoint2, Globals.rcvaudiocommandport2);

                    //          
                }
                else if (auxHFBut.Checked == false)
                {
                    SwitchBand("HF");
                    SwitchAudioAsync("LRON", udpbyteoff, tempendpoint, temprcvdmcport);
                    moxoffAsync(tempendpoint, temprcvdmcport, true);
                    if (mainHFBut.Checked == false)
                    {
                        Globals.auxHF = false;
                        Globals.mainHF = false;
                        _HF.Size = Globals.formsmallsize;
                        _HF.Location = Globals.formHFsmallloc;
                        _HF.pictureBox1.Size = Globals.boxsmallsize;
                        _HF.pictureBox1.Location = Globals.box1smallloc;
                        switch (_HF.BandSelect.Text)
                        {
                            case "160m":
                                _HF.Preamp_160 = _HF.Preamp.Checked;
                                break;
                            case "80m":
                                _HF.Preamp_80 = _HF.Preamp.Checked;
                                break;
                            case "40m":
                                _HF.Preamp_40 = _HF.Preamp.Checked;
                                break;
                            case "30m":
                                _HF.Preamp_30 = _HF.Preamp.Checked;
                                break;
                            case "20m":
                                _HF.Preamp_20 = _HF.Preamp.Checked;
                                break;
                            case "17m":
                                _HF.Preamp_17 = _HF.Preamp.Checked;
                                break;
                            case "15m":
                                _HF.Preamp_15 = _HF.Preamp.Checked;
                                break;
                            case "12m":
                                _HF.Preamp_12 = _HF.Preamp.Checked;
                                break;
                            case "10m":
                                _HF.Preamp_10 = _HF.Preamp.Checked;
                                break;
                            case "6m":
                                _HF.Preamp_6 = _HF.Preamp.Checked;
                                break;

                        }
                    }
                }
            }
        }

        //this switches band for Main Radio and resets global parameters
        public void SwitchMainBand(string band)
        {
            switch (band)
            {
                case "GHz":
                    Globals.w3szendpoint = MicroMHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8000;
                    break;

                case "50":
                    Globals.w3szendpoint = _50MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8001;
                    break;

                case "144":
                    Globals.w3szendpoint = _144MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8002;
                    break;

                case "222":
                    Globals.w3szendpoint = _222MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8003;
                    break;

                case "432":
                    Globals.w3szendpoint = _432MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8004;
                    break;

                case "903":
                    Globals.w3szendpoint = _903MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8005;
                    break;

                case "1296":
                    Globals.w3szendpoint = _1296MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8006;
                    break;

                case "HF":
                    Globals.w3szendpoint = _HF.w3szendpoint1;
                    Globals.rcvaudiocommandport = 8007;
                    break;
                default:
                    break;
            }
        }

        //this switches band for Aux Radio and resets global parameters
        public void SwitchAuxBand(string band)
        {
            switch (band)
            {
                case "GHz":
                    Globals.w3szendpoint2 = MicroMHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8000;
                    break;

                case "50":
                    Globals.w3szendpoint2 = _50MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8001;
                    break;

                case "144":
                    Globals.w3szendpoint2 = _144MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8002;
                    break;

                case "222":
                    Globals.w3szendpoint2 = _222MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8003;
                    break;

                case "432":
                    Globals.w3szendpoint2 = _432MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8004;
                    break;

                case "903":
                    Globals.w3szendpoint2 = _903MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8005;
                    break;

                case "1296":
                    Globals.w3szendpoint2 = _1296MHz.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8006;
                    break;

                case "HF":
                    Globals.w3szendpoint2 = _HF.w3szendpoint1;
                    Globals.rcvaudiocommandport2 = 8007;
                    break;
                default:
                    break;
            }


        }


        string tempendpoint;
        static int temprcvdmcport;

        //this switches band without changing global parameters.  Used when deselecting radio.
        public void SwitchBand(string band)
        {
            switch (band)
            {
                case "GHz":
                    tempendpoint = MicroMHz.w3szendpoint1;
                    temprcvdmcport = 8000;
                    break;

                case "50":
                    tempendpoint = _50MHz.w3szendpoint1;
                    temprcvdmcport = 8001;
                    break;

                case "144":
                    tempendpoint = _144MHz.w3szendpoint1;
                    temprcvdmcport = 8002;
                    break;

                case "222":
                    tempendpoint = _222MHz.w3szendpoint1;
                    temprcvdmcport = 8003;
                    break;

                case "432":
                    tempendpoint = _432MHz.w3szendpoint1;
                    temprcvdmcport = 8004;
                    break;

                case "903":
                    tempendpoint = _903MHz.w3szendpoint1;
                    temprcvdmcport = 8005;
                    break;

                case "1296":
                    tempendpoint = _1296MHz.w3szendpoint1;
                    temprcvdmcport = 8006;
                    break;

                case "HF":
                    tempendpoint = _HF.w3szendpoint1;
                    temprcvdmcport = 8007;
                    break;
                default:
                    break;
            }


        }

        //this turns Main Radio or Aux Radio audio on or off as radio is selected or deselected
        public void SwitchAudio(string RadioCode, string udpbyte, string w3szendpoint, int rcvaudiocommandport)
        {
            byte[] RcvOfftextstrng = StrToByteArray(RadioCode + udpbyte); //w3sz  convert to byte array
            int RcvOffstrngsize = RcvOfftextstrng.Length;  //w3sz  get byte array size

            //this is UI command sender; sends commands back to radio //w3sz
            UdpClient udpServer4 = new UdpClient(); //w3sz
            IPEndPoint localpt4 = new IPEndPoint(IPAddress.Parse(w3szendpoint), rcvaudiocommandport);  //w3sz  command 

            udpServer4.ExclusiveAddressUse = false;  //w3sz
            udpServer4.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
            try
            {
                udpServer4.Client.Connect(localpt4);  //w3sz changed was client.bind Jan 2024
            }
            catch
            {
                //  Console.Writeline("Error setting up udpServer4, receive audio command server, port 8007");
            }

            udpServer4.Send(RcvOfftextstrng, RcvOffstrngsize, localpt4); // reply back  //w3sz  
            udpServer4.Close(); //w3sz  
        }


        public async Task SwitchAudioAsync(string RadioCode, string udpbyte, string w3szendpoint, int rcvaudiocommandport)
        {
            Task task = Task.Run(() => SwitchAudio(RadioCode, udpbyte, w3szendpoint, rcvaudiocommandport));
            await task;
        }


        //this switches mic data to appropriate IP addresses for Main and Aux Radios
        void SwitchMic()
        {
            if ((micMainChk.Checked || TxN1MMChkBox.Checked || TxDigiChkBox.Checked) & (Globals.w3szendpoint != string.Empty))
            {
                udpServer7 = new UdpClient(); //w3sz
                localpt7 = new IPEndPoint(IPAddress.Parse(Globals.w3szendpoint), 11000);  //w3sz  command
                udpServer7.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz mic audio sender
                udpServer7.ExclusiveAddressUse = false;  //w3sz
                try
                {
                    udpServer7.Client.Connect(localpt7);  //w3sz changed was client.bind Jan 2024
                    //  Console.Writeline("Success setting up main udpServer7, mic audio server, port 11000");
                }
                catch
                {
                    //  Console.Writeline("Error setting up main udpServer7, mic audio server, port 11000");
                }
            }

            else if ((micMainChk.Checked) & (Globals.w3szendpoint == string.Empty))
            {
                micMainChk.Checked = false;
                micAuxChk.Checked = true;
            }
            if ((micAuxChk.Checked || TxDigiChkBoxAux.Checked || TxN1MMChkBoxAux.Checked) & (Globals.w3szendpoint2 != string.Empty))
            {
                udpServer18 = new UdpClient(); //w3sz
                localpt18 = new IPEndPoint(IPAddress.Parse(Globals.w3szendpoint2), 11000);  //w3sz  command
                udpServer18.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz mic audio sender
                udpServer18.ExclusiveAddressUse = false;  //w3sz
                try
                {
                    udpServer18.Client.Connect(localpt18);  //w3sz changed was client.bind Jan 2024
                    //  Console.Writeline("Success setting up aux udpServer18, mic audio server, port 11000");
                }
                catch
                {
                    //  Console.Writeline("Error setting up aux udpServer18, mic audio server, port 11000");
                }
            }
            else if ((micAuxChk.Checked) & (Globals.w3szendpoint2 == string.Empty))
            {
                micAuxChk.Checked = false;
                micMainChk.Checked = true;
            }
        }

        public async Task SwitchMicAsync()
        {
            Task task = Task.Run(() => SwitchMic());
            await task;
        }

        //this switches CW key between Main and Aux Radios
        void SwitchCW()
        {
            if ((KeyMainBut.Checked) & (Globals.w3szendpoint != string.Empty))
            {
                udpServer8 = new UdpClient(); //w3sz
                localpt8 = new IPEndPoint(IPAddress.Parse(Globals.w3szendpoint), 12000);  //w3sz  command
                udpServer8.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz mic audio sender
                udpServer8.ExclusiveAddressUse = false;  //w3sz

                try
                {
                    udpServer8.Client.Connect(localpt8);  //w3sz changed was client.bind Jan 2024
                }

                catch
                {
                    //  Console.Writeline("Error setting up main udpServer8, cw key server, port 12000");
                }
            }
            else if ((KeyMainBut.Checked) & (Globals.w3szendpoint == string.Empty))
            {
                try
                {
                    Invoke((MethodInvoker)delegate
                    {
                        KeyMainBut.Checked = false;
                        KeyAuxBut.Checked = true;
                    });
                }
                catch { }//  Console.Writeline("Error in Form1 in SwitchCW, Line 7292"); }
            }
            else if ((KeyAuxBut.Checked) & (Globals.w3szendpoint2 != string.Empty))
            {
                udpServer8 = new UdpClient(); //w3sz
                localpt8 = new IPEndPoint(IPAddress.Parse(Globals.w3szendpoint2), 12000);  //w3sz  command
                udpServer8.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz mic audio sender
                udpServer8.ExclusiveAddressUse = false;  //w3sz

                try
                {
                    udpServer8.Client.Connect(localpt8);  //w3sz changed was client.bind Jan 2024
                }

                catch
                {
                    //  Console.Writeline("Error setting up aux udpServer8, cw key server, port 12000");
                }
            }
            else if ((KeyAuxBut.Checked) & (Globals.w3szendpoint2 == string.Empty))
            {
                try
                {
                    Invoke((MethodInvoker)delegate
                    {
                        KeyAuxBut.Checked = false;
                        KeyMainBut.Checked = true;
                    });
                }
                catch { } //  Console.Writeline("Error in Form1 in SwitchCW, Line 7321"); }
            }
        }

        public async Task SwitchCWAsync()
        {
            Task task = Task.Run(() => SwitchCW());
            await task;
        }


        //methods below handle changes in Main and Aux RadioButtons for Mic, Key, Audio

        private void micAuxChk_CheckedChanged(object sender, EventArgs e)
        {
            if (micAuxChk.Checked == true)
            {
                Properties.Settings.Default.mainMicOn = false;
                Properties.Settings.Default.auxMicOn = true;
                Properties.Settings.Default.Save();


                Properties.Settings.Default.Save();

                byte[] auxMictextstrng = StrToByteArray("AMIC" + Properties.Settings.Default.auxMicOn); //w3sz  convert to byte array
                                                                                                          //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.GHzChecked)
                {

                    sendCommandAsync(auxMictextstrng, MicroMHz.w3szendpoint1, 8000);
                }

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.Checked50)
                {
                    sendCommandAsync(auxMictextstrng, _50MHz.w3szendpoint1, 8001);

                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked144)
                {
                    sendCommandAsync(auxMictextstrng, _144MHz.w3szendpoint1, 8002);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked222)
                {
                    sendCommandAsync(auxMictextstrng, _222MHz.w3szendpoint1, 8003);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked432)
                {
                    sendCommandAsync(auxMictextstrng, _432MHz.w3szendpoint1, 8004);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked903)
                {
                    sendCommandAsync(auxMictextstrng, _903MHz.w3szendpoint1, 8005);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked1296)
                {
                    sendCommandAsync(auxMictextstrng, _1296MHz.w3szendpoint1, 8006);
                }

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.HFChecked)
                {
                    sendCommandAsync(auxMictextstrng, _HF.w3szendpoint1, 8007);
                }

                micMainChk.Checked = false;
                TxDigiChkBoxAux.Checked = false;
                TxN1MMChkBoxAux.Checked = false;
                MicReset_Click(this, EventArgs.Empty);
                SwitchMicAsync();
            }
            StartMonAux();
        }

        private void micMainChk_CheckedChanged(object sender, EventArgs e)
        {
            if (micMainChk.Checked == true)
            {
                Properties.Settings.Default.mainMicOn = true;
                Properties.Settings.Default.auxMicOn = false;
                Properties.Settings.Default.Save();


                byte[] mainMictextstrng = StrToByteArray("MMIC" + Properties.Settings.Default.mainMicOn); //w3sz  convert to byte array
                                                                                                        //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.GHzChecked)
                {

                    sendCommandAsync(mainMictextstrng, MicroMHz.w3szendpoint1, 8000);
                }

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.Checked50)
                {
                    sendCommandAsync(mainMictextstrng, _50MHz.w3szendpoint1, 8001);

                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked144)
                {
                    sendCommandAsync(mainMictextstrng, _144MHz.w3szendpoint1, 8002);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked222)
                {
                    sendCommandAsync(mainMictextstrng, _222MHz.w3szendpoint1, 8003);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked432)
                {
                    sendCommandAsync(mainMictextstrng, _432MHz.w3szendpoint1, 8004);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked903)
                {
                    sendCommandAsync(mainMictextstrng, _903MHz.w3szendpoint1, 8005);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked1296)
                {
                    sendCommandAsync(mainMictextstrng, _1296MHz.w3szendpoint1, 8006);
                }

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.HFChecked)
                {
                    sendCommandAsync(mainMictextstrng, _HF.w3szendpoint1, 8007);
                }

                micAuxChk.Checked = false;
                TxDigiChkBox.Checked = false;
                TxN1MMChkBox.Checked = false;
                MicReset_Click(this, EventArgs.Empty);
                SwitchMicAsync();
            }
            StartMon();

        }

        private void KeyMainBut_CheckedChanged(object sender, EventArgs e)
        {
            SwitchCWAsync();  //used if sending CW keying commands over ethernet          
        }


        public static byte[] StrToByteArray(string str)
        {
            System.Text.UTF8Encoding encoding = new System.Text.UTF8Encoding();
            return encoding.GetBytes(str);
        }



        public async Task sendCommandAsync(byte[] command, string endpoint, int port)
        {
            Task task = Task.Run(() => sendCommand(command, endpoint, port));
            await task;
        }

        public void sendCommand(byte[] command, string endpoint, int port)
        {
            int commandLength = command.Length;
            //this is UI command sender; sends commands back to radio //w3sz

            UdpClient udpServer4 = new UdpClient(); //w3sz
            IPEndPoint localpt4 = new IPEndPoint(IPAddress.Parse(endpoint), port);  //w3sz  command

            udpServer4.ExclusiveAddressUse = false;  //w3sz
            udpServer4.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
            try
            {
                udpServer4.Client.Connect(localpt4);  //w3sz changed was client.bind Jan 2024
            }
            catch
            {
            }
            udpServer4.Send(command, commandLength, localpt4); // reply back  //w3sz 
            udpServer4.Close(); //w3sz   
        }



        private void setupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            gotoSetup();

            if (Setup_form2.CWPitchChanged == true)
            {
                Micro.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                _50.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                _144.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                _222.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                _432.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                _1296.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                //             if (Properties.Settings.Default.HFChecked)
                {
                    _HF.BandwidthTrackBar_Scroll(sender, EventArgs.Empty);
                }
                Setup_form2.CWPitchChanged = false;
                StopAudio();
                MicAudioStart();
                StartAudio();

                byte[] CWPitchtextstrng = StrToByteArray("CWPI" + Globals.CWPitch.ToString()); //w3sz  convert to byte array

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.GHzChecked)
                {

                    sendCommandAsync(CWPitchtextstrng, MicroMHz.w3szendpoint1, 8000);
                }

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.Checked50)
                {
                    sendCommandAsync(CWPitchtextstrng, _50MHz.w3szendpoint1, 8001);

                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked144)
                {
                    sendCommandAsync(CWPitchtextstrng, _144MHz.w3szendpoint1, 8002);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked222)
                {
                    sendCommandAsync(CWPitchtextstrng, _222MHz.w3szendpoint1, 8003);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked432)
                {
                    sendCommandAsync(CWPitchtextstrng, _432MHz.w3szendpoint1, 8004);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked903)
                {
                    sendCommandAsync(CWPitchtextstrng, _903MHz.w3szendpoint1, 8005);
                }


                //this is UI command sender; sends commands back to radio //w3sz
                if (Properties.Settings.Default.Checked1296)
                {
                    sendCommandAsync(CWPitchtextstrng, _1296MHz.w3szendpoint1, 8006);
                }

                //this is UI command sender; sends commands back to radio //w3sz

                if (Properties.Settings.Default.HFChecked)
                {
                    sendCommandAsync(CWPitchtextstrng, _HF.w3szendpoint1, 8007);
                }
            }
        }


        public bool SetupForm2Valid()
        {
            // if the form is null or disposed, it is NOT valid
            if (Setup_form2 == null || Setup_form2.IsDisposed)
                return false;
            else
                return true;
        }

        // Sends microwave band data to N3FTI band controller using parallel port


        public async Task SendBandDataAsync()
        {
            Task task = Task.Run(() => SendBandData());
            await task;
        }

        public void SendBandData()
        {

            //        A B C D
            //  PIN   2 7 8 9
            // BAND  
            // 2304   0 1 1 0   00000110  96
            // 3456   1 1 1 0   10000110  97
            // 5760   0 0 0 1   00000001  128
            // 10368  1 0 0 1   10000001  129
            // 24192  0 1 0 1   00000101  160

            if (MicroMHz.EnableBandData)
            {
                int dataout;
                switch (Globals.microband)
                {
                    case "2304":
                        {
                            dataout = 96;
                            break;
                        }
                    case "3456":
                        {
                            dataout = 97;
                            break;
                        }
                    case "5760":
                        {
                            dataout = 128;
                            break;
                        }
                    case "10368":
                        {
                            dataout = 129;
                            break;
                        }
                    case "24192":
                        {
                            dataout = 160;
                            break;
                        }
                    default:
                        {
                            dataout = 0;
                            break;
                        }

                }

                PortControl.Output(MicroMHz.LPT_Addr, dataout);
            }
            else
            {
                byte[] MWBandtextstrng = MicroMHz.StrToByteArray("MWBA" + Globals.microband); //w3sz  convert to byte array
                int MWBandstrngsize = MWBandtextstrng.Length;  //w3sz  get byte array size

                //this is UI command sender; sends commands back to radio //w3sz  sends microwave band info back to server for remote LPT band switching
                UdpClient udpServer4 = new UdpClient(); //w3sz
                IPEndPoint localpt4 = new IPEndPoint(IPAddress.Parse(MicroMHz.w3szendpoint1), 8000);  //w3sz  command

                udpServer4.ExclusiveAddressUse = false;  //w3sz
                udpServer4.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer4.Client.Connect(localpt4);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer4.Send(MWBandtextstrng, MWBandstrngsize, localpt4); // reply back  //w3sz  
                udpServer4.Close(); //w3sz 

            }
        }

        // Below is Sound section
        public SimpleMixer soundSource;  //receive audio for speakers, headphones..
        public SimpleMixer soundSource2;  //main receive audio for digital WSJT etc
        public SimpleMixer soundSource4;  //aux receive audio for digital WSJT etc
        public SimpleMixer soundSource3;  //receive audio for N1MM+

        //following two devices are for monitoring tx audio output from main and aux receivers
        public CSCore.SoundIn.WasapiCapture micCapture;
        public CSCore.SoundIn.WasapiCapture digiCapture;
        public CSCore.SoundIn.WasapiCapture N1MMCapture;
        //Tx Monitor for Main Radio
        public static CSCore.SoundOut.WasapiOut micMonOut;
        DSPGain gain;
        WasapiCapture capture;
        SoundInSource source;
        //Tx Monitor for Aux Radio
        public static CSCore.SoundOut.WasapiOut micMonOut2;
        DSPGain gain2;
        WasapiCapture capture2;
        SoundInSource source2;

        public void StartMon()  //start Main and Aux Radio Tx Audio Monitors
        {
            //for Main Radio
            try
            {
                capture.Dispose();
            }
            catch
            {

            }

            try
            {
                source.Dispose();
            }
            catch
            {

            }

            try
            {
                micMonOut.Dispose();
            }
            catch
            {

            }

            if (TxDigiChkBox.Checked || TxN1MMChkBox.Checked || micMainChk.Checked)
            {
                waveinformat = new WaveFormatExtensible(48000, 16, 1, AudioSubTypes.Pcm);
                capture = new CSCore.SoundIn.WasapiCapture(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50, waveinformat);

                try
                {
                    if (TxDigiChkBox.Checked)  //Uses chosen Digital WaveIn device to bring WSJT or other output to this program's mic in
                    {
                        capture.Device = TxDigidevices[DigiWaveInIndex];
                        //  Console.Writeline("Changed MainTx Monitor Device to Digital Device: " + DigiWaveInIndex);
                    }

                    else if (TxN1MMChkBox.Checked)
                    {
                        capture.Device = TxN1MMdevices[N1MMWaveInIndex];
                        //  Console.Writeline("Changed Main Tx Monitor Device to N1MM Device: " + N1MMWaveInIndex);
                    }
                    else if (micMainChk.Checked)  //Uses chosen WaveIn device to bring mic input to this program's mic in
                    {
                        capture.Device = micdevices[waveInIndex];
                        //  Console.Writeline("Changed Main Tx Monitor Device to Device: " + waveInIndex);
                    }
                }
                catch
                {
                    if (TxDigiChkBox.Checked)
                    {
                        capture.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                        //  Console.Writeline("Main Tx Monitor Digi Device change unsuccessful. DigiWaveInIndex = " + DigiWaveInIndex);
                    }
                    else if (TxN1MMChkBox.Checked)
                    {
                        capture.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                        //  Console.Writeline("Main Tx Monitor N1MM Device change unsuccessful. N1MMWaveInIndex = " + N1MMWaveInIndex);
                    }
                    else if (micMainChk.Checked)
                    {
                        capture.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                        //  Console.Writeline("Main Tx Monitor Mic_In Device change unsuccessful. waveInIndex = " + waveInIndex);
                    }
                }

                capture.Initialize();
                capture.Start();

                source = new CSCore.Streams.SoundInSource(capture) { FillWithZeros = true };
                micMonOut = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 5);

                gain = new DSPGain(source.ToSampleSource());
                gain.GainDB = 5;

                micMonOut.Initialize(gain.ToWaveSource(16));
                micMonOut.Play();
            }
            try
            {
                micMonOut.Volume = 0.0f;
            }
            catch { }
        }

        public void StartMonAux()  //start Main and Aux Radio Tx Audio Monitors
        {
            //---------------------for Aux Radio
            try
            {
                capture2.Dispose();
            }
            catch
            {

            }

            try
            {
                source2.Dispose();
            }
            catch
            {

            }

            try
            {
                micMonOut2.Dispose();
            }
            catch
            {

            }

            if (TxDigiChkBoxAux.Checked || TxN1MMChkBoxAux.Checked || micAuxChk.Checked)
            {
                waveinformat2 = new WaveFormatExtensible(48000, 16, 1, AudioSubTypes.Pcm);
                capture2 = new CSCore.SoundIn.WasapiCapture(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50, waveinformat2);

                try
                {
                    if (TxDigiChkBoxAux.Checked)  //Uses chosen Digital WaveIn device to bring WSJT or other output to this program's mic in
                    {
                        capture2.Device = TxDigidevicesAux[DigiWaveInIndexAux];
                        //  Console.Writeline("Changed Aux Tx Monitor Device to Digital Device: " + DigiWaveInIndex);
                    }

                    else if (TxN1MMChkBoxAux.Checked)
                    {
                        capture2.Device = TxN1MMdevices[N1MMWaveInIndex];
                        //  Console.Writeline("Changed Aux Tx Monitor Device to N1MM Device: " + N1MMWaveInIndex);
                    }
                    else if (micAuxChk.Checked)  //Uses chosen WaveIn device to bring mic input to this program's mic in
                    {
                        capture2.Device = micdevices[waveInIndex];
                        //  Console.Writeline("Changed Aux Tx Monitor Device to Device: " + waveInIndex);
                    }
                }
                catch
                {
                    if (TxDigiChkBoxAux.Checked)
                    {
                        capture2.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                        //  Console.Writeline("Aux  Tx Monitor Digi Device change unsuccessful. DigiWaveInIndex = " + DigiWaveInIndex);
                    }
                    else if (TxN1MMChkBoxAux.Checked)
                    {
                        capture2.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                        //  Console.Writeline("Aux Tx Monitor N1MM Device change unsuccessful. N1MMWaveInIndex = " + N1MMWaveInIndex);
                    }
                    else if (micAuxChk.Checked)
                    {
                        capture2.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                        //  Console.Writeline("Aux Tx Monitor Mic_In Device change unsuccessful. waveInIndex = " + waveInIndex);
                    }
                }

                capture2.Initialize();
                capture2.Start();

                source2 = new CSCore.Streams.SoundInSource(capture2) { FillWithZeros = true };
                micMonOut2 = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 5);

                gain2 = new DSPGain(source2.ToSampleSource());
                gain2.GainDB = 5;

                micMonOut2.Initialize(gain2.ToWaveSource(16));
                micMonOut2.Play();
            }
            try
            {
                micMonOut2.Volume = 0.0f;
            }
            catch { }

        }

        //Receive Audio Section
        public void StartAudio()  //start receive audio
        {
            if (receiveProvider2 == null)
            {
                receiveProvider2 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                receiveProvider1 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                //receiveProviders 3 and 4 are for Digital modes
                receiveProvider3 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                receiveProvider4 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                //receiveProviders 5 and 6 are for N1MM
                receiveProvider5 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));
                receiveProvider6 = new WriteableBufferingSource(new CSCore.WaveFormat(48000, 16, 1));

                soundSource = new SimpleMixer(1, 48000)
                {
                    FillWithZeros = true,
                    DivideResult = true
                };

                soundSource.AddSource(
                    new SineGenerator48(Globals.CWPitch, 1.0, 0)
                    .AppendSource(x => new VolumeSource(x), out sidetone_volume)
                    );
                sidetone_volume.Volume = 0.0f;

                //add second soundsource, for Main Digital modes
                soundSource2 = new SimpleMixer(1, 48000)
                {
                    FillWithZeros = true,
                    DivideResult = true
                };

                soundSource2.AddSource(receiveProvider4.ToSampleSource().AppendSource(xb => new VolumeSource(xb), out digimain_volume)
                        );


                //add fourth soundsource, for Aux Digital modes
                soundSource4 = new SimpleMixer(1, 48000)
                {
                    FillWithZeros = true,
                    DivideResult = true
                };
                soundSource4.AddSource(receiveProvider3.ToSampleSource().AppendSource(xc => new VolumeSource(xc), out digiaux_volume)
                        );

      //w3sz          if (DigiChkBox.Checked)
                {
                    digimain_volume.Volume = 1.0f;
                }
    //            else
                {
    //                digimain_volume.Volume = 0.0f;
                }

  //              if (DigiChkBoxAux.Checked)
                {
                    digiaux_volume.Volume = 1.0f;
                }
 //               else
                {
    //                digiaux_volume.Volume = 0.0f;
                }

                //add third soundsource, for N1MM+, QSOrder
                soundSource3 = new SimpleMixer(1, 48000)
                {
                    FillWithZeros = true,
                    DivideResult = true
                };

                soundSource3.AddSource(receiveProvider6.ToSampleSource().AppendSource(x => new VolumeSource(x), out N1MMmain_volume)
                    );


                soundSource3.AddSource(receiveProvider5.ToSampleSource().AppendSource(x => new VolumeSource(x), out N1MMaux_volume)
                    );


                if (N1MMChkBox.Checked)
                {
                    N1MMmain_volume.Volume = 1.0f;
                }
                else
                {
                    N1MMmain_volume.Volume = 0.0f;
                }

                if (N1MMChkBoxAux.Checked)
                {
                    N1MMaux_volume.Volume = 1.0f;
                }
                else
                {
                    N1MMaux_volume.Volume = 0.0f;
                }

                //waveOut for sidetone
                SidetoneWaveOut = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50);
                SidetoneWaveOut.Device = RxAudioOut[waveOutIndex];
                SidetoneWaveOut.Initialize(soundSource.ToWaveSource());   //(16));
                SidetoneWaveOut.Volume = Globals.sidetonevol;
                SidetoneWaveOut.Play();

                //waveOut for Main Radio
                RxWaveOut = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50);
                RxWaveOut.Device = RxAudioOut[waveOutIndex];
                RxWaveOut.Initialize(receiveProvider2);

                if (!MainMuteBox.Checked)
                {
                    RxWaveOut.Volume = RcvVolume.Value / 100.0f;
                }
                else
                {
                    RxWaveOut.Volume = 0.0f;
                }

                RxWaveOut.Play();
                //  Console.Writeline("Main Sound initialized");
                //  Console.Writeline("Main soundOut Status = " + RxWaveOut.PlaybackState.ToString());

                //waveOut for Aux Radio
                RxWaveOut0 = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50);
                RxWaveOut0.Device = RxAudioOut[waveOutIndex];
                RxWaveOut0.Initialize(receiveProvider1);

                if (!AuxMuteBox.Checked)
                {
                    RxWaveOut0.Volume = RcvVolume.Value / 100.0f;
                }
                else
                {
                    RxWaveOut0.Volume = 0.0f;
                }

                RxWaveOut0.Play();
                //  Console.Writeline("Aux Sound initialized");
                //  Console.Writeline("Aux soundOut Status = " + RxWaveOut0.PlaybackState.ToString());

                //add second waveout, for Main Digital Modes
                RxWaveOut2 = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 500);
                RxWaveOut2.Device = RxDigAudioOut[DigiWaveOutIndex];
                RxWaveOut2.Initialize(soundSource2.ToWaveSource(16));
                RxWaveOut2.Volume = 1.0f;
                RxWaveOut2.Play();
                //  Console.Writeline("Sound2 Initialized");
                //  Console.Writeline("RxWaveOut2 Status = " + RxWaveOut2.PlaybackState.ToString());


                //add fourth waveout, for Aux Digital Modes
                RxWaveOut4 = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 500);
                RxWaveOut4.Device = RxDigAudioOutAux[DigiWaveOutIndexAux];
                RxWaveOut4.Initialize(soundSource4.ToWaveSource(16));
                RxWaveOut4.Volume = 1.0f;
                RxWaveOut4.Play();
                //  Console.Writeline("Sound2 Initialized");
                //  Console.Writeline("RxWaveOut2 Status = " + RxWaveOut2.PlaybackState.ToString());


                //add third waveout, for N1MM+, QSOrder
                RxWaveOut3 = new WasapiOut(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50);
                RxWaveOut3.Device = RxN1MMAudioOut[N1MMWaveOutIndex];
                RxWaveOut3.Initialize(soundSource3.ToWaveSource(16));
                RxWaveOut3.Volume = 1.0f;
                RxWaveOut3.Play();
                //  Console.Writeline("Sound3 Initialized");
                //  Console.Writeline("RxWaveOut3 Status = " + RxWaveOut3.PlaybackState.ToString());
            }

        }



        public async Task StopAudioStreamAsync()
        {
            Task task = Task.Run(() => StopAudioStream());
            await task;
            StopAudio();
            StartAudio();
        }

        //stop receive audio streams from Main and Liaison Radio Servers
        public void StopAudioStream()
        {

            ///beginning of receive audio server shutdown


            byte[] RcvOfftextstrng = StrToByteArray("LRON" + udpbyteoff); //w3sz  convert to byte array
            int RcvOffstrngsize = RcvOfftextstrng.Length;  //w3sz  get byte array size

            byte[] RcvOfftextstrng2 = StrToByteArray("MRON" + udpbyteoff); //w3sz  convert to byte array
            int RcvOffstrngsize2 = RcvOfftextstrng2.Length;  //w3sz  get byte array size

            if (Properties.Settings.Default.Checked50)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer24 = new UdpClient(); //w3sz
                IPEndPoint localpt24 = new IPEndPoint(IPAddress.Parse(_50MHz.w3szendpoint1), 8001);  //w3sz  command

                udpServer24.ExclusiveAddressUse = false;  //w3sz
                udpServer24.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3s  
                try
                {
                    udpServer24.Client.Connect(localpt24);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer24.Send(RcvOfftextstrng, RcvOffstrngsize, localpt24); // reply back  //w3sz     
                udpServer24.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt24); // reply back  //w3sz  
                udpServer24.Close(); //w3sz  
            }

            if (Properties.Settings.Default.Checked144)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer25 = new UdpClient(); //w3sz
                IPEndPoint localpt25 = new IPEndPoint(IPAddress.Parse(_144MHz.w3szendpoint1), 8002);  //w3sz  command
                udpServer25.ExclusiveAddressUse = false;  //w3sz
                udpServer25.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer25.Client.Connect(localpt25);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer25.Send(RcvOfftextstrng, RcvOffstrngsize, localpt25); // reply back  //w3sz     
                udpServer25.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt25); // reply back  //w3sz  
                udpServer25.Close(); //w3sz   
            }

            if (Properties.Settings.Default.Checked222)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer27 = new UdpClient(); //w3sz
                IPEndPoint localpt27 = new IPEndPoint(IPAddress.Parse(_222MHz.w3szendpoint1), 8003);  //w3sz  command
                udpServer27.ExclusiveAddressUse = false;  //w3sz
                udpServer27.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer27.Client.Connect(localpt27);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer27.Send(RcvOfftextstrng, RcvOffstrngsize, localpt27); // reply back  //w3sz     
                udpServer27.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt27); // reply back  //w3sz  
                udpServer27.Close(); //w3sz   
            }

            if (Properties.Settings.Default.Checked432)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer28 = new UdpClient(); //w3sz
                IPEndPoint localpt28 = new IPEndPoint(IPAddress.Parse(_432MHz.w3szendpoint1), 8004);  //w3sz  command
                udpServer28.ExclusiveAddressUse = false;  //w3sz
                udpServer28.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer28.Client.Connect(localpt28);  //w3sz  changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer28.Send(RcvOfftextstrng, RcvOffstrngsize, localpt28); // reply back  //w3sz     
                udpServer28.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt28); // reply back  //w3sz  
                udpServer28.Close(); //w3sz  
            }

            if (Properties.Settings.Default.Checked903)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer29 = new UdpClient(); //w3sz
                IPEndPoint localpt29 = new IPEndPoint(IPAddress.Parse(_903MHz.w3szendpoint1), 8005);  //w3sz  command
                udpServer29.ExclusiveAddressUse = false;  //w3sz
                udpServer29.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer29.Client.Connect(localpt29);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer29.Send(RcvOfftextstrng, RcvOffstrngsize, localpt29); // reply back  //w3sz       
                udpServer29.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt29); // reply back  //w3sz  
                udpServer29.Close(); //w3sz   
            }

            if (Properties.Settings.Default.Checked1296)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer30 = new UdpClient(); //w3sz
                IPEndPoint localpt30 = new IPEndPoint(IPAddress.Parse(_1296MHz.w3szendpoint1), 8006);  //w3sz  command
                udpServer30.ExclusiveAddressUse = false;  //w3sz
                udpServer30.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer30.Client.Connect(localpt30);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer30.Send(RcvOfftextstrng, RcvOffstrngsize, localpt30); // reply back  //w3sz       
                udpServer30.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt30); // reply back  //w3sz  
                udpServer30.Close(); //w3sz   
            }

            if (Properties.Settings.Default.GHzChecked)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer31 = new UdpClient(); //w3sz
                IPEndPoint localpt31 = new IPEndPoint(IPAddress.Parse(MicroMHz.w3szendpoint1), 8000);  //w3sz  command
                udpServer31.ExclusiveAddressUse = false;  //w3sz
                udpServer31.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer31.Client.Connect(localpt31);  //w3sz changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer31.Send(RcvOfftextstrng, RcvOffstrngsize, localpt31); // reply back  //w3sz       
                udpServer31.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt31); // reply back  //w3sz  
                udpServer31.Close(); //w3sz   
            }

            if (Properties.Settings.Default.HFChecked)
            {
                //this is UI command sender; sends commands back to radio //w3sz
                UdpClient udpServer32 = new UdpClient(); //w3sz
                IPEndPoint localpt32 = new IPEndPoint(IPAddress.Parse(_HF.w3szendpoint1), 8007);  //w3sz  command
                udpServer32.ExclusiveAddressUse = false;  //w3sz
                udpServer32.Client.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);  //w3sz
                try
                {
                    udpServer32.Client.Connect(localpt32);  //w3sz   changed was client.bind Jan 2024
                }
                catch
                {
                }
                udpServer32.Send(RcvOfftextstrng, RcvOffstrngsize, localpt32); // reply back  //w3sz     
                udpServer32.Send(RcvOfftextstrng2, RcvOffstrngsize2, localpt32); // reply back  //w3sz  
                udpServer32.Close(); //w3sz 
            }



            ///end of receive audio server shutdown
            ///
        }

        //stop receive audio
        public void StopAudio()
        {
            try { soundSource.RemoveSource(sidetone_volume); }
            catch { }
            try { soundSource2.RemoveSource(digimain_volume); }
            catch { }
            try { soundSource4.RemoveSource(digiaux_volume); }
            catch { }
            try { sidetone_volume.Dispose(); }
            catch { }
            try { digimain_volume.Dispose(); }
            catch { }
            try { digiaux_volume.Dispose(); }
            catch { }
            try { soundSource2.RemoveSource(receiveProvider4.ToSampleSource()); }
            catch { }
            try { soundSource4.RemoveSource(receiveProvider3.ToSampleSource()); }
            catch { }
            try { soundSource3.RemoveSource(receiveProvider6.ToSampleSource()); }
            catch { }
            try { soundSource3.RemoveSource(receiveProvider5.ToSampleSource()); }
            catch { }
            try
            {
                receiveProvider1.Dispose();
            }
            catch
            {

            }

            try
            {
                receiveProvider2.Dispose();
            }
            catch
            {

            }
            try
            {
                receiveProvider3.Dispose();
            }
            catch
            {

            }
            try
            {
                receiveProvider4.Dispose();
            }
            catch
            {

            }
            try
            {
                receiveProvider5.Dispose();
            }
            catch
            {

            }
            try
            {
                receiveProvider6.Dispose();
            }
            catch
            {

            }
            try
            {
                soundSource.Dispose();
            }
            catch
            {

            }
            try
            {
                soundSource2.Dispose();
            }
            catch
            {

            }
            try
            {
                soundSource3.Dispose();
            }
            catch
            {

            }
            try
            {
                soundSource4.Dispose();
            }
            catch
            {

            }
            try
            {
                SidetoneWaveOut.Stop();
            }
            catch
            {
                try
                {
                    RxWaveOut0.Stop();
                }
                catch
                {

                }
                try
                {
                    RxWaveOut.Stop();
                }
                catch
                {

                }
                try
                {
                    RxWaveOut2.Stop();
                }
                catch
                {

                }
                try
                {
                    RxWaveOut3.Stop();
                }
                catch
                {

                }
                try
                {
                    RxWaveOut4.Stop();
                }
                catch
                {

                }
                try
                {
                    SidetoneWaveOut.Dispose();
                }
                catch
                {

                }
            }
            try
            {
                RxWaveOut0.Dispose();
            }
            catch
            {

            }
            try
            {
                RxWaveOut.Dispose();
            }
            catch
            {

            }
            try
            {
                RxWaveOut2.Dispose();
            }
            catch
            {

            }
            try
            {
                RxWaveOut3.Dispose();
            }
            catch
            {
            }
            try
            {
                RxWaveOut4.Dispose();
            }
            catch
            {
            }
            try
            {
                if (soundSource != null)
                {
                    soundSource = null;
                }
            }
            catch
            {
            }
            try
            {
                if (receiveProvider2 != null)
                {
                    receiveProvider2 = null;
                }
            }
            catch
            {

            }
            try
            {
                if (receiveProvider1 != null)
                {
                    receiveProvider1 = null;
                }
            }
            catch
            {

            }
            try
            {
                if (receiveProvider4 != null)
                {
                    receiveProvider4 = null;
                }
            }
            catch
            {

            }
            try
            {
                if (receiveProvider3 != null)
                {
                    receiveProvider3 = null;
                }
            }
            catch
            {

            }
            try
            {
                if (receiveProvider6 != null)
                {
                    receiveProvider6 = null;
                }
            }
            catch
            {

            }
            try
            {
                if (receiveProvider5 != null)
                {
                    receiveProvider5 = null;
                }
            }
            catch
            {
            }
        }


        // mic input methods
        public WaveFormatExtensible waveinformat;  //Main Radio
        public WaveFormatExtensible waveinformat2;  //Aux Radio

        //Opus stuff 
        public static OpusDecoder _decoder;
        public static OpusDecoder _decoder2;

        //Opus stuff for Main Radio
        int _segmentFrames;
        int _bytesPerSegment;
        public static ulong _bytesSent;
        public static byte[] buff;
        int _len;

        //Opus stuff for Aux Radio
        int _segmentFrames2;
        int _bytesPerSegment2;
        public static ulong _bytesSent2;
        public static byte[] buffA;
        int _len2;


        // Lines below are for setting up Digital and Standard [non-digital] WaveIn and WaveOut devices
        public static CSCore.SoundIn.WasapiCapture mic_in; // WaveIn Main Radio
        public static CSCore.SoundIn.WasapiCapture mic_in2; // WaveIn Main Radio
        public static CSCore.SoundOut.WasapiOut SidetoneWaveOut; //WaveOut for Sidetone
        public static CSCore.SoundOut.WasapiOut RxWaveOut0; //WaveOut for Aux Radio
        public static CSCore.SoundOut.WasapiOut RxWaveOut; //WaveOut for Main Radio
        public static CSCore.SoundOut.WasapiOut RxWaveOut2; //main WaveOut for Digital Modes
        public static CSCore.SoundOut.WasapiOut RxWaveOut4; //aux WaveOut for Digital Modes
        public static CSCore.SoundOut.WasapiOut RxWaveOut3; //WaveOut for N1MM

        MMDeviceEnumerator deviceEnum;
        public static MMDeviceCollection micdevices;
        public static MMDeviceCollection RxAudioOut;
        public static int waveInIndex;
        public static int waveOutIndex;

        public static CSCore.DSP.ChannelMatrix channelMatrix_main = CSCore.DSP.ChannelMatrix.StereoToMonoMatrix;
        public static CSCore.DSP.ChannelMatrix channelMatrix_aux = CSCore.DSP.ChannelMatrix.StereoToMonoMatrix;

        public static MMDeviceCollection TxDigidevices;
        public static MMDeviceCollection TxDigidevicesAux;
        public static MMDeviceCollection RxDigAudioOut;
        public static MMDeviceCollection RxDigAudioOutAux;
        public static int DigiWaveInIndex;
        public static int DigiWaveOutIndex;
        public static int DigiWaveInIndexAux;
        public static int DigiWaveOutIndexAux;

        public static MMDeviceCollection TxN1MMdevices;
        public static MMDeviceCollection RxN1MMAudioOut;
        public static int N1MMWaveInIndex;
        public static int N1MMWaveOutIndex;
        public static int mic_in_chanNo = 1;
        public static int mic_in2_chanNo = 1;
        public static WaveFormat windowsFormat;
        public static WaveFormat windowsFormat2;
        public static int mic_in_SampleRate;
        public static int mic_in2_SampleRate;

        //This method sets up WaveIn device for mic or digital audio input.
        //It is activated by MicReset button on Form1, and is called whenever WaveIn device selections are changed.
        public void MicAudioStart()
        {
            MMDeviceEnumerator deviceEnum = new MMDeviceEnumerator();
            //dispose of mic_in if already running
            try
            {
                mic_in.Dispose();
            }
            catch
            {

            }
            try
            {
                mic_in2.Dispose();
            }
            catch
            {

            }

            //Opus Stuff for Main Radio
            _bytesSent = 0;
            _len = 0;
            _segmentFrames = 960;

            if (_encoder == null)
            {
                if (!TxDigiChkBox.Checked)
                {
                    if (Properties.Settings.Default.voiceCodecIsVoip)
                    {
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                    }
                    else
                    {
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                    }
                }
                else
                {
                    if (Properties.Settings.Default.dataCodecIsVoip)
                    {
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                    }
                    else
                    {
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                    }
                }
            }
            _encoder.ForwardErrorCorrection = true;
            _bytesPerSegment = _encoder.FrameByteCount(_segmentFrames);

            //Opus Stuff for Aux Radio
            _bytesSent2 = 0;
            _len2 = 0;
            _segmentFrames2 = 960;

            if (_encoder2 == null)
            {
                if (!TxDigiChkBoxAux.Checked)
                {
                    if (Properties.Settings.Default.txvoiceCodecIsVoip)
                    {
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                    }
                    else
                    {
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                    }
                }
                else
                {
                    if (Properties.Settings.Default.txdataCodecIsVoip)
                    {
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                    }
                    else
                    {
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                    }
                }
            }

            _encoder2.ForwardErrorCorrection = true;
            _bytesPerSegment2 = _encoder2.FrameByteCount(_segmentFrames2);


            //Mic In Stuff for Main Radio
            waveinformat = new WaveFormatExtensible(48000, 16, 1, AudioSubTypes.Pcm);
            mic_in = new CSCore.SoundIn.WasapiCapture(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50, waveinformat);
            //            mic_in.Latency = 50;
            try
            {
                if (TxDigiChkBox.Checked)  //Uses chosen Digital WaveIn device to bring WSJT or other output to this program's mic in
                {
                    mic_in.Device = TxDigidevices[DigiWaveInIndex];
                    //  Console.Writeline("Changed Mic_In Device to Digital Device: " + DigiWaveInIndex);
                }

                else if (TxN1MMChkBox.Checked)
                {
                    mic_in.Device = TxN1MMdevices[N1MMWaveInIndex];
                    //  Console.Writeline("Changed Mic_In Device to N1MM Device: " + N1MMWaveInIndex);
                }
                else if (micMainChk.Checked)  //Uses chosen WaveIn device to bring mic input to this program's mic in
                {
                    mic_in.Device = micdevices[waveInIndex];
                    windowsFormat = mic_in.Device.DeviceFormat;
                    mic_in_SampleRate = windowsFormat.SampleRate;
                    if (mic_in_SampleRate != 48000)
                    {
                        MessageBox.Show("The Mic Sample Rate set by Windows for " + mic_in.Device + " is " + mic_in_SampleRate + ".  The Sample Rate must be 48000.  Please exit this program and set the Mic Sample Rate to 48000 in Windows.");
                    }
                    mic_in_chanNo = windowsFormat.Channels; //AudioMeterInformation.FromDevice(mic_in.Device).MeteringChannelCount;
                    if (mic_in_chanNo == 2)
                    {
                        waveinformat = new WaveFormatExtensible(48000, 16, 2, AudioSubTypes.Pcm);
                        mic_in = new CSCore.SoundIn.WasapiCapture(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50, waveinformat);
                    }
                    //  Console.Writeline("mic_in number of channels = " + mic_in_chanNo);
                    //  Console.Writeline("Changed Mic_In Device to Device: " + waveInIndex);
                }
            }
            catch
            {
                if (TxDigiChkBox.Checked)
                {
                    //             mic_in.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                    //  Console.Writeline("Mic_In Device change unsuccessful. DigiWaveInIndex = " + DigiWaveInIndex);
                }
                else if (TxN1MMChkBox.Checked)
                {
                    //                  mic_in.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                    //  Console.Writeline("Mic_In Device change unsuccessful. N1MMWaveInIndex = " + N1MMWaveInIndex);
                }
                else if (micMainChk.Checked)
                {
                    //              mic_in.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                    //  Console.Writeline("Mic_In Device change unsuccessful. waveInIndex = " + waveInIndex);
                }
            }
            mic_in.Initialize();
            mic_in.DataAvailable += mic_in_DataAvailable;
            mic_data_queue = new ConcurrentQueue<byte[]>();

            //Mic In Stuff for Aux Radio

            waveinformat2 = new WaveFormatExtensible(48000, 16, 1, AudioSubTypes.Pcm);
            mic_in2 = new CSCore.SoundIn.WasapiCapture(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50, waveinformat2);

            try
            {
                if (TxDigiChkBoxAux.Checked)  //Uses chosen Digital WaveIn device to bring WSJT or other output to this program's mic in
                {
                    mic_in2.Device = TxDigidevicesAux[DigiWaveInIndexAux];
                    //  Console.Writeline("Changed Mic_In2 Device to Digital Device: " + DigiWaveInIndex);
                }

                else if (TxN1MMChkBoxAux.Checked)
                {
                    mic_in2.Device = TxN1MMdevices[N1MMWaveInIndex];
                    //  Console.Writeline("Changed Mic_In2 Device to N1MM Device: " + N1MMWaveInIndex);
                }
                else if (micAuxChk.Checked)  //Uses chosen WaveIn device to bring mic input to this program's mic in
                {
                    mic_in2.Device = micdevices[waveInIndex];
                    windowsFormat2 = mic_in2.Device.DeviceFormat;
                    mic_in2_SampleRate = windowsFormat2.SampleRate;
                    if (mic_in2_SampleRate != 48000)
                    {
                        MessageBox.Show("The Mic Sample Rate set by Windows is " + mic_in2_SampleRate + ".  The Sample Rate must be 48000.  Please exit this program and set the Mic Sample Rate to 48000 in Windows.");
                    }
                    mic_in2_chanNo = windowsFormat2.Channels; // AudioMeterInformation.FromDevice(mic_in2.Device).MeteringChannelCount;
                    if (mic_in2_chanNo == 2)
                    {
                        waveinformat2 = new WaveFormatExtensible(48000, 16, 2, AudioSubTypes.Pcm);
                        mic_in2 = new CSCore.SoundIn.WasapiCapture(false, CSCore.CoreAudioAPI.AudioClientShareMode.Shared, 50, waveinformat2);
                    }
                    //  Console.Writeline("mic_in2 number of channels = " + mic_in2_chanNo);
                    //  Console.Writeline("Changed Mic_In2 Device to Device: " + waveInIndex);
                }
            }
            catch
            {
                if (TxDigiChkBoxAux.Checked)
                {
                    //              mic_in2.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                    //  Console.Writeline("Mic_In2 Device change unsuccessful. DigiWaveInIndex = " + DigiWaveInIndex);
                }
                else if (TxN1MMChkBoxAux.Checked)
                {
                    //                  mic_in2.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                    //  Console.Writeline("Mic_In2 Device change unsuccessful. N1MMWaveInIndex = " + N1MMWaveInIndex);
                }
                else if (micAuxChk.Checked)
                {
                    //              mic_in2.Device = deviceEnum.GetDefaultAudioEndpoint(DataFlow.Capture, Role.Multimedia);
                    //  Console.Writeline("Mic_In2 Device change unsuccessful. waveInIndex = " + waveInIndex);
                }

            }
            mic_in2.Initialize();
            mic_in2.DataAvailable += mic_in_DataAvailable2;
            mic_data_queue2 = new ConcurrentQueue<byte[]>();
        }
        //---------------------------------------------------------

        // mic wavein callback
        public byte[] final_mic_bytes;
        byte[] _notEncodedBuffer = new byte[0];
        byte[] buff2;

        // for Wavein for Main Radio Tx
        public void mic_in_DataAvailable(object sender, CSCore.SoundIn.DataAvailableEventArgs e)
        {

            if (Globals.sendMic && e.ByteCount >0)
            {
                //     Console.WriteLine("Number of mic channels = {0}", mic_in2_chanNo);
                //     Console.WriteLine("byteCount= {0}", e.ByteCount);
               //      Console.WriteLine("Format = {0}", e.Format);
                if (mic_in_chanNo == 1 || !micMainChk.Checked)  //digital modes or 1 channel mic
                {
                    if (true) //Properties.Settings.Default.opusTxComp)
                    {
                        byte[] soundBuffer = new byte[e.ByteCount + _notEncodedBuffer.Length];
                        for (int i = 0; i < _notEncodedBuffer.Length; i++)
                            soundBuffer[i] = _notEncodedBuffer[i];
                        for (int i = 0; i < e.ByteCount; i++)
                            soundBuffer[i + _notEncodedBuffer.Length] = e.Data[i];

                        int byteCap = _bytesPerSegment;
                        int segmentCount = (int)Math.Floor((decimal)soundBuffer.Length / byteCap);
                        int segmentsEnd = segmentCount * byteCap;
                        int notEncodedCount = soundBuffer.Length - segmentsEnd;
                        _notEncodedBuffer = new byte[notEncodedCount];
                        for (int i = 0; i < notEncodedCount; i++)
                        {
                            _notEncodedBuffer[i] = soundBuffer[segmentsEnd + i];
                        }

                        for (int i = 0; i < segmentCount; i++)
                        {
                            byte[] segment = new byte[byteCap];
                            for (int j = 0; j < segment.Length; j++)
                                segment[j] = soundBuffer[(i * byteCap) + j];
                            int len;
                            buff = _encoder.Encode(segment, segment.Length, out len);
                            if (len > 0)
                            {
                                buff2 = new byte[len];
                                System.Buffer.BlockCopy(buff, 0, buff2, 0, len);
                                mic_data_queue.Enqueue(buff2);
                                //           //  Console.Writeline("Send Buffer size is {0}", buff2.Length);
                            }
                            _bytesSent += (ulong)len;
                        }
                    }
                    else
                    {
                        byte[] soundBuffer = new byte[e.ByteCount + _notEncodedBuffer.Length];
                        for (int i = 0; i < _notEncodedBuffer.Length; i++)
                            soundBuffer[i] = _notEncodedBuffer[i];
                        for (int i = 0; i < e.ByteCount; i++)
                            soundBuffer[i + _notEncodedBuffer.Length] = e.Data[i];

                        int byteCap = _bytesPerSegment;
                        int segmentCount = (int)Math.Floor((decimal)soundBuffer.Length / byteCap);
                        int segmentsEnd = segmentCount * byteCap;
                        int notEncodedCount = soundBuffer.Length - segmentsEnd;
                        _notEncodedBuffer = new byte[notEncodedCount];
                        for (int i = 0; i < notEncodedCount; i++)
                        {
                            _notEncodedBuffer[i] = soundBuffer[segmentsEnd + i];
                        }

                        for (int i = 0; i < segmentCount; i++)
                        {
                            byte[] segment = new byte[byteCap];
                            for (int j = 0; j < segment.Length; j++)
                                segment[j] = soundBuffer[(i * byteCap) + j];
                            int len;
                            buff = _encoder.Encode(segment, segment.Length, out len);
                            if (len > 0)
                            {
                                buff2 = new byte[len];
                                System.Buffer.BlockCopy(buff, 0, buff2, 0, len);
                                mic_data_queue.Enqueue(buff2);
                                //           //  Console.Writeline("Send Buffer size is {0}", buff2.Length);
                            }
                            _bytesSent += (ulong)len;
                        }
                        //      mic_data_queue.Enqueue(e.Data);
                        //        byte[] soundBuffer = new byte[e.ByteCount];
                        //        for (int i = 0; i < e.ByteCount; i++)
                        {
                  //          soundBuffer[i] = e.Data[i];
                    //        mic_data_queue.Enqueue(soundBuffer); //this is called when using digital modes
                        }
                    } 
                }

                else if (mic_in_chanNo == 2 && micMainChk.Checked) // 2 channel mic 
                {
                    byte[] monoData = new byte[e.ByteCount / 2];
                    for (int k = 0; k < e.ByteCount / 2; k += 2)
                    {
                        monoData[k] = e.Data[2 * k];
                        monoData[k + 1] = e.Data[2 * k + 1];
                    }

                    if (true) // Properties.Settings.Default.opusTxComp)
                    {
                        byte[] soundBuffer = new byte[e.ByteCount / 2 + _notEncodedBuffer.Length];
                        for (int i = 0; i < _notEncodedBuffer.Length; i++)
                            soundBuffer[i] = _notEncodedBuffer[i];
                        for (int i = 0; i < e.ByteCount / 2; i++)
                            soundBuffer[i + _notEncodedBuffer.Length] = monoData[i];



                        int byteCap = _bytesPerSegment;
                        int segmentCount = (int)Math.Floor((decimal)soundBuffer.Length / byteCap);
                        int segmentsEnd = segmentCount * byteCap;
                        int notEncodedCount = soundBuffer.Length - segmentsEnd;
                        _notEncodedBuffer = new byte[notEncodedCount];
                        for (int i = 0; i < notEncodedCount; i++)
                        {
                            _notEncodedBuffer[i] = soundBuffer[segmentsEnd + i];
                        }

                        for (int i = 0; i < segmentCount; i++)
                        {
                            byte[] segment = new byte[byteCap];
                            for (int j = 0; j < segment.Length; j++)
                                segment[j] = soundBuffer[(i * byteCap) + j];
                            int len;
                            buff = _encoder.Encode(segment, segment.Length, out len);
                            if (len > 0)
                            {
                                buff2 = new byte[len];
                                System.Buffer.BlockCopy(buff, 0, buff2, 0, len);
                                mic_data_queue.Enqueue(buff2);
                                //  Console.Writeline("Send Buffer size is {0}", buff2.Length);
                            }
                            _bytesSent += (ulong)len;
                        }
                    }
                    else
                    {
                        mic_data_queue.Enqueue(monoData);  //this is called when Plantronics USB Headset is used
                    }
                }
            }
        } 

        // mic2 wavein callback
        public byte[] final_mic_bytes2;
        byte[] _notEncodedBuffer2 = new byte[0];
        byte[] buff2A;

        // for Wavein for Aux Radio Tx
        public void mic_in_DataAvailable2(object sender, CSCore.SoundIn.DataAvailableEventArgs e)
        {
            if (Globals.sendMicAux && e.ByteCount>0)
            {

                if (mic_in2_chanNo == 1 || !micAuxChk.Checked)
                {
                    if (true)  // Properties.Settings.Default.opusTxComp)
                    {
                        byte[] soundBuffer2 = new byte[e.ByteCount + _notEncodedBuffer2.Length];
                        for (int i = 0; i < _notEncodedBuffer2.Length; i++)
                            soundBuffer2[i] = _notEncodedBuffer2[i];
                        for (int i = 0; i < e.ByteCount; i++)
                            soundBuffer2[i + _notEncodedBuffer2.Length] = e.Data[i];

                        int byteCap2 = _bytesPerSegment2;
                        int segmentCount2 = (int)Math.Floor((decimal)soundBuffer2.Length / byteCap2);
                        int segmentsEnd2 = segmentCount2 * byteCap2;
                        int notEncodedCount2 = soundBuffer2.Length - segmentsEnd2;
                        _notEncodedBuffer2 = new byte[notEncodedCount2];
                        for (int i = 0; i < notEncodedCount2; i++)
                        {
                            _notEncodedBuffer2[i] = soundBuffer2[segmentsEnd2 + i];
                        }

                        for (int i = 0; i < segmentCount2; i++)
                        {
                            byte[] segment2 = new byte[byteCap2];
                            for (int j = 0; j < segment2.Length; j++)
                                segment2[j] = soundBuffer2[(i * byteCap2) + j];
                            int len2;
                            buffA = _encoder2.Encode(segment2, segment2.Length, out len2);
                            if (len2 > 0)
                            {
                                buff2A = new byte[len2];
                                System.Buffer.BlockCopy(buffA, 0, buff2A, 0, len2);
                                mic_data_queue2.Enqueue(buff2A);
                                //           //  Console.Writeline("Send Buffer size is {0}", buff2A.Length);
                            }
                            _bytesSent2 += (ulong)len2;
                        }
                    }
                        else
                        {
                              byte[] soundBuffer2 = new byte[e.ByteCount];
                              for (int i = 0; i < e.ByteCount; i++)
                              {
                                  soundBuffer2[i] = e.Data[i];
                                  mic_data_queue.Enqueue(soundBuffer2);
                              }
                        } 
                }
                else if (mic_in_chanNo == 2 && micAuxChk.Checked)
                {
                    byte[] monoData2 = new byte[e.ByteCount / 2];
                    for (int k = 0; k < e.ByteCount / 2; k += 2)
                    {
                        monoData2[k] = e.Data[2 * k];
                        monoData2[k + 1] = e.Data[2 * k + 1];
                    }

                    if (true) // Properties.Settings.Default.opusTxComp)
                    {

                        byte[] soundBuffer2 = new byte[e.ByteCount / 2 + _notEncodedBuffer2.Length];
                        for (int i = 0; i < _notEncodedBuffer2.Length; i++)
                            soundBuffer2[i] = _notEncodedBuffer2[i];
                        for (int i = 0; i < e.ByteCount / 2; i++)
                            soundBuffer2[i + _notEncodedBuffer2.Length] = monoData2[i];

                        int byteCap2 = _bytesPerSegment2;
                        int segmentCount2 = (int)Math.Floor((decimal)soundBuffer2.Length / byteCap2);
                        int segmentsEnd2 = segmentCount2 * byteCap2;
                        int notEncodedCount2 = soundBuffer2.Length - segmentsEnd2;
                        _notEncodedBuffer2 = new byte[notEncodedCount2];
                        for (int i = 0; i < notEncodedCount2; i++)
                        {
                            _notEncodedBuffer2[i] = soundBuffer2[segmentsEnd2 + i];
                        }

                        for (int i = 0; i < segmentCount2; i++)
                        {
                            byte[] segment2 = new byte[byteCap2];
                            for (int j = 0; j < segment2.Length; j++)
                                segment2[j] = soundBuffer2[(i * byteCap2) + j];
                            int len2;
                            buffA = _encoder2.Encode(segment2, segment2.Length, out len2);
                            if (len2 > 0)
                            {
                                buff2A = new byte[len2];
                                System.Buffer.BlockCopy(buffA, 0, buff2A, 0, len2);
                                mic_data_queue2.Enqueue(buff2A);
                                //           //  Console.Writeline("Send Buffer size is {0}", buff2A.Length);
                            }
                            _bytesSent2 += (ulong)len2;
                        }
                    }
                    else
                    {
                        mic_data_queue.Enqueue(monoData2);
                     } 
                }
           }
            
        }

        private void MicReset_Click(object sender, EventArgs e)
        {
            MicAudioStart();
        }

        class PortControl // Import dll to project
        {
            [DllImport("inpout32.dll", EntryPoint = "Out32")]
            public static extern void Output(int address, int value); // decimal
        }

        public class SimpleMixer : ISampleSource
        {
            private readonly CSCore.WaveFormat _waveFormat;
            private readonly List<ISampleSource> _sampleSources = new List<ISampleSource>();
            private readonly object _lockObj = new object();
            private float[] _mixerBuffer;

            public bool FillWithZeros { get; set; }

            public bool DivideResult { get; set; }

            public SimpleMixer(int channelCount, int sampleRate)
            {
                if (channelCount < 1)
                    throw new ArgumentOutOfRangeException("channelCount");
                if (sampleRate < 1)
                    throw new ArgumentOutOfRangeException("sampleRate");

                _waveFormat = new CSCore.WaveFormat(sampleRate, 32, channelCount, AudioEncoding.IeeeFloat);
            }

            public void AddSource(ISampleSource source)
            {
                if (source == null)
                    throw new ArgumentNullException("source");

                if (source.WaveFormat.Channels != WaveFormat.Channels ||
                   source.WaveFormat.SampleRate != WaveFormat.SampleRate)
                    throw new ArgumentException("Invalid format.", "source");

                lock (_lockObj)
                {
                    if (!Contains(source))
                        _sampleSources.Add(source);
                }
            }

            public void RemoveSource(ISampleSource source)
            {
                //don't throw null ex here
                lock (_lockObj)
                {
                    if (Contains(source))
                        _sampleSources.Remove(source);
                }
            }

            public bool Contains(ISampleSource source)
            {
                if (source == null)
                    return false;
                return _sampleSources.Contains(source);
            }

            public int Read(float[] buffer, int offset, int count)
            {
                int numberOfStoredSamples = 0;

                try
                {
                    if (count > 0 && _sampleSources.Count > 0)
                    {
                        lock (_lockObj)
                        {
                            _mixerBuffer = _mixerBuffer.CheckBuffer(count);
                            List<int> numberOfReadSamples = new List<int>();
                            for (int m = _sampleSources.Count - 1; m >= 0; m--)
                            {
                                var sampleSource = _sampleSources[m];
                                int read = sampleSource.Read(_mixerBuffer, 0, count);
                                for (int i = offset, n = 0; n < read; i++, n++)
                                {
                                    if (numberOfStoredSamples < i)
                                        buffer[i] = _mixerBuffer[n];
                                    else
                                        buffer[i] += _mixerBuffer[n];
                                }
                                if (read > numberOfStoredSamples)
                                    numberOfStoredSamples = read;

                                if (read > 0)
                                    numberOfReadSamples.Add(read);
                                else
                                {
                                    //raise event here
                                    RemoveSource(sampleSource); //remove the input to make sure that the event gets only raised once.
                                }
                            }

                            if (DivideResult)
                            {
                                numberOfReadSamples.Sort();
                                int currentOffset = offset;
                                int remainingSources = numberOfReadSamples.Count;

                                foreach (var readSamples in numberOfReadSamples)
                                {
                                    if (remainingSources == 0)
                                        break;

                                    while (currentOffset < offset + readSamples)
                                    {
                                        buffer[currentOffset] /= remainingSources;
                                        buffer[currentOffset] = Math.Max(-1, Math.Min(1, buffer[currentOffset]));
                                        currentOffset++;
                                    }
                                    remainingSources--;
                                }
                            }
                        }
                    }

                    if (FillWithZeros && numberOfStoredSamples != count)
                    {
                        Array.Clear(
                            buffer,
                            Math.Max(offset + numberOfStoredSamples - 1, 0),
                            count - numberOfStoredSamples);

                        return count;
                    }

                }
                catch { }
                return numberOfStoredSamples;
            }

            public bool CanSeek { get { return false; } }

            public CSCore.WaveFormat WaveFormat
            {
                get { return _waveFormat; }
            }

            public long Position
            {
                get { return 0; }
                set
                {
                    throw new NotSupportedException();
                }
            }

            public long Length
            {
                get { return 0; }
            }

            public void Dispose()
            {
                lock (_lockObj)
                {
                    {
                        try
                        {
                            foreach (var sampleSource in _sampleSources)
                            {
                                if (sampleSource != null)
                                    sampleSource.Dispose();
                                _sampleSources.Remove(sampleSource);
                            }
                        }
                        catch
                        { }
                    }
                }
            }
        }


        // This method takes input from com port for CW keyer and footpedal MOX control
        // Software currently written for CW coming from WinKeyer, and does not use
        // separate dot and dash signals.

        //This is SEPARATE from the com port that communicates with the serial port on the WinKeyer

        bool man_PTT = false;  //added to debounce manual PTT switch; otherwise program hangs with switch bounce
                               // when in any voice mode, at mic_in.start in form1 or form1.mic_in.start in whichever radio
                               // has focus when manual PTT activated
        long PTT_time = DateTime.Now.Ticks;
        long PTT_time_old = DateTime.Now.Ticks;

        void Com_StatusChanged(object sender, SerialPinChangedEventArgs e)
        {
            string info = string.Empty;
            switch (e.EventType)
            {
                case SerialPinChange.CtsChanged:
                    if (port.CtsHolding == true)
                    {
                        Globals.CW_dot = true;
                    }
                    else if (port.CtsHolding == false)
                    {
                        Globals.CW_dot = false;
                    }
                    //info = string.Format("Clear to Send (CTS) signal changed state. CTS State = {0}", port.CtsHolding);   
                    break;
                case SerialPinChange.DsrChanged:
                    if (port.DsrHolding == true)
                    {
                        Globals.CW_dash = true;
                    }
                    else
                    {
                        Globals.CW_dash = false;
                    }
                    //info = string.Format("Data Set Ready (DSR) signal changed state. DSR State = {0}", port.DsrHolding);
                    break;
                case SerialPinChange.CDChanged:
                    //  Console.Writeline("PTT ACTIVATED: CD State = {0}", port.CDHolding);
                    PTT_time = DateTime.Now.Ticks;
                    if ((port.CDHolding == true) & (man_PTT == false) & ((PTT_time - PTT_time_old) > 1000000))
                    {
                        man_PTT = true;
                        //  Console.Writeline("Switch ON " + (PTT_time - PTT_time_old).ToString());
                        PTT_time_old = PTT_time;
                        // put radio into transmit mode
                        try
                        {
                            if (LFootMainBut.Checked == true)
                            {
                                SwitchBand(Globals.mainradio);
                                //     moxoffAsync(tempendpoint, temprcvdmcport, false);
                                moxoff(tempendpoint, temprcvdmcport, false);
                            }
                            else if (LFootAuxBut.Checked == true)
                            {
                                SwitchBand(Globals.auxradio);
                                //      moxoffAsync(tempendpoint, temprcvdmcport, false);
                                moxoff(tempendpoint, temprcvdmcport, false);
                            }
                        }
                        catch { }
                    }
                    else if ((port.CDHolding == false) & (man_PTT == true) & ((PTT_time - PTT_time_old) > 1000000))
                    {
                        man_PTT = false;
                        //  Console.Writeline("Switch OFF " + (PTT_time - PTT_time_old).ToString());
                        //  Console.Writeline();
                        PTT_time_old = PTT_time;
                        //put radio into receive mode
                        try
                        {
                            if (LFootMainBut.Checked == true)
                            {
                                SwitchBand(Globals.mainradio);
                                //              moxoffAsync(tempendpoint, temprcvdmcport, true);
                                moxoff(tempendpoint, temprcvdmcport, true);
                            }
                            else if (LFootAuxBut.Checked == true)
                            {
                                SwitchBand(Globals.auxradio);
                                //            moxoffAsync(tempendpoint, temprcvdmcport, true);
                                moxoff(tempendpoint, temprcvdmcport, true);
                            }
                        }
                        catch { }

                    }
                    //info = string.Format("Carrier Detect (CD) signal changed state. CD State = {0}", port.CDHolding);
                    break;
                default:
                    break;
            }
            ////  Console.Writeline(info);
        }


        // This method takes input from com port for Digital PTT (e.g. WSJT)        
        //This is SEPARATE from the com port that communicates with the serial port on the WinKeyer and com port that is used for hardware CW and PTT
        void Com_StatusChanged2(object sender, SerialPinChangedEventArgs e)
        {
            string info = string.Empty;
            switch (e.EventType)
            {
                case SerialPinChange.DsrChanged:
                    if (port2.DsrHolding == true)
                    {

                        // put radio into transmit
                        //  Console.Writeline("Transmit is ON");
                        if (TxDigiChkBox.Checked)
                        {
                            SwitchBand(Globals.mainradio);
                            //     moxoffAsync(tempendpoint, temprcvdmcport, false);
                            moxoff(tempendpoint, temprcvdmcport, false);
                        }
                    }
                    else if (port2.DsrHolding == false)
                    {
                        //put radio into receive mode
                        //  Console.Writeline("Receive is ON");
                        if (TxDigiChkBox.Checked)
                        {
                            SwitchBand(Globals.mainradio);
                            //    moxoffAsync(tempendpoint, temprcvdmcport, true);
                            moxoff(tempendpoint, temprcvdmcport, true);
                        }
                    }

                    //info = string.Format("Carrier Detect (CD) signal changed state. CD State = {0}", port.CDHolding);
                    break;
                default:
                    break;
            }
            ////  Console.Writeline(info);
        }


        // This method takes input from com port for N1MM Main PTT        
        //This is SEPARATE from the com port that communicates with the serial port on the WinKeyer and com port that is used for hardware CW and PTT and Digital COM Port
        void Com_StatusChanged3(object sender, SerialPinChangedEventArgs e)
        {
            string info = string.Empty;
            switch (e.EventType)
            {
                case SerialPinChange.DsrChanged:
                    if (port3.DsrHolding == true)
                    {
                                if (KeyAuxBut.Checked == true)
                                {
                                    break;
                                }
                        // put radio into transmit
                        //  Console.Writeline("Transmit is ON");
                        SwitchBand(Globals.mainradio);
                        //      moxoffAsync(tempendpoint, temprcvdmcport, false);
                        moxoff(tempendpoint, temprcvdmcport, false);
                    }
                    else if (port3.DsrHolding == false)
                    {
                        //Turn off CW 
                        Globals.CW_dash = false;
                        Globals.CW_dot = false;
                        //put radio into receive mode
                        //  Console.Writeline("Receive is ON");
                        SwitchBand(Globals.mainradio);
                            //     moxoffAsync(tempendpoint, temprcvdmcport, true);
                            moxoff(tempendpoint, temprcvdmcport, true);
                    }
                    //info = string.Format("Carrier Detect (CD) signal changed state. CD State = {0}", port.CDHolding);
                    break;
                default:
                    break;
            }
            ////  Console.Writeline(info);
        }


        // This method takes input from com port for N1MM Aux PTT        
        //This is SEPARATE from the com port that communicates with the serial port on the WinKeyer and com port that is used for hardware CW and PTT and Digital COM Port
        void Com_StatusChanged5(object sender, SerialPinChangedEventArgs e)
        {
            string info = string.Empty;
            switch (e.EventType)
            {
                case SerialPinChange.DsrChanged:
                    if (port5.DsrHolding == true)
                    { 
                                if (KeyMainBut.Checked == true)
                                {
                                    break;
                                }
                        // put radio into transmit
                        //  Console.Writeline("Transmit is ON");
                        SwitchBand(Globals.auxradio);
                        //    moxoffAsync(tempendpoint, temprcvdmcport, false);
                        moxoff(tempendpoint, temprcvdmcport, false);

                    }
                    else if (port5.DsrHolding == false)
                    {
                             //Turn off CW 
                            Globals.CW_dash = false;
                            Globals.CW_dot = false;
                            //put radio into receive mode
                            //  Console.Writeline("Receive is ON");
                            SwitchBand(Globals.auxradio);
                            //     moxoffAsync(tempendpoint, temprcvdmcport, true);
                            moxoff(tempendpoint, temprcvdmcport, true);
                    }
                    //info = string.Format("Carrier Detect (CD) signal changed state. CD State = {0}", port.CDHolding);
                    break;
                default:
                    break;
            }
            ////  Console.Writeline(info);
        }


        // This method takes input from com port for Aux Digital PTT (e.g. WSJT)        
        //This is SEPARATE from the com port that communicates with the serial port on the WinKeyer and com port that is used for hardware CW and PTT
        void Com_StatusChanged4(object sender, SerialPinChangedEventArgs e)
        {
            string info = string.Empty;
            switch (e.EventType)
            {
                case SerialPinChange.DsrChanged:
                    if (port4.DsrHolding == true)
                    {

                        // put radio into transmit
                        //  Console.Writeline("Transmit is ON");
                      
                        if (TxDigiChkBoxAux.Checked)
                        {
                            SwitchBand(Globals.auxradio);
                            //     moxoffAsync(tempendpoint, temprcvdmcport, false);
                            moxoff(tempendpoint, temprcvdmcport, false);
                        }

                    }
                    else if (port4.DsrHolding == false)
                    {
                        //put radio into receive mode
                        //  Console.Writeline("Receive is ON");

                        if (TxDigiChkBoxAux.Checked)
                        {
                            SwitchBand(Globals.auxradio);
                            //    moxoffAsync(tempendpoint, temprcvdmcport, true);
                            moxoff(tempendpoint, temprcvdmcport, true);
                        }

                    }

                    //info = string.Format("Carrier Detect (CD) signal changed state. CD State = {0}", port.CDHolding);
                    break;
                default:
                    break;
            }
            ////  Console.Writeline(info);
        }


        private void AudioReset_Click(object sender, EventArgs e)
        {
            StopAudioStreamAsync();
        }

        private void Form_Load(object sender, EventArgs e)
        {

            if (Properties.Settings.Default.PositionControllerBar != null)
            {
                Properties.Settings.Default.PositionControllerBar.Restore(this);
            }
        }

        private void DigiChkBox_CheckedChanged(object sender, EventArgs e)
        {
            if (DigiChkBox.Checked)
            {
                TxDigiChkBox.Checked = true;
                digimain_volume.Volume = 1.0f;
                MRBW.Text = bw2str(Properties.Settings.Default.dataBandwidth);
                MRBR.Text = Properties.Settings.Default.dataBitrate.ToString();
                if(Properties.Settings.Default.dataCodecIsVoip)
                {
                    MRType.Text = "Voip";
                }
                else
                {
                    MRType.Text = "Audio";
                }
            }
            else
            {
                digimain_volume.Volume = 1.0f;

                //         digimain_volume.Volume = 0.0f;
                TxDigiChkBox.Checked = false;
                MRBW.Text = bw2str(Properties.Settings.Default.voiceBandwidth);
                MRBR.Text = Properties.Settings.Default.voiceBitrate.ToString();
                if (Properties.Settings.Default.voiceCodecIsVoip)
                {
                    MRType.Text = "Voip";
                }
                else
                {
                    MRType.Text = "Audio";
                }
            }
            byte[] digiOnString = StrToByteArray("DiOn" + DigiChkBox.Checked.ToString());
            SwitchBand(Globals.mainradio);
            sendCommandAsync(digiOnString, tempendpoint, temprcvdmcport);
        }

        private void DigiChkBoxAux_CheckedChanged(object sender, EventArgs e)
        {
            if (DigiChkBoxAux.Checked)
            {
                TxDigiChkBoxAux.Checked = true;
                digiaux_volume.Volume = 1.0f;
                ARBW.Text = bw2str(Properties.Settings.Default.dataBandwidth);
                ARBR.Text = Properties.Settings.Default.dataBitrate.ToString();
                if (Properties.Settings.Default.dataCodecIsVoip)
                {
                    ARType.Text = "Voip";
                }
                else
                {
                    ARType.Text = "Audio";
                }
            }
            else
            {
                digiaux_volume.Volume = 1.0f;
                TxDigiChkBoxAux.Checked = false;
                //              digiaux_volume.Volume = 0.0f;
                ARBW.Text = bw2str(Properties.Settings.Default.voiceBandwidth);
                ARBR.Text = Properties.Settings.Default.voiceBitrate.ToString();
                if (Properties.Settings.Default.voiceCodecIsVoip)
                {
                    ARType.Text = "Voip";
                }
                else
                {
                    ARType.Text = "Audio";
                }
            }
            byte[] digiAuxOnString = StrToByteArray("DiOn" + DigiChkBoxAux.Checked.ToString());
            SwitchBand(Globals.auxradio);
            sendCommandAsync(digiAuxOnString, tempendpoint, temprcvdmcport);
        }

        private void TxDigiChkBox_CheckedChanged(object sender, EventArgs e)
        {
            if (TxDigiChkBox.Checked)
            {
                DigiChkBox.Checked = true;
                TxN1MMChkBox.Checked = false;
                micMainChk.Checked = false;
                Properties.Settings.Default.mainMicOn = false;
                MicReset_Click(this, EventArgs.Empty);
                SwitchMicAsync();
                if (Properties.Settings.Default.txdataCodecIsVoip != Properties.Settings.Default.txvoiceCodecIsVoip)
                {
                    if (Properties.Settings.Default.txdataCodecIsVoip)
                    {
                        if (_encoder != null)
                        {
                            _encoder.Dispose();
                        }
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                        MTType.Text = "Voip";
                    }
                    else
                    {
                        if (_encoder != null)
                        {
                            _encoder.Dispose();
                        }
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                        MTType.Text = "Audio";
                    }
                }
                _encoder.Bitrate = Properties.Settings.Default.txdataBitrate;
                _encoder.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                MTBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                MTBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);
            }
            else
            {
                if (Properties.Settings.Default.txdataCodecIsVoip != Properties.Settings.Default.txvoiceCodecIsVoip)
                {
                    if (Properties.Settings.Default.txvoiceCodecIsVoip)
                    {
                        if (_encoder != null)
                        {
                            _encoder.Dispose();
                        }
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                        MTType.Text = "Voip";
                    }
                    else
                    {
                        if (_encoder != null)
                        {
                            _encoder.Dispose();
                        }
                        _encoder = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                        MTType.Text = "Audio";
                    }
                }
                _encoder.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                _encoder.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                MTBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                MTBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);

            }
            StartMon();
        }


        private void TxDigiChkBoxAux_CheckedChanged(object sender, EventArgs e)
        {
            if (TxDigiChkBoxAux.Checked)
            {
                DigiChkBoxAux.Checked = true;
                TxN1MMChkBoxAux.Checked = false;
                micAuxChk.Checked = false;
                Properties.Settings.Default.auxMicOn = false;
                MicReset_Click(this, EventArgs.Empty);
                SwitchMicAsync();
                if (Properties.Settings.Default.txdataCodecIsVoip != Properties.Settings.Default.txvoiceCodecIsVoip)
                {
                    if (Properties.Settings.Default.txdataCodecIsVoip)
                    {
                        if (_encoder2 != null)
                        {
                            _encoder2.Dispose();
                        }
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                        ATType.Text = "Voip";
                    }
                    else
                    {
                        if (_encoder2 != null)
                        {
                            _encoder2.Dispose();
                        }
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                        ATType.Text = "Audio";
                    }
                }
                _encoder2.Bitrate = Properties.Settings.Default.txdataBitrate;
                _encoder2.Bandwidth = Properties.Settings.Default.txdataBandwidth;
                ATBR.Text = Properties.Settings.Default.txdataBitrate.ToString();
                ATBW.Text = bw2str(Properties.Settings.Default.txdataBandwidth);

            }
            else
            {
                if (Properties.Settings.Default.txdataCodecIsVoip != Properties.Settings.Default.txvoiceCodecIsVoip)
                {
                    if (Properties.Settings.Default.txvoiceCodecIsVoip)
                    {
                        if (_encoder2 != null)
                        {
                            _encoder2.Dispose();
                        }
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Voip);
                        _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                        ATType.Text = "Voip";
                    }
                    else
                    {
                        if (_encoder2 != null)
                        {
                            _encoder2.Dispose();
                        }
                        _encoder2 = OpusEncoder.Create(48000, 1, FragLabs.Audio.Codecs.Opus.Application.Audio);
                        _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                        _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                        ATType.Text = "Audio";
                    }
                }
                _encoder2.Bitrate = Properties.Settings.Default.txvoiceBitrate;
                _encoder2.Bandwidth = Properties.Settings.Default.txvoiceBandwidth;
                ATBR.Text = Properties.Settings.Default.txvoiceBitrate.ToString();
                ATBW.Text = bw2str(Properties.Settings.Default.txvoiceBandwidth);

            }
            StartMonAux();
        }


        private void N1MMChkBox_CheckedChanged(object sender, EventArgs e)
        {
            if (N1MMChkBox.Checked)
            {
                N1MMmain_volume.Volume = 1.0f;
            }
            else
            {
                N1MMmain_volume.Volume = 0.0f;
            }
        }

        private void N1MMChkBoxAux_CheckedChanged(object sender, EventArgs e)
        {
            if (N1MMChkBoxAux.Checked)
            {
                N1MMaux_volume.Volume = 1.0f;
            }
            else
            {
                N1MMaux_volume.Volume = 0.0f;
            }
        }

        private void TxN1MMChkBox_CheckedChanged(object sender, EventArgs e)
        {
            if (TxN1MMChkBox.Checked)
            {
                TxN1MMChkBoxAux.Checked = false;
                TxDigiChkBox.Checked = false;
                micMainChk.Checked = false;
                Properties.Settings.Default.mainMicOn = false;
                MicReset_Click(this, EventArgs.Empty);
                SwitchMicAsync();
            }
            StartMon();
        }


        private void TxN1MMChkBoxAux_CheckedChanged(object sender, EventArgs e)
        {
            if (TxN1MMChkBoxAux.Checked)
            {
                TxN1MMChkBox.Checked = false;
                TxDigiChkBoxAux.Checked = false;
                micAuxChk.Checked = false;
                Properties.Settings.Default.auxMicOn = false;
                MicReset_Click(this, EventArgs.Empty);
                SwitchMicAsync();
            }
            StartMonAux();
        }

        private void MainMuteBox_CheckedChanged(object sender, EventArgs e)
        {
            if (MainMuteBox.Checked)
            {
                RxWaveOut.Volume = 0.0f;
                if (!AuxMuteBox.Checked)
                {
                    RxWaveOut0.Volume = RcvVolume.Value / 100.0f;
                }
            }
            else
            {
                RxWaveOut.Volume = RcvVolume.Value / 100.0f;
            }
        }

        private void AuxMuteBox_CheckedChanged(object sender, EventArgs e)
        {
            if (AuxMuteBox.Checked)
            {
                RxWaveOut0.Volume = 0.0f;
                if (!MainMuteBox.Checked)
                {
                    RxWaveOut.Volume = RcvVolume.Value / 100.0f;
                }
            }
            else
            {
                RxWaveOut0.Volume = RcvVolume.Value / 100.0f;
            }
        }

        private void RcvVolume_Scroll(object sender, EventArgs e)
        {
            try
            {
                if (!AuxMuteBox.Checked)
                {
                    RxWaveOut0.Volume = RcvVolume.Value / 100.0f;
                }
                if (!MainMuteBox.Checked)
                {
                    RxWaveOut.Volume = RcvVolume.Value / 100.0f;
                }
            }
            catch { }
            Properties.Settings.Default.RcvVolume = RcvVolume.Value;
            Properties.Settings.Default.Save();
        }

        public static float monVol;

        private void txMonVol_Scroll(object sender, EventArgs e)
        {
            monVol = txMonVol.Value / 100.0f;
            Properties.Settings.Default.txMonVol = txMonVol.Value;
            Properties.Settings.Default.Save();
            if (TxDigiChkBox.Checked || TxN1MMChkBox.Checked || micMainChk.Checked)
            {
                try
                {
                    micMonOut.Volume = monVol;
                }
                catch { }
            }
            else if (TxDigiChkBoxAux.Checked || TxN1MMChkBoxAux.Checked || micAuxChk.Checked)
            {
                try
                {
                    micMonOut2.Volume = monVol;
                }
                catch { }
            }
        }

        private void digiCATBox_CheckedChanged(object sender, EventArgs e)
        {
            if (digiCATBox.Checked)
            {
                Globals.digiCAT = true;
                digiCATBox.BackColor = SystemColors.Control;
                DigiCAT();
            }
            else
            {
                Globals.digiCAT = false;
                digiCATBox.BackColor = SystemColors.Control;
            }
        }

        StringBuilder hpsdrmodeswitch(int hpsdrmode)
        {
            StringBuilder a = new StringBuilder("");
            switch (hpsdrmode)
            {
                case 0:
                    a = a.Append("5");
                    return a; ;
                case 1:
                    a = a.Append("5");
                    return a; ;
                case 2:
                    a = a.Append("4");
                    return a; ;
                case 3:
                    a = a.Append("2");
                    return a;
                case 4:
                    a = a.Append("1");
                    return a;
                case 5:
                    a = a.Append("3");
                    return a;
                case 6:
                    a = a.Append("7");
                    return a;
                default:
                    a = null;
                    return a;
            }

        }


        public StringBuilder[] DigiCATCalcs(bool MainDigiBox,string MainBand, StringBuilder[] a)
        {
            {
        //        if (MainDigiBox)
                {
                    if (MainBand != null)
                    {
                        //  Console.Writeline("MainBand = {0}", MainBand);
                        switch (MainBand)
                        {
                            case "50":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode50;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("000").Append(Convert.ToString(_50.xvtr_frequency));
                                    a[1] = a[1].Append("000").Append(_50.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split50).ToString());
                                    return a;

                                }


                            case "144":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode144;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_144.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_144.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split144).ToString());
                                    return a;

                                }


                            case "222":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode222;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_222.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_222.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split222).ToString());
                                    return a;

                                }


                            case "432":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode432;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_432.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_432.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split432).ToString());
                                    return a;

                                }


                            case "903":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode903;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_903.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_903.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split903).ToString());
                                    return a;

                                }


                            case "1296":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode1296;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("0").Append(Convert.ToString(_1296.xvtr_frequency));
                                    a[1] = a[1].Append("0").Append(_1296.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split1296).ToString());
                                    return a;

                                }
                            case "GHz":
                                {
                                    int hpsdrmode = Properties.Settings.Default.ModeGHz;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    if (Micro.xvtr_frequency < 10000000000)
                                    {
                                        a[0] = a[0].Append("0").Append(Convert.ToString(Micro.xvtr_frequency));
                                    }
                                    else
                                    {
                                        a[0].Append(Convert.ToString(Micro.xvtr_frequency));
                                    }

                                    if (Micro.xvtr_frequency < 10000000000)
                                    {
                                        a[1] = a[1].Append("0").Append(Micro.split_tx_freq);
                                    }
                                    else
                                    {
                                        a[1] = a[1].Append(Micro.split_tx_freq);
                                    }
                                    a[3] = a[3].Append((Properties.Settings.Default.splitGHz).ToString());
                                    //  Console.Writeline("SPLIT IS {0}", Properties.Settings.Default.splitGHz.ToString());
                                    return a;

                                }


                            case "HF":
                                {
                                    int hpsdrmode = Properties.Settings.Default.ModeHF;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    if (_HF.set_frequency.Value < 10000000)
                                    {
                                        a[0] = a[0].Append("0000").Append(Convert.ToString(_HF.set_frequency.Value));
                                    }
                                    else
                                    {
                                        a[0] = a[0].Append("000").Append(Convert.ToString(_HF.set_frequency.Value));
                                    }
                                    if (long.Parse(_HF.split_tx_freq) < 10000000)
                                    {
                                        a[1] = a[1].Append("0000").Append(_HF.split_tx_freq);
                                    }
                                    else
                                    {
                                        a[1] = a[1].Append("000").Append(_HF.split_tx_freq);
                                    }
                                    a[3] = a[3].Append((Properties.Settings.Default.splitHF).ToString());
                                    return a;

                                }
                            default:
                                { return null; }
                        }
                    }
                }
      //          else return null;
            }
            return null;
        }


        public StringBuilder[] DigiCATCalcsAux(bool AuxDigiBox, string AuxBand, StringBuilder[] a)
        {
            {
     //           if (AuxDigiBox)
                {
                    if (AuxBand != null)
                    {
                        switch (AuxBand)
                        {
                            case "50":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode50;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("000").Append(Convert.ToString(_50.xvtr_frequency));
                                    a[1] = a[1].Append("000").Append(_50.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split50).ToString());
                                    return a;

                                }


                            case "144":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode144;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_144.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_144.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split144).ToString());
                                    return a;

                                }


                            case "222":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode222;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_222.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_222.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split222).ToString());
                                    return a;

                                }


                            case "432":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode432;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_432.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_432.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split432).ToString());
                                    return a;

                                }


                            case "903":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode903;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("00").Append(Convert.ToString(_903.xvtr_frequency));
                                    a[1] = a[1].Append("00").Append(_903.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split903).ToString());
                                    return a;

                                }


                            case "1296":
                                {
                                    int hpsdrmode = Properties.Settings.Default.Mode1296;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    a[0] = a[0].Append("0").Append(Convert.ToString(_1296.xvtr_frequency));
                                    a[1] = a[1].Append("0").Append(_1296.split_tx_freq);
                                    a[3] = a[3].Append((Properties.Settings.Default.split1296).ToString());
                                    return a;

                                }


                            case "GHz":
                                {
                                    int hpsdrmode = Properties.Settings.Default.ModeGHz;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    if (Micro.xvtr_frequency < 10000000000)
                                    {
                                        a[0] = a[0].Append("0").Append(Convert.ToString(Micro.xvtr_frequency));
                                    }
                                    else
                                    {
                                        a[0].Append(Convert.ToString(Micro.xvtr_frequency));
                                    }

                                    if (Micro.xvtr_frequency < 10000000000)
                                    {
                                        a[1] = a[1].Append("0").Append(Micro.split_tx_freq);
                                    }
                                    else
                                    {
                                        a[1] = a[1].Append(Micro.split_tx_freq);
                                    }
                                    a[3] = a[3].Append((Properties.Settings.Default.splitGHz).ToString());
                                    return a;

                                }


                            case "HF":
                                {
                                    int hpsdrmode = Properties.Settings.Default.ModeHF;
                                    a[2] = hpsdrmodeswitch(hpsdrmode);
                                    if (_HF.set_frequency.Value < 10000000)
                                    {
                                        a[0] = a[0].Append("0000").Append(Convert.ToString(_HF.set_frequency.Value));
                                    }
                                    else
                                    {
                                        a[0] = a[0].Append("000").Append(Convert.ToString(_HF.set_frequency.Value));
                                    }
                                    if (long.Parse(_HF.split_tx_freq) < 10000000)
                                    {
                                        a[1] = a[1].Append("0000").Append(_HF.split_tx_freq);
                                    }
                                    else
                                    {
                                        a[1] = a[1].Append("000").Append(_HF.split_tx_freq);
                                    }
                                    a[3] = a[3].Append((Properties.Settings.Default.splitHF).ToString());
                                    return a;

                                }

                            default:
                                { return null; }
                        }
                    }
                }
            //    else return null;
            }
            return null;
        }

        private void digiCATBoxAux_CheckedChanged(object sender, EventArgs e)
        {
            if (digiCATBoxAux.Checked)
            {
                Globals.digiCATAux = true;
                digiCATBoxAux.BackColor = SystemColors.Control;
                DigiCATAux();
            }
            else
            {
                Globals.digiCATAux = false;
                digiCATBoxAux.BackColor = SystemColors.Control;
            }

        }
    }


    public enum OperMode : int
        {
            AM,
            FM,
            SAM,
            USB,
            LSB,
            CWU,
            CWL
        };

        public enum OpBand : int
        {
            M50,
            M144,
            M222,
            M432,
            M903,
            M1296,
            M2304,
            M3456,
            M5760,
            M10368,
            M24192,
            GC,
            XVRT1,
            XVRT2
        };

        public enum KKMethod : int
        {
            USB = 0,
            Ethernet = 1
        };



        class DSPGain : ISampleSource
        {
            ISampleSource _source;
            public DSPGain(ISampleSource source)
            {
                if (source == null)
                    throw new ArgumentNullException("source");
                _source = source;
            }
            public int Read(float[] buffer, int offset, int count)
            {
                float gainAmplification = (float)(Math.Pow(10.0, GainDB / 20.0));
                int samples = _source.Read(buffer, offset, count);
                for (int i = offset; i < offset + samples; i++)
                {
                    buffer[i] = Math.Max(Math.Min(buffer[i] * gainAmplification, 1), -1);
                }
                return samples;
            }

            public float GainDB { get; set; }

            public bool CanSeek
            {
                get { return _source.CanSeek; }
            }

            public CSCore.WaveFormat WaveFormat
            {
                get { return _source.WaveFormat; }
            }

            public long Position
            {
                get
                {
                    return _source.Position;
                }
                set
                {
                    _source.Position = value;
                }
            }

            public long Length
            {
                get { return _source.Length; }
            }

            public void Dispose()
            {
            }
        }

    
}
